/*
 *
 * Copyright  (C) 2005 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file plugins/e2p_rename_ext.h 
 * @brief header for file-rename plugin
 */

#ifndef E2P_RENAMEXT_MAIN_H
# define E2P_RENAMEXT_MAIN_H

#include "emelfm2.h"
#include<stdlib.h>
#include "e2_plugins.h"
#include "e2_dialog.h"

//up to this many regex expressions processed in any pattern
#define E2_CHUNK_LIMIT 32

typedef enum
	{	SEARCH_SUBDIRS_P = 0, CONFIRM_P,
		SEARCH_ALL_P, SEARCH_CURRENT_P,
		SEARCH_OTHER_P, SEARCH_THIS_P,
		SEARCH_WILD_P, SEARCH_REGEX_P,
		MAX_FLAGS	//no. of entries in the array
	} flag_t ;

typedef struct
{
	GtkWidget *dialog;
	GtkWidget *directory; // entry with text for directory to search in/from
	GtkWidget *pattern;  // entry with text for file name to search for
	GtkWidget *newpattern;  // entry with text for file name to substitute
	GtkWidget *stop_button;  // button widget, remebered for changing sensitivity
	GtkWidget *start_button;  // ditto
	GtkWidget *help_button;  // ditto
	GSList *groups;  //list of lists of grouped toggle btns
//	gint find_pid;
	gboolean extended_rgx;	//flag for whether 'find' command can use extended regex
	gboolean parsed;	//flag for whether the replacemment pattern has been converted to chunks
	gboolean abort;  //flag for stop btn pressed
	gboolean flags[MAX_FLAGS];	//cache for toggle values
	gchar *chunks[E2_CHUNK_LIMIT];  //parts of old name pattern which are needed in the new name
} E2_RenDialogRuntime;

#define P_RT ren_rt
E2_RenDialogRuntime *P_RT;

//extern void find (gpointer *find_button, gpointer *pattern);

GtkWidget *e2pr_create_vbox (GtkWidget *container);
GtkWidget *e2pr_create_hbox (GtkWidget *container);
GtkWidget *e2pr_create_hseparator (GtkWidget *container);
GtkWidget *e2pr_create_label (GtkWidget *container, gchar *text);
//GtkWidget *e2pr_create_button (GtkWidget *container, 
//	void  (*callback) (), gpointer callback_data);
GtkWidget *e2pr_create_entry (GtkWidget *container);
GtkWidget *e2pr_create_toggle_grouped_button (GtkWidget *container, 
	flag_t f, gboolean state, gchar *label, GtkWidget *leader,
	E2_RenDialogRuntime *rt);
GtkWidget *e2pr_create_toggle_button (GtkWidget *container, 
	flag_t f, gboolean state, gchar *label, E2_RenDialogRuntime *rt);
GtkWidget *e2pr_create_radio_button (GtkWidget *container, 
	flag_t f, gboolean state, gchar *label, E2_RenDialogRuntime *rt);
GtkWidget *e2pr_create_radio_grouped_button (GtkWidget *container, GtkWidget *group, 
	flag_t f, gboolean state, gchar *label, E2_RenDialogRuntime *rt);

void e2pr_set_flag (flag_t f, gboolean value, E2_RenDialogRuntime *rt);
gboolean e2pr_get_flag (flag_t f, E2_RenDialogRuntime *rt);
void e2pr_reset_flags (E2_RenDialogRuntime *rt);

#endif //ndef E2P_RENAMEXT_MAIN_H
