/* $Id: e2_action.h 762 2005-05-13 04:00:38Z tpgww $
 * 
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file src/actions/e2_action.h 
 * @brief action system header
 *
 * This is the header file for the action system.
 **/

#ifndef E2_ACTION_H
#	define E2_ACTION_H

typedef enum
{
	E2_ACTION_TYPE_ITEM,
	E2_ACTION_TYPE_SUBMENU,
	E2_ACTION_TYPE_FILE_ACTIONS,
	E2_ACTION_TYPE_CUSTOM_COMMAND,
	E2_ACTION_TYPE_PLUGINS,
	E2_ACTION_TYPE_SEPARATOR,
	E2_ACTION_TYPE_TEAR_OFF_MENU,
	E2_ACTION_TYPE_TOGGLE,
	E2_ACTION_TYPE_COMMAND_LINE,
	E2_ACTION_TYPE_BOOKMARKS,
	E2_ACTION_TYPE_CHECK_ITEM,  //UNUSED
	E2_ACTION_TYPE_FILTERS,	//UNUSED
	E2_ACTION_TYPE_DUMMY
} E2_ACTION_TYPE;

typedef enum
{
	E2_ACTION_EXCLUDE_TOOLBAR = 1 << 0,
	E2_ACTION_EXCLUDE_MENU = 1 << 1,
	E2_ACTION_EXCLUDE_LAYOUT = 1 << 2,	//not yet used
	E2_ACTION_EXCLUDE_ACCEL = 1 << 3,
} E2_ACTION_EXCLUDE;

typedef enum
{
	E2_TOGGLE_NONE,
	E2_TOGGLE_INIT,
	E2_TOGGLE_DESTROYED
} E2_TOGGLE_STATUS;

typedef struct _E2_Action
{
	gchar *name;
	E2_ACTION_TYPE type;
	gpointer func;
	gboolean has_arg;
	E2_ACTION_EXCLUDE exclude;
	gpointer data;
	gpointer data2;  //for toggle actions, this pts to a E2_ToggleActionRuntime
	GtkTreeRowReference *ref;
} E2_Action;

typedef struct _E2_ActionRuntime
{
	E2_Action *action;
	gpointer data;
	gpointer data_free;
} E2_ActionRuntime;

typedef struct _E2_ToggleAction
{  // this is 'extended' data for toggle actions
	gpointer bar_rt;  //ptr to bar that includes the toggle FIXME support > 1 bar
	E2_TOGGLE_STATUS state;  //signal for how to handle (re)builds 
	gboolean current_state;
	GtkWidget *true_button;
	GtkWidget *false_button;
	gpointer toggle_data;
	gpointer toggle_data_free; //unused
} E2_ToggleActionEx;

GPtrArray *toggles_array;

typedef gchar *_action_labels[95];	//make sure this is big enough for all labels
_action_labels action_labels;
#define _A(d) action_labels[d]

GtkTreeModel *e2_actions_fill_store (gpointer data);
E2_Action *e2_action_register_simple (gchar *name, E2_ACTION_TYPE type,
	void *func, gpointer data, gboolean has_arg);
E2_Action *e2_action_register (gchar *name, E2_ACTION_TYPE type,
	void *func, gpointer data, gboolean has_arg,
	E2_ACTION_EXCLUDE exclude, gpointer data2);
gboolean e2_action_unregister (const gchar *name);
E2_Action *e2_action_get (gchar *name);
gboolean e2_action_run_simple (gchar *name, gpointer arg);
gboolean e2_action_run_simple_from (gchar *name, gpointer arg,
	GtkWidget *from);
gboolean e2_action_run (GtkWidget *widget, E2_ActionRuntime *rt);
gboolean e2_action_run_from (E2_ActionRuntime *rt, GtkWidget *from);
E2_ActionRuntime *e2_action_pack_runtime (E2_Action *action, gpointer data,
	gpointer data_free);
void e2_action_free_runtime (E2_ActionRuntime *rt);

//void e2_actions_clean ();
//initialization things
void e2_action_setup_labels(void);
void e2_actions_init (void);

#endif //ndef E2_ACTION_H
