/*
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_dialog.h"
#include "e2_filelist.h"

static GtkWidget *dialog;	//the created dialog
static GtkWidget *pattern_entry;
static GtkWidget *invert_check;
static GtkWidget *case_sensitive_check;
static GtkWidget *menu_item;	//copy of pointer to clicked filters menu item
static gboolean itemstate;	//initial state of menu_item
//static gboolean blocked = FALSE;	//flag to prevent toggles when not wanted

/*static void _e2_name_filter_dialog_item_toggle_cb (void)
{
  gulong handler = g_signal_handler_find (menu_item, G_SIGNAL_MATCH_FUNC, 
	0, 0, NULL, e2_name_filter_dialog_create_cb, NULL);
  g_signal_handler_block (menu_item, handler);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !itemstate);
  g_signal_handler_unblock (menu_item, handler);
}

static gboolean _e2_name_filter_dialog_destroy_cb (GtkWidget *widget)
{
	if (blocked)
	  blocked = FALSE;
  else
	  _e2_name_filter_dialog_item_toggle_cb ();
    return FALSE;
}

static gboolean _e2_name_filter_dialog_key_press_cb 
	(GtkWidget *widget, GdkEventKey *event)
{
  if (event->keyval == GDK_Escape)
    _e2_name_filter_dialog_destroy_cb (widget);
  return FALSE;
}
*/
static void _e2_name_filter_dialog_ok (FileView *view)
{
  const gchar *s = gtk_entry_get_text (GTK_ENTRY (pattern_entry));
  if (strlen (s) == 0)
  {
    status_message (_("Invalid filename pattern\n"));
  }
  else
  {
	//filter pattern is saved as utf, as required for matching, later
	g_strlcpy (view->filename_filter.pattern, s,
		sizeof (view->filename_filter.pattern));
	view->filename_filter.invert_mask = GTK_TOGGLE_BUTTON (invert_check)->active;
	view->filename_filter.case_sensitive = GTK_TOGGLE_BUTTON (case_sensitive_check)->active;
//    if (! itemstate)
	 //we're staying filtered, so re-toggle the menu item
//	  _e2_name_filter_dialog_item_toggle_cb ();
    //and conform the cached copy
	view->filename_filter.active = TRUE;
	//show the results
	e2_filelist_filter_toggle (view);
    e2_filelist_refresh_list (view);
  }
}

static void _e2_name_filter_dialog_response_cb (GtkWidget *button,
	gint response, FileView *view)
{
	switch (response)
	{
		case GTK_RESPONSE_OK:
			_e2_name_filter_dialog_ok (view);
		  	//prevent toggle when dialog is destroyed
//			blocked = TRUE;
		  break;
		case E2_RESPONSE_REMOVE:
		  //update the cached copy of the state
		  view->filename_filter.active = FALSE;
		  //show the results
		  e2_filelist_filter_toggle (view);
		  e2_filelist_refresh_list (view);
		  //prevent toggle when dialog is destroyed
//		  blocked = TRUE;
		  break;
//		case GTK_RESPONSE_CANCEL:
		default:
		  //the menu item will be re-toggled when dialog is destroyed
		break;
	}
	gtk_widget_destroy (dialog);	
	gtk_main_quit ();
	return;
}
/**
 * @brief create file name filter dialog
 *
 * The state of @a item when it arrives here is opposite to that
 * shown in the menu, when clicked
 *
 * @param item the clicked checkbox widget in the fillters menu
 * @param view data structure for the view to which the file list belongs
 *
 * @return
 */
void e2_name_filter_dialog_create_cb (GtkWidget *item, FileView *view)
{
//  GtkWidget *dialog; use static instead
  GtkWidget *vbox, *table;

  //save local copies, for later use in other functions
  menu_item = item;
  itemstate = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM (item));

  dialog = e2_dialog_create (NULL, _("Display only the items named like:"),
		_("name filter"), _e2_name_filter_dialog_response_cb, view);
  vbox = GTK_DIALOG (dialog)->vbox;

  pattern_entry = add_entry (vbox, view->filename_filter.pattern,
	FALSE, 0);

  e2_widget_add_label (vbox, _("example: *.c,*.h"), 0.0, FALSE, 0);

  table = add_table (vbox, 1, 2, FALSE, FALSE, 0);
  invert_check = e2_button_add_toggle_to_table (table, _("Invert"),
	view->filename_filter.invert_mask, NULL, NULL, 0, 1, 0, 1);
  case_sensitive_check = e2_button_add_toggle_to_table (table, _("Case sensitive"),
	view->filename_filter.case_sensitive, NULL, NULL, 1, 2, 0, 1);
  gtk_tooltips_set_tip (app.tooltips, invert_check,
	_("Show files that DO NOT match the given mask"), NULL);

  //we also need to intercept these
/*  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2_name_filter_dialog_key_press_cb), NULL);
  g_signal_connect (G_OBJECT (dialog), "destroy",
	G_CALLBACK (_e2_name_filter_dialog_destroy_cb), NULL);
*/
  e2_dialog_setup (dialog, app.main_window);
  //now the buttons
  if (!itemstate)
	  //user has just toggled off
	  e2_dialog_add_defined_button (dialog, &E2_BUTTON_REMOVE);
  	//non-modal dialog
	e2_dialog_show (dialog, app.main_window,
		E2_DIALOG_FREE,  //no  _RUN (hence window stays sensitive)
		&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
	gtk_main ();
}
