/*
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include "e2_dialog.h"
#include "e2_filelist.h"

static GtkWidget *dialog;	//the created dialog
static GtkWidget *operation_combo;
static GtkWidget *size_entry;
static GtkWidget *units_combo;
static GtkWidget *menu_item;	//copy of pointer to clicked filters menu item
static gboolean itemstate;	//initial state of menu_item
//static gboolean blocked = FALSE;	//flag to prevent toggles when not wanted

/*static void _e2_size_filter_dialog_item_toggle_cb (void)
{
  gulong handler = g_signal_handler_find (menu_item, G_SIGNAL_MATCH_FUNC, 
	0, 0, NULL, e2_size_filter_dialog_create_cb, NULL);
  g_signal_handler_block (menu_item, handler);
  gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), !itemstate);
  g_signal_handler_unblock (menu_item, handler);
}

static gboolean _e2_size_filter_dialog_destroy_cb (GtkWidget *widget)
{
	if (blocked)
	  blocked = FALSE;
  else
	  _e2_size_filter_dialog_item_toggle_cb ();
    return FALSE;
}

static gboolean _e2_size_filter_dialog_key_press_cb 
	(GtkWidget *widget, GdkEventKey *event)
{
  if (event->keyval == GDK_Escape)
    _e2_size_filter_dialog_destroy_cb (widget);
  return FALSE;
}
*/
static void _e2_size_filter_dialog_ok (FileView *view)
{
  gint index = gtk_combo_box_get_active (GTK_COMBO_BOX (operation_combo));
  if (index == 1) //_("bigger than")
	view->size_filter.op = GT;
  else
	view->size_filter.op = LT;

  gchar *s = (gchar *) gtk_entry_get_text (GTK_ENTRY (size_entry));
  gdouble size = atof (s);

  index = gtk_combo_box_get_active (GTK_COMBO_BOX (units_combo));
  switch (index)
  {
	case 0: // _("bytes")
		view->size_filter.size = (off_t)size;
		break;
	case 1: // _("kbytes")
		view->size_filter.size = (off_t)(size * (1 << 10));
		break;
	case 2:  // _("Mytes")
		view->size_filter.size = (off_t)(size * (1 << 20));
	    break;
	default:
		view->size_filter.size = (off_t)size;
		break;
	}
//	if (! itemstate)
	 //we're staying filtered, so re-toggle the menu item
//	  _e2_size_filter_dialog_item_toggle_cb ();
    //and conform the cached copy
	view->size_filter.active = TRUE;
	//show the results
	e2_filelist_filter_toggle (view);
    e2_filelist_refresh_list (view);
}

static void _e2_size_filter_dialog_response_cb (GtkWidget *button,
	gint response, FileView *view)
{
	switch (response)
	{
		case GTK_RESPONSE_OK:
			_e2_size_filter_dialog_ok (view);
		  	//prevent toggle when dialog is destroyed
//			blocked = TRUE;
		  break;
		case E2_RESPONSE_REMOVE:
		  //update the cached copy of the state
		  view->size_filter.active = FALSE;
		  //show the results
		  e2_filelist_filter_toggle (view);
		  e2_filelist_refresh_list (view);
		  //prevent toggle when dialog is destroyed
//		  blocked = TRUE;
		  break;
//		case GTK_RESPONSE_CANCEL:
		default:
		  //the menu item will be re-toggled when dialog is destroyed
		break;
	}
	gtk_widget_destroy (dialog);	
	gtk_main_quit ();
	return;
}
/**
 * @brief create file size filter dialog
 *
 * The state of @a item when it arrives here is opposite to that
 * shown in the menu, when clicked
 *
 * @param item the clicked checkbox widget in the fillters menu
 * @param view data structure for the view to which the file list belongs
 *
 * @return
 */
void e2_size_filter_dialog_create_cb (GtkWidget *item, FileView *view)
{
//  GtkWidget *dialog; use static instead
  GtkWidget *hbox;
  gchar size_string[32];

  //save local copies, for later use in other functions
  menu_item = item;
  itemstate = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM (item));

  dialog = e2_dialog_create (NULL, _("Display only the items which are:"),
		_("size filter"), _e2_size_filter_dialog_response_cb, view);
  hbox = add_hbox (GTK_DIALOG (dialog)->vbox, FALSE, E2_PADDING, TRUE, E2_PADDING);

  operation_combo = gtk_combo_box_new_text ();
  //don't change the order of these - the index is used
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("smaller than"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(operation_combo), _("bigger than"));
  gint index = ( view->size_filter.op == LT)  ? 0 : 1 ;
  gtk_combo_box_set_active (GTK_COMBO_BOX (operation_combo), index) ;	//0=1st entry index
//	gtk_widget_set_size_request (operation_combo, 150, 30);
  gtk_box_pack_start (GTK_BOX (hbox), operation_combo, FALSE, FALSE, 0);
  gtk_widget_show (operation_combo);

  if (view->size_filter.size < (1 << 10))
  {
	g_snprintf (size_string, sizeof (size_string), "%ld",
		(glong) view->size_filter.size);
	index = 0; 	//0=1st combo entry index
  }
  else if (view->size_filter.size < (1 << 20))
  {
	g_snprintf (size_string, sizeof (size_string), "%.2f", 
		(gdouble)((gdouble)view->size_filter.size / (gdouble)(1 << 10)));
	index = 1;
  }
  else
  {
	g_snprintf (size_string, sizeof (size_string), "%.2f",
		(gdouble)((gdouble)view->size_filter.size / (gdouble)(1 << 20)));
	index = 2;
  }

//  size_entry = add_entry (hbox, "", FALSE, 0); this pair add the text without selection
// gtk_entry_set_text (GTK_ENTRY (size_entry), size_string);
  size_entry = add_entry (hbox, size_string, FALSE, 0);
  gtk_widget_set_size_request (size_entry, 100, 30);

  units_combo = gtk_combo_box_new_text ();
  gtk_combo_box_append_text (GTK_COMBO_BOX(units_combo),_("bytes"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(units_combo),_("kbytes"));
  gtk_combo_box_append_text (GTK_COMBO_BOX(units_combo),_("Mbytes"));
  gtk_combo_box_set_active (GTK_COMBO_BOX (units_combo), index) ;
//	gtk_widget_set_size_request (units_combo, 100, 30);
  gtk_box_pack_start (GTK_BOX (hbox), units_combo, FALSE, FALSE, 0);
  gtk_widget_show (units_combo);

  //we also need to intercept these
/*  g_signal_connect (G_OBJECT (dialog), "key_press_event",
	G_CALLBACK (_e2_size_filter_dialog_key_press_cb), NULL);
  g_signal_connect (G_OBJECT (dialog), "destroy",
	G_CALLBACK (_e2_size_filter_dialog_destroy_cb), NULL);
*/
  e2_dialog_setup (dialog, app.main_window);
  //now the buttons
  if (!itemstate)
	  //user has just toggled off
	  e2_dialog_add_defined_button (dialog, &E2_BUTTON_REMOVE);
  	//non-modal dialog
	e2_dialog_show (dialog, app.main_window,
		E2_DIALOG_FREE,  //no  _RUN (hence window stays sensitive)
		&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
	gtk_main ();
}
