/* $Id: e2_view_dialog.c 1053 2005-08-16 11:16:45Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_dialog.h"
#include "e2_search_dialog.h"
#include "e2_view_dialog.h"

static E2_OptionSet *opt_view_width;
static E2_OptionSet *opt_view_height;
static E2_OptionSet *opt_view_wrap;
static E2_ViewDialogRuntime *rt;

static void _e2_view_dialog_destroy (E2_ViewDialogRuntime *rt);
static gboolean _e2_view_dialog_refresh (GtkWidget *textview, gchar *filename);
//static GtkWidget *_e2_view_dialog_file_create (gchar *filename);

  /*********************/
 /***** callbacks *****/
/*********************/

static void _e2_view_dialog_response_cb (GtkWidget *dialog, gint response, E2_ViewDialogRuntime *rt)
{
	switch (response)
	{
		case E2_RESPONSE_USER1:  //wrap
			rt->wrap = !rt->wrap;
			gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
				rt->wrap ? GTK_WRAP_WORD: GTK_WRAP_NONE);
			break;
		case E2_RESPONSE_FIND:
			e2_search_dialog_show (rt->textview, NULL, dialog);
			break;
		case E2_RESPONSE_REFRESH:
			_e2_view_dialog_refresh (rt->textview, rt->filename);
			break;
		default:
			_e2_view_dialog_destroy (rt);
			return;
			break;
	}
	gtk_widget_grab_focus (GTK_WIDGET (rt->textview));
}

  /*****************/
 /***** utils *****/
/*****************/

static void _e2_view_dialog_destroy (E2_ViewDialogRuntime *rt)
{
	gtk_widget_destroy (rt->dialog);
	g_free (rt->filename);
	if (rt->idle_id != 0)
		g_source_remove (rt->idle_id);
	g_free (rt);
}

static gboolean _e2_view_dialog_refresh (GtkWidget *textview, gchar *filepath)
{
	gchar *cont;
	if (! e2_fs_get_file_contents (filepath, &cont))
	{
		printd (WARN, "could not open file '%s' for reading", filepath);
		gchar *msg = g_strdup_printf (_("Could not open file '%s' for reading"), filepath);
		status_error (msg);
		g_free (msg);
		return FALSE;
	}
	GtkTextBuffer *buffer = rt->textbuffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview));

	if (!g_utf8_validate (cont, -1, NULL))
	{
		printd (DEBUG, "trying to convert '%s' to UTF-8", filepath);
		GError *utf_error = NULL;
		gchar *utf = g_locale_to_utf8 (cont, -1, NULL, NULL, &utf_error);
		if (utf != NULL)
			gtk_text_buffer_set_text (buffer, utf, -1);
		else
		{
			gtk_text_buffer_set_text (buffer, utf_error->message, -1);
			g_error_free (utf_error);
		}
		g_free (utf);
	}
	else
	{
		printd (DEBUG, "'%s' is UTF-8 encoded", filepath);
		gtk_text_buffer_set_text (buffer, cont, -1);
	}
	g_free (cont);
	//set cursor to start of buffer, for searching etc
	GtkTextIter start;
	gtk_text_buffer_get_start_iter (buffer, &start);
	gtk_text_buffer_place_cursor (buffer, &start);

	return TRUE;
}

static GtkWidget *_e2_view_dialog_file_create (gchar *filename)
{
	gchar *file;
	if (!g_path_is_absolute (filename))
		file = g_build_filename (curr_view->dir, filename, NULL);
	else
		file = g_strdup (filename);
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (file);
	if (e2_fs_access (local, F_OK))
	{
		gchar *msg = g_strdup_printf (_("Cannot open '%s', because it does not exist"), file);
		status_error (msg);
		g_free (msg);
		g_free (file);
		F_FREE (local);
		return NULL;
	}
	if (e2_fs_is_dir3 (local))
	{
		gchar *msg = g_strdup_printf (_("Cannot view directory '%s' in this file viewer"), file);
		status_error (msg);
		g_free (msg);
		g_free (file);
		F_FREE (local);
		return NULL;
	}
	F_FREE (local);
	//init runtime object
//	E2_ViewDialogRuntime * use the global pointer
	rt = g_malloc (sizeof (E2_ViewDialogRuntime));
	rt->filename = file;
	rt->wrap = _e2_option_bool_get (opt_view_wrap);
	rt->window_width = _e2_option_int_get (opt_view_width);
	rt->window_height = _e2_option_int_get (opt_view_height);
	rt->idle_id = 0;

	rt->dialog = e2_dialog_create (NULL, file, _("displaying file"),
		_e2_view_dialog_response_cb, rt);
	//create the view
	GtkWidget *sw = e2_widget_add_sw (GTK_DIALOG (rt->dialog)->vbox,
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, TRUE, E2_PADDING);
	rt->textview = gtk_text_view_new ();
	gtk_container_add (GTK_CONTAINER (sw), rt->textview);
	// set view defaults
	gtk_text_view_set_editable (GTK_TEXT_VIEW (rt->textview), FALSE);
	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (rt->textview), FALSE);
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (rt->textview),
				rt->wrap ? GTK_WRAP_WORD : GTK_WRAP_NONE);
	gint char_width, char_height;
	e2_widget_set_font (rt->textview, e2_option_str_get ("output-font"),
				 &char_width, &char_height);
	// add the text
	_e2_view_dialog_refresh (rt->textview, file);
	// add the buttons
	GtkWidget *wrap_button = _e2_dialog_add_check_button (rt->dialog, TRUE, _("_Wrap"), E2_RESPONSE_USER1);
	g_object_set_data (G_OBJECT (rt->dialog), "wrap", wrap_button);
	e2_dialog_add_undefined_button (rt->dialog, GTK_STOCK_FIND, _("_Find"), E2_RESPONSE_FIND);
	//standard buttons added when dialog shown (see below)
/*	e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_REFRESH);
	e2_dialog_add_defined_button (rt->dialog, &E2_BUTTON_CLOSE); */

//	GtkWidget *menu = e2_option_create_menu (NULL,
//		opt_show_last, toggled_cb1, rt,
//		opt_case_sensitive, toggled_cb2, rt,
//		opt_view_wrap, view_wrap_cb, rt,
//		NULL);
//	e2_menu_connect (menu,FALSE);
//	e2_dialog_attach_menu (rt->dialog, menu);

	// display it
	gtk_window_resize (GTK_WINDOW (rt->dialog), char_width * rt->window_width,
				char_height *rt->window_height);
	return rt->dialog;
}
  /*******************/
 /***** actions *****/
/******************/
/**
 * @brief open file for viewing, at a location where there is a specified string
 * @param filename NULL always
 * @param arg string with filepath and an argument that is a text string to find in the file
 * @return
 **/
void e2_view_dialog_openat (gchar *filename, gchar *arg)
{
	gchar *locator = strchr (arg, ' ');	//always ascii ' ', don't need g_utf8_strchr()
	if (locator == NULL)
		return;	//error msg ??
	*locator = '\0';
	while (*(++locator) == ' ') {}	//FIXME utf, not ascii

	GtkWidget *dialog = _e2_view_dialog_file_create (arg);
	if (dialog == NULL)
		return;	//error msg ??
	//find the locator and open there
	GtkTextIter iter, start, end;
	gtk_text_buffer_get_start_iter (rt->textbuffer, &iter);
	if (gtk_text_iter_forward_search (&iter, locator, 0, &start, &end, NULL))
	{	//need to add a mark to get the scroll to work properly
		GtkTextMark *mark = gtk_text_buffer_create_mark
			(rt->textbuffer, NULL, &start, FALSE);
		gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW (rt->textview), mark,
			0.0, TRUE, 0.0, 0.1);
		//to support incremental searching, move the cursor
		//to the start of the located string
		gtk_text_buffer_place_cursor (rt->textbuffer, &start);
	}
	e2_dialog_show (dialog, app.main_window, 0, &E2_BUTTON_CLOSE, NULL);
	gtk_window_present (GTK_WINDOW (dialog));
}
  /******************/
 /***** public *****/
/******************/

void e2_view_dialog_show (gchar *filename)
{
	GtkWidget *dialog = _e2_view_dialog_file_create (filename);
	e2_dialog_show (dialog, app.main_window, 0, &E2_BUTTON_REFRESH,
		&E2_BUTTON_CLOSE, NULL);
	gtk_window_present (GTK_WINDOW (dialog));
}

void e2_view_dialog_actions_register (void)
{
	gchar *action_name = g_strconcat(_A(3),".",_A(75),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		e2_view_dialog_show, NULL, FALSE);
	action_name = g_strconcat(_A(5),".",_A(76),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		e2_view_dialog_openat, NULL, TRUE);
}

void e2_view_dialog_options_register (void)
{
//CHECKME = strdup's for group, to support cleanups
	gchar *group_name = g_strconcat(_C(12),".",_C(38),":",_C(23),NULL); //_("dialogs.view:misc"
	opt_view_wrap = e2_option_bool_register ("dialog-view-wrap",
		group_name, _("wrap text"),
		_("This causes the view window to open with text-wrapping enabled"),
		NULL, TRUE,
		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP);
	opt_view_width = e2_option_int_register ("dialog-view-width",
		group_name, _("window width"),
		_("The view window will default to showing this many characters per line (but the the displayed buttons may make it wider than this)")
		, NULL, 84, 20, 1000,
		E2_OPTION_FLAG_ADVANCED);
	opt_view_height = e2_option_int_register ("dialog-view-height",
		group_name, _("window height"),
		_("The view window will default to showing this many lines of text"), NULL, 30, 10, 1000,
		E2_OPTION_FLAG_ADVANCED);
}
