/* $Id: e2_context_menu.c 1049 2005-08-16 11:13:34Z tpgww $
 * 
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * 09-20-2000: Modified by Aurelien Gateau
 *             Added code for Named actions for filetypes
 */

/**
 * @file src/e2_context_menu.c
 * @brief file-pane context menu functions
 *
 * Some of the functions are used in other contexts eg toolbar menus
 * Other context menus are handled in the same file as their 'context'.
 *
**/

#include "e2_context_menu.h"
#include <string.h>
#include "e2_filelist.h"
#include "e2_filetype.h"

// File-pane Context Menu Callbacks
static void _e2_context_menu_choose_filetype_action_cb (GtkWidget *widget)
{
	printd (DEBUG, "context menu choose filetype action cb");
	e2_filetype_exec_action ((gchar *) gtk_widget_get_name (widget));
}

static GtkWidget *_e2_context_menu_create_filetype_actions_menu
	(GtkWidget *menu, gchar **actions)
{
	gchar *sep;
	GtkWidget *menu_item;

	while (*actions != NULL)
	{
		if ((sep = strchr (*actions, '@')) != NULL)  //if always ascii @, don't need g_utf8_strchr()
		{
			*sep = '\0';
			menu_item = e2_menu_add (menu, *actions, NULL, NULL,
				_e2_context_menu_choose_filetype_action_cb, NULL);
			*sep = '@';	//revert to original form (this is the 'source' data)
			gtk_widget_set_name (GTK_WIDGET(menu_item), sep+1);
		}
		else
		{
			menu_item = e2_menu_add (menu, *actions, NULL, NULL,
				_e2_context_menu_choose_filetype_action_cb, NULL);
			gtk_widget_set_name (GTK_WIDGET(menu_item), *actions);
		}
		actions++;
	}

	return menu;
}

static void _e2_context_menu_add_filetype_item (GtkWidget *menu, FileInfo *info)
{
	gchar *ext, *ext2;
	gboolean freeext = TRUE;
	gchar **actions;

	if (info == NULL)
		return;
	if (e2_fs_is_dir (info))
		ext = g_strconcat (".", _("<directory>"), NULL);
	else if (e2_fs_is_executable (info))
		ext = g_strconcat (".", _("<executable>"), NULL);
	else
	{
		freeext = FALSE;
		//assumes extension is that part of the name after the leftmost '.'
		ext = strchr (info->filename, '.');	//localised text, but assumes . is ascii
		if (ext != NULL)
		{
			ext2 = ext;
			ext = S_FILENAME_FROM_LOCALE (ext);	//get utf
			if (ext != ext2)	//conversion actually happened
				freeext = TRUE;	//so free ext before exit
		}
	}

	if (ext != NULL)
	{
		ext2 = ext;	//remember what to free, if need be
		//check all possible extensions for a matching filetype
		do
		{
			//skip leading dot "."
			ext++;	//NCHR(ext); ascii '.'. always single char
			actions = e2_filetype_get_actions (ext);
			if (actions != NULL)
			{
				_e2_context_menu_create_filetype_actions_menu (menu, actions);
				if (freeext)
					g_free (ext2);
				return;
			}
		} while ((ext = strchr (ext, '.')) != NULL);	//always ascii '.', don't need g_utf8_strchr()

		if (freeext)
			g_free (ext2);
	}
}

/*
static gboolean _e2_context_menu_item_click_cb (GtkWidget *widget, GdkEventButton *event, E2_ActionRuntime *rt)
{
	printd (DEBUG, "item_click_cb (widget:_,event:_,rt:_)");
	if (event->button == 1)
	{
		e2_action_run (widget, rt);
		return TRUE;
	}
	return FALSE;
}
*/
//info is only used for filetype menu items
static void _e2_context_menu_add_item (GtkWidget *menu, GtkTreeModel *model,
	GtkTreeIter *iter, gint level, FileInfo *info, gint mtype)
{
	GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (curr_view->treeview));
	gboolean tagged = (gtk_tree_selection_count_selected_rows (sel) > 0);
	do
	{
		gchar *icon, *str_data, *type, *arg;
		gboolean _mtype;
		switch (mtype)
		{
			case 1:
				// shift menu
				gtk_tree_model_get (model, iter, 0, &str_data, 1, &icon, 2, &_mtype, 4, &type, 5, &arg, -1);
				break;
			case 2:
				// ctrl menu
				gtk_tree_model_get (model, iter, 0, &str_data, 1, &icon, 3, &_mtype, 4, &type, 5, &arg, -1);
				break;
			default:
				gtk_tree_model_get (model, iter, 0, &str_data, 1, &icon, 4, &type, 5, &arg, -1);
				break;
		}
		if ((mtype != 0) && !_mtype)   //CHECKME should be _mtype
		{
			GtkTreeIter iter2;
			if (gtk_tree_model_iter_children (model, &iter2, iter))
				//recurse
				_e2_context_menu_add_item (menu, model, &iter2, level, info, mtype);
		}
		else
		{
			E2_Action *action = e2_action_get (type);
			if (action != NULL) 
			switch (action->type)
			{
				case E2_ACTION_TYPE_SUBMENU:
				{	//FIXME = don't show actions sub-menu if no file is selected
					//can't test by str_data (=menu name), it can be changed at runtime
					GtkWidget *item = e2_menu_add (menu, str_data, icon, NULL, NULL, NULL);
					GtkWidget *menu2 = gtk_menu_new ();
					GtkTreeIter iter2;
					if (gtk_tree_model_iter_children (model, &iter2, iter))
						//re-enter this fn
						_e2_context_menu_add_item (menu2, model, &iter2, level + 1, info, mtype);
					gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu2);
					break;
				}
				case E2_ACTION_TYPE_BOOKMARKS:
				{
					E2_OptionSet *set = e2_option_get ("bookmarks");
					GtkTreeIter iter;
					if ((arg != NULL) && (*arg != '\0'))
					{
						if (e2_tree_find_iter_from_str (set->ex.tree.model, 0, arg, &iter, TRUE))
						{
							GtkTreeIter iter2;
							if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
								e2_menu_add_bookmarks_submenu (action, set, menu, &iter2);
						}
					} else if (gtk_tree_model_get_iter_first (set->ex.tree.model, &iter))
							e2_menu_add_bookmarks_submenu (action, set, menu, &iter);
				}
					break;
				case E2_ACTION_TYPE_FILE_ACTIONS:
					if (tagged)
						_e2_context_menu_add_filetype_item (menu, info);
					break;
				case E2_ACTION_TYPE_PLUGINS:
					e2_menu_create_plugins_menu (menu);
					break; 
				case E2_ACTION_TYPE_SEPARATOR:
					e2_menu_add_separator (menu);
					break;
				case E2_ACTION_TYPE_TEAR_OFF_MENU:
					e2_menu_add_tear_off (menu);
					break;
				case E2_ACTION_TYPE_TOGGLE:
				case E2_ACTION_TYPE_ITEM:
					{
						E2_ActionRuntime *actionrt = e2_action_pack_runtime (action, arg, g_free);
						GtkWidget *item;
						if (action->type == E2_ACTION_TYPE_TOGGLE)
							item = e2_menu_add_check (menu, str_data, e2_option_bool_get
								(action->data2), e2_action_run, actionrt);
						else
							item = e2_menu_add (menu, str_data, icon, NULL, e2_action_run,
								actionrt);
//	WHAT IS THIS FOR ?
//						g_object_set_data_full (G_OBJECT (item), "free-callback-data", actionrt,
//							(GDestroyNotify) e2_action_free_runtime);
					}
					break;
				default:
					break;
			} 
		}
		//cleanup
		g_free (icon);
		g_free (str_data);
		g_free (type);
//		g_free (arg);
	} while ((gtk_tree_model_iter_next (model, iter)) == TRUE);
}

static void _e2_context_menu_destroy_menu_cb (GtkWidget *menu, gpointer data)
{
	printd (DEBUG, "destroy_menu_cb");
	//no enable refresh, here as it is done somewhere else!!
//	printd (DEBUG, "destroy_menu_cb, refresh ref = %d", refresh_ref);
	gtk_container_foreach (GTK_CONTAINER (menu), (GtkCallback) gtk_widget_destroy, NULL);
	gtk_widget_destroy (menu);
//#ifdef E2_REFRESH_DEBUG
//	printd (DEBUG, "enable refresh, destroy_menu_cb");
//#endif
//	e2_filelist_enable_refresh();
	app.context_menu = NULL;
}

  /******************/
 /***** public *****/
/******************/

void e2_context_menu_show (guint button, guint32 time, gint type)
{
	FileInfo *info;
//FIXME if we need to disable refresh dduring the poppup,
// make all menu choices depart via a fn that enables refresh again
//#ifdef E2_REFRESH_DEBUG
//	printd (DEBUG, "disable refresh, context menu show");
//#endif
//	e2_filelist_disable_refresh();
	
//	if (curr_view->tagged != NULL)
//		info = curr_view->tagged->data;
//	else
//	{
		GtkTreeIter iter;
		if (gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (curr_view->store),
			&iter, NULL, curr_view->row))
			gtk_tree_model_get (GTK_TREE_MODEL (curr_view->store), &iter, FINFO, &info, -1);
		else
		{
			printd (WARN, "missing model row");
			return;
		}
//	}
	E2_OptionSet *set = e2_option_get ("context-menu");
	app.context_menu = gtk_menu_new ();
//	GtkTreeIter iter;
	if ((gtk_tree_model_get_iter_first (set->ex.tree.model, &iter)) == TRUE)
		_e2_context_menu_add_item (app.context_menu, set->ex.tree.model, &iter, 0, info, type);

	g_signal_connect (G_OBJECT (app.context_menu), "selection-done",
		G_CALLBACK (_e2_context_menu_destroy_menu_cb), NULL);
	if (button == 0)
		//this was a menu key press
		gtk_menu_popup (GTK_MENU (app.context_menu), NULL, NULL,
		(GtkMenuPositionFunc) e2_fileview_set_menu_position,
		curr_view->treeview, 0, time);
	else
		//this was a button-3 click
		gtk_menu_popup (GTK_MENU (app.context_menu), NULL, NULL,
		NULL, NULL, button, time);
}

void e2_context_menu_show_menu_action (gpointer data, gchar *arg, GtkWidget *origin)
{
	g_strstrip (arg);
//	gchar *str_down = g_utf8_strdown (arg, -1);
	if (g_str_equal (arg, _A(92)))	//_(shift
		e2_context_menu_show (3, 0, 1);
	else if (g_str_equal (arg, _A(86)))	//_(ctrl
		e2_context_menu_show (3, 0, 2);
	else
		e2_context_menu_show (3, 0, 0);
//	g_free (str_down);
}

void e2_context_menu_options_register (void)
{
	//this option does not require a rebuild after config change
	E2_OptionSet *set = e2_option_tree_register ("context-menu", _C(10), _C(10),  //_("context menu"
		NULL, NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_tree_add_column (set, _("Label"), E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Icon"), E2_OPTION_TREE_TYPE_ICON, 0, "",
		NULL, NULL, 0, NULL, NULL);
	//"false" is public name, but is widely used internally, not translated
	e2_option_tree_add_column (set, _("Shift"), E2_OPTION_TREE_TYPE_BOOL, FALSE, "false",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Ctrl"), E2_OPTION_TREE_TYPE_BOOL, FALSE,
		"false", NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Action"), E2_OPTION_TREE_TYPE_SEL, 0, _A(17),
		e2_actions_fill_store, GINT_TO_POINTER (E2_ACTION_EXCLUDE_MENU), 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Argument"), E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_sync_columns (set);

	_e2_option_tree_prepare_default (set,
		g_strdup("<context-menu"),  //internal name
		g_strconcat("||false|false|",_A(5),".",_A(20),"|",NULL),
		g_strconcat(_("Open _with.."),"|"ICON_DIR"open_with_"E2IP".png|false|false|",
			_A(5),".",_A(53),"|",NULL),
		g_strconcat("||false|false|",_A(17),"|",NULL),
		g_strconcat(_("_View"),"|"ICON_DIR"view_"E2IP".png|false|false|",
			_A(5),".",_A(75),"|",NULL),
		g_strconcat(_("_Info"),"|gtk-dialog-info|false|false|",_A(5),".",_A(37),"|",NULL),
		g_strconcat("||false|false|",_A(17),"|",NULL),
		g_strconcat(_("_Actions"),"|gtk-execute|false|false|",_A(18),"|",NULL),
		g_strconcat("\t",_("_Copy"),"|gtk-copy|true|false|",_A(5),".",_A(33),"|",NULL),
		g_strconcat("\t",_("_Move"),"|"ICON_DIR"move_"E2IP".png|true|false|",_A(5),".",_A(48),"|",NULL),
		g_strconcat("\t",_("_Link"),"|"ICON_DIR"symlink_"E2IP".png|true|false|",_A(5),".",_A(68),"|",NULL),
		g_strconcat("\t",_("_Trash"),"|"ICON_DIR"trash_"E2IP".png|true|false|",_A(5),".",_A(81),"|",NULL),
		g_strconcat("\t","\\<span foreground='red'>",_("_Delete"),"</span>|gtk-delete|true|false|",
			_A(5),".",_A(35),"|",NULL),
		g_strconcat("\t",_("_Rename.."),"|gtk-convert|true|false|",_A(5),".",_A(60),"|",NULL),
		g_strconcat("\t",_("Change _owners.."),"|"ICON_DIR"owners_"E2IP".png|true|false|",
			_A(5),".",_A(54),"|",NULL),
		g_strconcat("\t",_("Change _permissions.."),"|"ICON_DIR"permissions_"E2IP".png|true|false|",
			_A(5),".",_A(57),"|",NULL),
		g_strconcat("\t",_("Copy as.."),"|"ICON_DIR"copy_as_"E2IP".png|true|false|",_A(5),".",_A(34),"|",NULL),
		g_strconcat("\t",_("Move as.."),"|"ICON_DIR"move_as_"E2IP".png|true|false|",_A(5),".",_A(49),"|",NULL),
		g_strconcat("\t",_("Link as.."),"|"ICON_DIR"symlink_as_"E2IP".png|true|false|",_A(5),".",_A(69),"|",NULL),
		g_strconcat(_("_Plugins"),"|gtk-index|false|false|",_A(18),"|",NULL),
		g_strconcat("\t||false|false|",_A(12),".",_A(23),"|",NULL),
		g_strconcat("\t||false|false|",_A(17),"|",NULL),
		g_strconcat("\t",_("_Edit plugins.."),"|gtk-preferences|false|false|",_A(2),".",_C(31),"|",NULL),
		g_strconcat(_("_User commands"),"|"ICON_DIR"user_commands_"E2IP".png|false|false|",_A(18),"|",NULL),
		g_strconcat("\t",_("_Make new file.."),"|gtk-new|false|true|",_A(16),"|touch %{",_("Enter file name:"),"}",NULL),
		g_strconcat("\t",_("_Compare files"),"||false|true|",_A(16),
				"|>cmp -s %f %F && echo \"",_("The files are identical"),"\"\\n \\|\\| echo \"",_("The files are different"),"\"\\n",NULL),
		g_strconcat("\t",_("Compare _directories"),"||false|true|",_A(16),"|diff %d %D",NULL),
//		g_strconcat("\t",_("_Easytag"),"||false|true|",_A(16),"|easytag %d &",NULL),  //not common enough to be a default
		g_strconcat("\t",_("_Remove spaces"),"||false|true|",_A(16),"|>mv %f `echo %f \\| sed -e 's/ //g'` 2>/dev/null &",NULL),
		g_strconcat(_("Ma_ke dir.."),"|gtk-open|false|false|",_A(3),".",_A(50),"|",NULL),  //same _ as main toolbar
		g_strconcat("||false|false|",_A(17),"|",NULL),
		g_strconcat(_("_Bookmarks"),"|"ICON_DIR"bookmark_"E2IP".png|false|false|",_A(18),"|",NULL),
		g_strconcat("\t",_("Add to _top"),"||false|false|",_A(0),".",_A(27),"|",_A(93),NULL),
		g_strconcat("\t||false|false|",_A(0),".",_A(23),"|",NULL),
		g_strconcat("\t",_("Add to _bottom"),"|gtk-add|false|false|",_A(0),".",_A(27),"|",NULL),
		g_strconcat("\t||false|false|",_A(17),"|",NULL),
		g_strconcat("\t",_("_Edit bookmarks.."),"|gtk-preferences|false|false|",_A(2),".",_C(1),"|",NULL),
		g_strconcat("||false|false|",_A(17),"|",NULL),
		g_strconcat(_("_Edit filetype.."),"|gtk-preferences|false|false|",_A(2),".",_C(15),"|",NULL),
		g_strdup(">"),	//allow this to be freed
		NULL);
}
