/* $Id: e2_filetype.c 1124 2005-09-13 16:44:50Z tpgww $
 * 
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 1999 Michael Clark.
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
#include "e2_filetype.h"
#include "e2_config_dialog.h"

/**
 * @brief get actions 'list' for specified file extension
 *
 * Returns the actions array (in the file types hash)
 * for the extension.
 * The returned array must NOT be altered or freed.
 * Used only by the context-menu creator.
 *
 * @param ext string containing file extension to be matched
 *
 * @return null-terminated string array of actions for ext, or NULL if none found
 **/
gchar **e2_filetype_get_actions (gchar *ext)
{
	gchar **actions = g_hash_table_lookup (cfg.filetypes, ext);
	if (actions != NULL && *actions == '\0')
		return NULL;
	else
		return actions;
}
/**
 * @brief get default action for specified file extension
 *
 * Allocates and returns the first action from the actions array
 * (in the file types hash) for the specified extension.
 * The returned string needs to be freed.
 * Used only by callback open_cb.
 *
 * @param ext string containing extension to be matched
 *
* @return string default action for ext, or NULL if none found
 **/
gchar *e2_filetype_get_default_action (gchar *ext)
{
	gchar **actions = g_hash_table_lookup (cfg.filetypes, ext);
	if (actions == NULL || *actions == '\0')
		return NULL;
	else
		return g_strdup (*actions);
}
/**
 * @brief search the filetype hash for the specified file ext
 *
 * Used by 'get action' functions in this module
 * and by callback e2_menu_action_edit_filetype_cb()
 *
 * @param ext string containing extension to be matched
 *
 * @return null-terminated string array of actions for ext, or NULL if none found
 **/
/*FileType *e2_filetype_get (gchar *ext)
{
	GList *tmp;
	gchar *ext_copy, *s, *s2, *free_this;

	ext = e2_utils_str_to_lower(ext);
	for (tmp = cfg.filetypes; tmp != NULL; tmp = tmp->next)
	{
		FileType *ft = tmp->data;

		free_this = ext_copy = g_strdup(ft->extensions);
		while ((s = s2 = strchr (ext_copy, ',')) != NULL)
		{
			while (*(s2-1) == ' ') // get rid of whitespace
				PCHR(s2);
			*s2 = '\0';
			while ((*ext_copy == '.') || (*ext_copy == ' '))
				NCHR(ext_copy);

			if (g_str_equal(ext_copy, ext))
			{
				g_free(free_this);
				g_free(ext);
				return ft;
			}
			ext_copy = s+1;
		}
		while ((*ext_copy == '.') || (*ext_copy == ' '))
			NCHR(ext_copy);
		if (g_str_equal(ext_copy, ext))
		{
			g_free(free_this);
			g_free(ext);
			return ft;
		}
		g_free(free_this);
	}
	g_free(ext);

	return NULL;
} */

/* first check to see if a filetype with the same extensions exists..
 * if not it creates and adds a new filetype
 * /
void add_filetype(gchar *extensions, gchar *actions, gchar *description)
{
	GList *tmp;
	FileType *ft;

	for (tmp = cfg.filetypes; tmp != NULL; tmp = tmp->next)
	{
		ft = tmp->data;

		if (g_str_equal(extensions, ft->extensions))
		{
			g_strlcpy (ft->actions, actions, sizeof(ft->actions));
			g_strlcpy (ft->description, description, sizeof(ft->description));
			return;
		}
	}

	{
		ft = g_new0(FileType, 1);
		g_strlcpy (ft->description, description, sizeof(ft->description));
		g_strlcpy (ft->extensions, extensions, sizeof(ft->extensions));
		g_strlcpy (ft->actions, actions, sizeof(ft->actions));
		cfg.filetypes = g_list_append(cfg.filetypes, ft);
	}
} */
/*	if (!read_filetypes_file())
	{
		// Setup some default filetypes
		add_filetype(_I("<directory>"), "_Open@cd %%d/%%f,O_pen in other@!?file.open in other", _("directories"));
		add_filetype(_I("<executable>"), "_Run@"E2_COMMAND_PREFIX"%f", _("executables"));
		add_filetype("jpeg,jpg,png,xpm,gif", "xv,gimp", _("image files"));
		add_filetype("c,cpp,h,pl,java", "Edit with _vi@x vi,Edit with _emacs@x emacs", _("source code files"));
		add_filetype("o,so,a", "_View symbols@x nm %f | less", _("object files"));
		add_filetype("htm,html", "_Mozilla@mozilla %d/%f,_Lynx@x lynx", _("HTML documents"));
		add_filetype("tar", "Unpack@tar xvf %f,Unpack in other pane@cd %D; tar xvf %d/%f,PLUGIN:Unpack@!?Unpack,View contents@tar tvf %f | less", _("plain tarballs"));
		add_filetype("tar.gz,tgz", "_Unpack@gunzip -c %f | tar xvf -,Unpack in _other pane@cd %D; gunzip -c %d/%f | tar xvf -,PLUGIN:Unpack@!?Unpack,_View contents@gunzip -c %f | tar tvf - | less", _("gzip tarballs"));
		add_filetype("tar.bz2","_Unpack@bzip2 -d -c %f | tar xvf -,Unpack in _other pane@cd %D; bzip2 -d -c %d/%f | tar xvf -,PLUGIN:Unpack@!?Unpack,_View contents@bzip2 -d -c %f | tar tvf - | less", _("bzip2 tarballs"));
		add_filetype("zip", "_Unzip@unzip %f,Unzip in _other pane@cd %D; unzip %d/%f,PLUGIN:Unpack@!?Unpack,_View contents@unzip -l %f | less", _("zip archives"));
		add_filetype("rpm", "_View info@rpm -qlip %f,_Install@su \"rpm -Uvh \\\"%f\\\" \"", _("RPM packages"));
		add_filetype("pdf", "acroread,xpdf", _("Adobe Acrobat documents"));
		add_filetype("ps", "gv", _("Postscript documents"));
		add_filetype("txt", "PLUGIN:View@!?EmelView,_Edit with vi@x vi", _("text documents"));
		add_filetype("mp3,m3u,wav,ogg,mp2", "xmms,xmms -e", _("audio files"));
		add_filetype("avi,divx,mpg,mpeg,mov", "mplayer", _("video files"));
	} */

/**
 * @brief load all filetype info into hash
 *
 * Converts config tree data into filetype runtime data hash
 * by means of a depth-first search of 3-level tree (graph)
 * For each 'category' of filetypes, the order of level-2 
 * 'commands' and 'extensions' nodes is irrelevant, and multiple 
 * 'commands' and/or 'extensions' nodes are allowed.
 * The hash entry for each file extension is an arrray of command strings.
 * If a (level-3) command has a separate menu name, the command string is
 * stored as menuname"@"commandstring (so menu names can't have a @ in them)
 * NB in the previous line, the "" around the @ are to fix doxygen, the string 
 * doesn't really have them
 *
 * @return
 **/
void e2_filetype_add_all (void)
{
	E2_OptionSet *set;
	GtkTreeIter iter, iter2, iter3;  //3 tree levels to scan
	gchar *category, *extension, *cmd_string;
	GPtrArray *e_ray = g_ptr_array_sized_new (5);

	cfg.filetypes = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
	cfg.typelist = NULL;

	if (
		((set = e2_option_get_simple ("filetypes")) != NULL) &&
		(gtk_tree_model_get_iter_first (set->ex.tree.model, &iter) == TRUE)
		)
	{
		do
		{  //category loop = level 1
			//re-use the extensions array
			if (e_ray->len > 0)
				g_ptr_array_remove_range (e_ray, 0, e_ray->len);
			//setup a fresh commands  array for each category
			GPtrArray *c_ray = g_ptr_array_sized_new (5);
			//note the category, in case error msg is needed
			gtk_tree_model_get (set->ex.tree.model, &iter, 0, &category, -1);
			if (gtk_tree_model_iter_children (set->ex.tree.model, &iter2, &iter))
			{
				do
				{  // extension or command loop = level 2
					gtk_tree_model_get (set->ex.tree.model, &iter2, 1, &extension, -1);
					if (gtk_tree_model_iter_children (set->ex.tree.model, &iter3, &iter2))
					{
						if (g_str_equal (extension, _C(42)))  //extensions node found
						{
							g_free (extension);
							do
							{  //extension loop = level 3, build the extensions array
								gtk_tree_model_get (set->ex.tree.model, &iter3, 1, &extension,
									 -1);
								g_ptr_array_add (e_ray, extension);
							} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter3));
						}
						else if (g_str_equal (extension, _C(8)))  //commands node found
						{
							g_free (extension);
							do
							{  //commands loop also = level 3, build the commands array
								//this time, extension is really a command name 
								gtk_tree_model_get (set->ex.tree.model, &iter3, 1, &extension,
									2, &cmd_string, -1);
								if (*cmd_string != '\0')
								{  //concatenate with separator
									gchar *freeme = extension;
									extension = g_strconcat (extension, "@", cmd_string, NULL);
									g_free (freeme);
								}
								g_free (cmd_string);
								g_ptr_array_add (c_ray, extension);
							} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter3));
						}
						else
						{ //OOPS
							printd (WARN, "un-recognised node in filetypes config for %s", category);
						}
					}
					else
						g_free (extension); //no children, just cleanup

				} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter2));

				g_ptr_array_add (c_ray, NULL);	//add end marker (for detecting end, also to support g_strfreev)
				//register the new filetype(s)
				gint i;
				gchar **iterator = (gchar **) e_ray->pdata;
				for (i = 0; i < e_ray->len; i++, iterator++)
					g_hash_table_insert (cfg.filetypes, *iterator, c_ray->pdata);
				//one copy of the commands array, so it can't be freed when hash is destroyed,
				//so note where it is, for separate cleanup purposes
				cfg.typelist = g_slist_append (cfg.typelist, c_ray->pdata);
				g_ptr_array_free (c_ray, FALSE);  //cleanup, ready for next category
			}
			g_free (category);
		} while (gtk_tree_model_iter_next (set->ex.tree.model, &iter));

		g_ptr_array_free (e_ray, FALSE);  //no need to free any strings, that is done when they are added to hash

	} else
	{
		printd (WARN, "no config filetype information");
	}
}

/**
 * @brief run filetype action on selected items when %f missing
 *
 * This is like a regular do_command, but if there is no %f 
 * in the action given, it appends the selected item(s) to the
 * end of the action string
 * This is so the user can just enter like "xmms" as an action and not have
 * to remember to append the %f 
 *
 * @param action string
 *
* @return
 **/
void e2_filetype_exec_action (gchar *action)
{
	e2_command_run (action, E2_COMMAND_RANGE_FILE_ACTION);
	/*
	FileInfo *info;
	GString *command;

	g_return_if_fail(action != NULL);

	if (strncmp(action, "PLUGIN:", 7) == 0)   //_I(
	{
		e2_plugin_do_action (NULL, get_plugin_by_name(action+7));
		return;
	}
	gboolean other = FALSE;
	if (action[0] == '!')
	{
		NCHR(action);
		other = TRUE;
	}

	if (!other && (strchr (action, '%') == NULL))
	{ // There wasn't a macro found in the action
		GList *tmp, *base;

		e2_filelist_disable_refresh();
		base = tmp = get_selection(curr_view);
		if (tmp == NULL)
		{
			e2_filelist_enable_refresh();
			return;
		}

		command = g_string_sized_new(E2_MAX_LEN);
		g_string_assign(command, action);

		for (; tmp != NULL; tmp = tmp->next)
		{
			info = tmp->data;
			g_string_append_printf(command, " \"%s\"", info->filename);
		}
		do_command(command->str);
		g_list_free(base);
		g_string_free(command, TRUE);
		e2_filelist_enable_refresh();
	} else
	{
		//There was a macro in the action
		if ((command = e2_utils_expand_macros(action, NULL)) != NULL)
		{
			do_command(command->str);
			g_string_free(command, TRUE);
		}
	}
	*/
}
/**
 * @brief change filetype config data for a selected item
 *
 * @param info pointer to data struct for the item, or NULL to use the 1st selected item in the active pane
 *
 * @return
 **/
void e2_filetype_config (FileInfo *info)
{
	//find current filetype
	if (info == NULL)
	{
		info = e2_fileview_get_selected_first_local (curr_view);
		if (info == NULL)
			return;
	}
	gchar *ext;

	if (e2_fs_is_dir (info))
		ext =  _("<directory>");
	else if (e2_fs_is_executable (info))
		ext = _("<executable>");
	else
	{
		//maybe extra '.'(s) in the name eg version no so ext needs to be reparsed
		ext = strchr (info->filename, '.');	//localised text, assumes '.' is ascii
		if (ext == NULL || *(ext+1) == '\0')
			ext = _("<none>");
		else
			ext++;
	}
	//open the config dialog at the appropriate place
	F_DECLARE;
	gchar *utf = F_FILENAME_FROM_LOCALE (ext);
	e2_filetype_config_show (utf);
	F_FREE (utf);
}
/**
 * @brief open filetypes config dialog
 *
 * This may be used in a callback from the filetype dialog,
 * in which case @a extension == NULL
 * OR this may be called with a specific extension, 
 * in which case @a extension != NULL
 *
 * @param extension pointer to extension to change, or NULL
 *
 * @return
 **/
void e2_filetype_config_show (gchar *extension)
{
	//create dialog, but don't show it yet
	E2_SpecificConfDialogRuntime *rt = e2_config_dialog_single 
		("filetypes", e2_filetype_apply_allnew, FALSE); //internal name, no translation
	//get path to filetypes node
	GtkTreeModel *model = rt->set->ex.tree.model;
	GtkTreeIter iter;
	if (!gtk_tree_model_get_iter_first (model, &iter))
	{
		printd (WARN, "e2_filetype_config_show: can't find config data for filetypes");
		gtk_widget_destroy (rt->dialog);
		g_free (rt);
		return;
	}
	GtkTreeView *treeview = GTK_TREE_VIEW (rt->set->widget);

	gboolean found = FALSE;
	if (extension != NULL && *extension != '\0')
	{
		//if this is a specific-extension edit, find it in the tree
		gchar *tmp = extension;
		while (!found)
		{
			found = _e2_tree_find_iter_from_str (model, 1, 
				tmp, &iter, FALSE);
			//make sure there's no 'smaller' valid extension
			if (!found)
			{
				tmp = strchr (tmp, '.');	//always ascii
				if (tmp == NULL)
					break;
				tmp++;	//pass the '.'
				gtk_tree_model_get_iter_first (model, &iter);
			}
		}
	}

	gtk_tree_view_collapse_all (treeview);	//this fn causes dialog window to be shown !
	if (found)
	{
		//path to extension
		GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
		GtkTreePath *path2 = gtk_tree_path_copy (path);
		gtk_tree_path_up (path2);	//extensions node
		gtk_tree_path_up (path2);	//category node
		//show it
//		gtk_tree_view_expand_to_path (treeview, path);
		gtk_tree_view_expand_row (treeview, path2, TRUE);
		GtkTreeViewColumn *col = gtk_tree_view_get_column (treeview, 0);
		gtk_tree_view_set_cursor  (treeview, path, col, FALSE);
		gtk_tree_view_scroll_to_cell (treeview, path, col, TRUE, 0.5, 0.5);
		gtk_tree_path_free (path);
		gtk_tree_path_free (path2);
	}
//	else
//	{
		//non-recognised extension, just show all the categories
//		gtk_tree_view_collapse_all (treeview);
//		gtk_tree_view_expand_row (treeview, path, FALSE);
//	}
	//go there
//	GtkTreeViewColumn *col = gtk_tree_view_get_column (treeview, 0);
//	gtk_tree_view_set_cursor  (treeview, path, col, FALSE);
//	gtk_tree_view_scroll_to_cell (treeview, path, col, TRUE, 0.5, 0.5);
//	gtk_tree_path_free (path);

	//now we're ready to show this dialog
//	gtk_widget_show_all (rt->dialog);
	gtk_widget_show_all (rt->set->widget);
}
/**
 * @brief update filetypes information
 *
 * This is the 'apply' fn for the edit filetypes dialog, also used in 
 * general config dialog
 * The entries in cfg.typelist are NULL-terminated string arrays,
 * but are created as pointer arrays, not with g_strsplit
 *
 * @return
 **/
void e2_filetype_apply_allnew (void)
{
	g_hash_table_destroy (cfg.filetypes);
	g_slist_foreach (cfg.typelist, (GFunc) g_strfreev, NULL);
	g_slist_free (cfg.typelist);
	e2_filetype_add_all ();
}
/**
 * @brief register filetype action data
 *
 * @return
 **/
void e2_filetype_actions_register (void)
{
	gchar *action_name = g_strconcat(_A(2),".",_C(15),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		e2_filetype_config, NULL, FALSE);
}

/**
 * @brief register filetype config data
 *
 * Ths sets up for the corresponding page in the config dialog
 * and prepares the default tree of filetype config data
 * The format of the data needs to conform with the 3-level
 * interrogation scheme used in e2_filetype_add_all().
 *
 * @return
 **/
void e2_filetype_options_register (void)
{
	//no screen rebuilds needed after any change to this option
	gchar *group_name = _C(15); //_("filetypes")
	E2_OptionSet *set = e2_option_tree_register ("filetypes", group_name,
		group_name, NULL, NULL, NULL,
		E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL,
		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_BUILDFILES);
	e2_option_tree_add_column (set, _("Category"), E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	//this column used for headings, extensions and command labels
	e2_option_tree_add_column (set, "", E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Command"), E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_sync_columns (set);

	_e2_option_tree_prepare_default (set,
		g_strdup ("<filetypes"),
		g_strconcat (_("directories"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),	// _("extensions")
				g_strconcat ("\t\t|",_("<directory>"),"|",NULL), //_I(
			g_strconcat ("\t|",_C(8),"|",NULL),  // _("commands")
				g_strconcat ("\t\t|",_("_Open"),"|cd %%f",NULL),  //FIXME = explain the extra %, in user help
				g_strconcat ("\t\t|",_("O_pen in other"),"|<",_A(5),".",_A(52),NULL),
				g_strdup ("\t\t|Mount|mount %%d%%f"),
				g_strdup ("\t\t|Unmount|umount %%d%%f"),
		g_strconcat (_("executables"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strconcat ("\t\t|",_("<executable>"),"|",NULL),	//_I(
				g_strconcat ("\t\t|",_("<none>"),"|",NULL),	//_I( //no extension
				g_strdup ("\t\t|sh|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Run"),"|"E2_COMMAND_PREFIX"%f",NULL),
//				g_strconcat ("\t\t|",_("Edit with _vi"),"|x vi",NULL),
//				g_strconcat ("\t\t|",_("Edit with _emacs"),"|x emacs",NULL),
		g_strconcat (_("office documents"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|doc|"),
				g_strdup ("\t\t|odt|"),
				g_strdup ("\t\t|sxw|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Openoffice"),"|oowriter",NULL),
		g_strconcat (_("HTML documents"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|htm|"),
				g_strdup ("\t\t|html|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Firefox"),"|mozilla-firefox file://%%d%f",NULL),
				g_strconcat ("\t\t|",_("_Mozilla"),"|mozilla file://%%d%f",NULL),
				g_strconcat ("\t\t|",_("_Lynx"),"|x lynx",NULL),
		g_strconcat (_("PDF documents"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|pdf|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strdup ("\t\t|xpdf|xpdf"),
				g_strdup ("\t\t|acroread|acroread"),
		g_strconcat (_("postscript documents"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|ps|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strdup ("\t\t|gv|gv"),
		g_strconcat (_("text documents"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|txt|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
// not loaded				g_strconcat ("\t\t|",PLUGIN,":",_("View"),"|?",PLUGIN,".","view",NULL),  //conformed name
				g_strconcat ("\t\t|",_("View"),"|<",_A(5),".",_A(75),NULL), //_("file.view")
				g_strconcat ("\t\t|",_("_Edit with.."),"|%{",_("Editor command:"),"} %f",NULL),
				g_strconcat ("\t\t|",_("Edit with vi"),"|x vi",NULL),
		g_strconcat (_("spreadsheets"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|xls|"),
				g_strdup ("\t\t|ods|"),
				g_strdup ("\t\t|sxc|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Openoffice"),"|oocalc",NULL),
		g_strconcat (_("audio files"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|mp3|"),
				g_strdup ("\t\t|m3u|"),
				g_strdup ("\t\t|wav|"),
				g_strdup ("\t\t|ogg|"),
				g_strdup ("\t\t|mp2|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strdup ("\t\t|_Muine|muine"),
				g_strdup ("\t\t|_Xmms|xmms"),
				g_strdup ("\t\t|xmms _queue|xmms -e"),
		g_strconcat (_("image files"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|jpeg|"),
				g_strdup ("\t\t|jpg|"),
				g_strdup ("\t\t|gif|"),
				g_strdup ("\t\t|svg|"),
				g_strdup ("\t\t|png|"),
				g_strdup ("\t\t|xpm|"),
//				g_strdup ("\t\t|sxd|"),
//				g_strdup ("\t\t|sxi|"),
//				g_strdup ("\t\t|sxp|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strdup ("\t\t|_gview|gview"),
				g_strdup ("\t\t|G_Qview|gqview"),
//				g_strdup ("\t\t|GImage_View|gimageview"),
//				g_strconcat ("\t\t|",_("_Oo impress"),"|ooimpress",NULL),
//				g_strconcat ("\t\t|",_("Oo _draw"),"|oodraw",NULL),
				g_strdup ("\t\t|gi_mp|gimp"),
				g_strdup ("\t\t|i_nkscape|inkscape"),
				g_strdup ("\t\t|_xv|xv"),
		g_strconcat (_("video files"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|avi|"),
				g_strdup ("\t\t|divx|"),
				g_strdup ("\t\t|mpg|"),
				g_strdup ("\t\t|mpeg|"),
				g_strdup ("\t\t|mov|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strdup ("\t\t|_Totem|totem"),
				g_strdup ("\t\t|_mplayer|mplayer"),
		g_strconcat (_("plain tarballs"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|tar|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("Unpack"),"|tar -xf %f",NULL),
			g_strconcat ("\t\t|",_("Unpack in other pane"),"|<",_A(11),".",_A(78),";x tar -C %D -xf %f;<",_A(11),".",_A(79),NULL),
				g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|<",PLUGIN,".","unpack",NULL),  //conformed name
				g_strconcat ("\t\t|",_("_List contents"),"|>tar -tf %f \\| less",NULL),
		g_strconcat (_("gzip tarballs"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|tar.gz|"),
				g_strdup ("\t\t|tgz|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Unpack"),"|tar -zxf  %f",NULL),
				g_strconcat ("\t\t|",_("Unpack in _other pane"),"|<",_A(11),".",_A(78),";tar -C %D -zxf %f;<",_A(11),".",_A(79),NULL),
				g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|<",PLUGIN,".","unpack",NULL),   //conformed name
				g_strconcat ("\t\t|",_("_List contents"),"|>tar -ztf %f \\| less",NULL),
		g_strconcat (_("bzip2 tarballs"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|tar.bz2|"),
				g_strdup ("\t\t|tbz2|"),
				g_strdup ("\t\t|tbz|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Unpack"),"|>bzip2 -d -c %f \\| tar -xf -",NULL),
				g_strconcat ("\t\t|",_("Unpack in _other pane"),"|<",_A(11),".",_A(78),";>bzip2 -d -c %f \\| tar -C %D -xf -;<",_A(11),".",_A(79),NULL),
				g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|<",PLUGIN,".","unpack",NULL),    //conformed name
				g_strconcat ("\t\t|",_("_List contents"),"|>bzip2 -d -c %f \\| tar -tf - \\| less",NULL),
		g_strconcat (_("zip archives"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|zip|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Unzip"),"|gunzip %f",NULL),
				//NB space after %F in next line is needed
				g_strconcat ("\t\t|",_("Unzip in _other pane"),"|<",_A(11),".",_A(67),";unzip %F;<",_A(11),".",_A(67),NULL),
				g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|<",PLUGIN,".","unpack",NULL),    //conformed name
				g_strconcat ("\t\t|",_("_List contents"),"|>unzip -l %f \\| less",NULL),
/* plugin functionality not tested yet, for these
		g_strconcat (_("arj archives"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|arj|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|<",PLUGIN,".","unpack",NULL),
		g_strconcat (_("rar archives"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|rar|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",PLUGIN,":",_("Unpack"),"|<",PLUGIN,".","unpack",NULL),
*/
		g_strconcat (_("RPM packages"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|rpm|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("In_formation"),"|rpm -qlip %f",NULL),
				g_strconcat ("\t\t|",_("_Install"),"|su \"rpm -Uvh %%f\"",NULL),
		g_strconcat (_("source code files"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|c|"),
				g_strdup ("\t\t|cpp|"),
				g_strdup ("\t\t|h|"),
				g_strdup ("\t\t|pl|"),
				g_strdup ("\t\t|java|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_Edit with.."),"|%{",_("Editor command:"),"} %f",NULL),
				g_strconcat ("\t\t|",_("Edit with _vi"),"|x vi",NULL),
//				g_strconcat ("\t\t|",_("Edit with _emacs"),"|x emacs",NULL),
		g_strconcat (_("object files"),"||",NULL),
			g_strconcat ("\t|",_C(42),"|",NULL),
				g_strdup ("\t\t|o|"),
				g_strdup ("\t\t|so|"),
				g_strdup ("\t\t|a|"),
			g_strconcat ("\t|",_C(8),"|",NULL),
				g_strconcat ("\t\t|",_("_View symbols"),"|>nm %f \\| less",NULL),
		g_strdup (">"),
		NULL);
}
