/* $Id: e2_output.c 529 2005-03-25 12:44:57Z tpgww $
 * 
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Portions copyright (C) 2004 Florian Zähringer <flo.zaehringer@web.de>
 * Portions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file src/output/e2_output.c 
 * @brief output pane creation and action functions
 *
 * includes actions related to output contents, but not 
 * to the output visibility
 *
 **/

#include "emelfm2.h"
#include <string.h>
//FIXME: bad trick, what's the right way to do it?
//#define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
//#include <gtk/gtktextlayout.h>
//#undef GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
#include "e2_output.h"

#define E2_PANED_TOLERANCE 10

  /*****************/
 /***** utils *****/
/*****************/
/**
 * @brief determine whether a position in the output pane text buffer is presently visible
 *
 * lifted from gtktextview.c
 *
 * @param text_view the output pane textview widget
 * @param iter pointer to data for the position in @a text_view that is to be checked
 *
 * @return TRUE if @a iter relates to a point in @a text_view that is NOT presently visible
 **/
static gboolean _e2_output_iter_offscreen 
	(GtkTextView *text_view, GtkTextIter *iter)
{
	GdkRectangle visible_rect;
	gtk_text_view_get_visible_rect (text_view, &visible_rect);

	//FIXME this is undocumented gtk-internal stuff
//	return gtk_text_layout_clamp_iter_to_vrange (text_view->layout, iter,
//		visible_rect.y, visible_rect.y + visible_rect.height);

	GdkRectangle iter_rect;
	gtk_text_view_get_iter_location (text_view, iter, &iter_rect);
	gboolean result = (iter_rect.y + iter_rect.height) > (visible_rect.y + visible_rect.height)
			|| (iter_rect.y < visible_rect.y);
	return (result);
}
/**
 * @brief move the 'page' displayed in the output pane, relative to its text content
 *
 * @param down TRUE to move down, FALSE to move up
 * @param arg string containing a number, the no. of 'moves'
 * @param page TRUE move @a arg 'pages', FALSE move @a arg 'steps'
 *
 * @return
 **/
static void _e2_output_scroll_helper (gboolean down, gchar *arg, gboolean page)
{
	printd (DEBUG, "scroll_helper (down:%d,arg:%s,page:%d)", down, arg, page);
	E2_OutputRuntime *rt = &app.output;
	g_return_if_fail (rt->scroll != NULL);
	gchar *end = NULL;
	gdouble times = 1.0;
	if (arg != NULL)
		times = g_ascii_strtod (arg, &end);
	if (end == arg)
		times = 1.0;
	GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment 
		(GTK_SCROLLED_WINDOW (rt->scroll));
	gdouble value = gtk_adjustment_get_value (vadj);
	gint inc = page ? vadj->page_increment : vadj->step_increment;
	if (down)
	{
		value += (inc * times);
		if (value > (vadj->upper - vadj->page_size))
			value = vadj->upper - vadj->page_size;
	}
	else
		value -= (inc * times);

	gtk_adjustment_set_value (vadj, value);
}
/**
 * @brief set popup menu position
 *
 * This function is supplied when calling gtk_menu_popup(), to position
 * the displayed menu.
 * set @a push_in to TRUE for menu completely inside the screen,
 * FALSE for menu clamped to screen size
 *
 * @param menu the GtkMenu to be positioned
 * @param x	place to store gint representing the menu left
 * @param y  place to store gint representing the menu top
 * @param push_in place to store pushin flag
 * @param textview output pane widget in focus when the menu key was pressed
 *
 * @return
 **/
static void _e2_output_set_menu_position (GtkWidget *menu,
	gint *x, gint *y, gboolean *push_in, GtkWidget *textview)
{
	gint left, top;
	gtk_window_get_position (GTK_WINDOW (app.main_window),
		&left, &top);
	GtkAllocation alloc = textview->allocation;
	*x = left + alloc.x + alloc.width/2;
	*y = top + alloc.y + alloc.height/2;
	*push_in = TRUE;
}

  /*******************/
 /***** actions *****/
/*******************/

/**
 * @brief print to output pane, with default settings
 *
 * Any escaped newlines are converted to real ones.
 *
 * @param data UNUSED
 * @param message string to be printed
 *
 * @return
 **/
static void _e2_output_print_action (gpointer data, gchar *message)
{
	gchar *real_message = e2_utils_str_replace (message, "\\n", "\n");
	e2_output_print (real_message, NULL, FALSE, FALSE, FALSE, NULL);
	g_free (real_message);
}
/**
 * @brief print help message to output pane
 *
 * @param data UNUSED
 * @param arg string indicating which message is to be displayed
 *
 * @return
 **/
static void _e2_output_help_action (gpointer data, gchar *arg)
{
	if ((arg == NULL) || (*arg == '\0') || g_str_has_prefix (arg, _("commands")))
		e2_command_output_help ();
	else if (g_str_has_prefix (arg, _("keys")))	//this must be translated same as in default aliases
		e2_keybinding_output_help (arg);
}
/**
 * @brief move the displayed text window by @a arg 'steps'
 *
 * @param down TRUE to move down, FALSE to move up
 * @param arg string containing a number, the no. of 'steps' to move
 *
 * @return
 **/
static void _e2_output_scroll_step (gboolean down, gchar *arg)
{
	_e2_output_scroll_helper (down, arg, FALSE);
}
/**
 * @brief move the displayed text window by @a arg 'pages'
 *
 * @param down TRUE to move down, FALSE to move up
 * @param arg string containing a number, the no. of 'pages' to move
 *
 * @return
 **/
static void _e2_output_scroll_page (gboolean down, gchar *arg)
{
	_e2_output_scroll_helper (down, arg, TRUE);
}
/**
 * @brief move the displayed text window to start or end of its content
 *
 * @param down TRUE to move to end, FALSE to move to start
 * @param arg string UNUSED
 *
 * @return
 **/
static void _e2_output_scroll_all (gboolean down, gchar *arg)
{
	E2_OutputRuntime *rt = &app.output;
	g_return_if_fail (rt->scroll != NULL);
	GtkAdjustment *vadj = gtk_scrolled_window_get_vadjustment 
		(GTK_SCROLLED_WINDOW (rt->scroll));
	if (down)
		gtk_adjustment_set_value (vadj, vadj->upper - vadj->page_size);
	else
		gtk_adjustment_set_value (vadj, 0.0);
}
/**
 * @brief clear all content of the output pane
 *
 * @return
 **/
static void _e2_output_clear (void)
{
	GtkTextTagTable *table = gtk_text_buffer_get_tag_table (app.output.buffer);
	GtkTextBuffer *buffer = gtk_text_buffer_new (table);
	GtkTextIter start;
	gtk_text_buffer_get_start_iter (buffer, &start);
	app.output.mark = gtk_text_buffer_create_mark (buffer,
		"internal-end-mark", &start, FALSE);

	gtk_text_view_set_buffer (app.output.text, buffer);
	app.output.buffer = gtk_text_view_get_buffer (app.output.text);
	g_object_unref (buffer);
	app.output.onscreen = TRUE;
}
/**
 * @brief update gtk's flag which sets output pane text wrapping
 *
 * This is a hook fn
 *
 * @param pvalue pointerised version of the T/F value to be set
 * @param rt pointer to data struct for output pane
 *
 * @return
 **/
static void _e2_output_set_op_wrap (gpointer pvalue, E2_OutputRuntime *rt)
{
//	printd (DEBUG, "_e2_output_set_op_wrap (pvalue:_,rt:_)");
	gint value = GPOINTER_TO_INT (pvalue);
	gint cur = gtk_text_view_get_wrap_mode (rt->text);
	if (cur != value)
		gtk_text_view_set_wrap_mode (rt->text, value);
}
/**
 * @brief construct and show output-pane context menu
 *
 * @param textview the textview widget where the click happened
 * @param event_button which mouse button was clicked (0 for a menu key)
 * @param event_time time that the event happened (0 for a menu key)
 * @param rt runtime struct to work on
 *
 * @return
 **/
static void _e2_output_show_context_menu (GtkWidget *textview,
	guint event_button, gint event_time, E2_OutputRuntime *rt)
{
	gchar *item_name;
	GtkWidget *menu = gtk_menu_new ();
	item_name = g_strconcat (_A(8),".",_A(28),NULL);
	e2_menu_add_action (menu, _("_Hide"), ICON_DIR"output_hide_"E2IP".png",
		_("Do not show the output pane"),
		item_name, "1", NULL);  //no string translation
	g_free(item_name);
	item_name = g_strconcat (_A(8),".",_A(28),NULL);
	e2_menu_add_action (menu, _("_Toggle full"), NULL,
		_("Toggle output pane size to/from the full window size"),
		item_name, "*,0", NULL);  //no string translation
	g_free(item_name);
	item_name = g_strconcat (_A(8),".",_A(29),NULL);
	e2_menu_add_action (menu, _("_Clear"), GTK_STOCK_CLEAR,
		_("Clear output pane"),
		item_name, NULL, NULL);
	g_free(item_name);
/*	item_name = g_strconcat (_A(8),".",_A(45),NULL);
	e2_menu_add_action (menu, _("C_ommand help"), GTK_STOCK_HELP,
		_("Show information about using the command line"),
		item_name, "", NULL);
	g_free(item_name);
*/	GtkWidget *submenu = e2_menu_add_submenu (menu, _("_Settings"));
	e2_option_create_menu (GTK_WIDGET (rt->text), submenu,
		rt->opt_wrap, NULL, NULL,
		rt->opt_show_on_new, NULL, NULL,
		rt->opt_show_on_focus_in, NULL, NULL,
		rt->opt_hide_on_focus_out, NULL, NULL,
		rt->opt_jump, NULL, NULL,
		rt->opt_jump_follow, NULL, NULL,
		rt->opt_jump_end, NULL, NULL,
		NULL);
	e2_menu_add_separator (submenu);
	item_name = g_strconcat (_A(2),".",_A(32),NULL);
	e2_menu_add_action (submenu, _("_Other"), GTK_STOCK_PREFERENCES,
		_("Open the configuration dialog at the output options page"),
		item_name, _C(25), //_("output")
		NULL);
	g_free(item_name);
	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);
	if (event_button == 0)
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
			(GtkMenuPositionFunc) _e2_output_set_menu_position,
			textview, event_button, event_time);
	else
		//this was a button-3 click
		gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
			NULL, NULL, event_button, event_time);
}

  /*********************/
 /***** callbacks *****/
/*********************/

/**
 * @brief process mouse button-click in the output pane
 *
 * L = focus, M = hide, R = context menu
 *
 * @param widget the textview widget where the click happened
 * @param event gdk event data struct
 * @param rt runtime struct to work on
 *
 * @return TRUE if the signal has been handled
 **/
static gboolean _e2_output_output_pane_clicked_cb (GtkWidget *widget,
	GdkEventButton *event, E2_OutputRuntime *rt)
{
	if (event->button == 1)
	{
		gchar *action_name = g_strconcat (_A(1),".",_A(38),NULL);  //_("command.focus")	
		e2_action_run_simple (action_name, NULL);
		g_free (action_name);
		return FALSE;
	}
	else if (event->button == 2)
	{
		e2_window_output_hide (NULL, NULL);
		return TRUE;
	}
	else if (event->button == 3)
	{
		_e2_output_show_context_menu (widget, event->button, event->time, rt);
		return TRUE;
	}
	return FALSE;
}
/**
 * @brief process menu button press in the output pane
 *
 * @param widget the textview widget where the press happened
 * @param rt output runtime struct to work on
 *
 * @return TRUE if the signal has been handled
 **/
static gboolean _e2_output_popup_menu_cb (GtkWidget *widget,
	E2_OutputRuntime *rt)
{
	gint event_time = gtk_get_current_event_time ();
	_e2_output_show_context_menu (widget, 0, event_time, rt);
	return TRUE;
}
/**
 * @brief Determine whether the buffer end-mark is on-screen, after manual change to the vertical adjustment 
 *
 * This is a callback for manual changes to the output textview vertical adjustment
 *
 * @param adjust UNUSED the vertical adjustment for the textview
 * @param rt data struct for the output pane
 *
 * @return
 **/
static void _e2_output_scrolled_cb (GtkAdjustment *adjust, E2_OutputRuntime *rt)
{
	GtkTextIter iter;
	if (_e2_option_bool_get (app.output.opt_jump_end))
		//scrolling only when new output belongs to the last context in the textbuffer
		gtk_text_buffer_get_end_iter (rt->buffer, &iter);
	else
		gtk_text_buffer_get_iter_at_mark (rt->buffer, &iter, rt->mark);

	if (_e2_output_iter_offscreen (rt->text, &iter))
	{
		if (rt->onscreen)
			printd (NOTICE, "not following anymore");
		rt->onscreen = FALSE;
	}
	else
	{
		if (!rt->onscreen)
			printd (NOTICE, "following again");
		rt->onscreen = TRUE;
	}
}
/*
static gboolean test_cb (GtkWidget *widget, GdkEventMotion *event)
{
	int x, y;
	GdkModifierType state;
	if (event->is_hint)
	{
	        gdk_window_get_pointer(event->window, &x, &y, &state);
	        printf("x = %d y = %d\n", x, y);
	}
	else 
	{
	        printf("x: %lf y: %lf\n", event->x, event->y);
	}
	return FALSE;  
} 
*/
  /******************/
 /***** public *****/
/******************/

/**
 * @brief show @a msg in output pane
 *
 * While parsing @a msg, it is treated as ascii, for speed,
 * then converted to utf before display.
 * for an error message, style parameters are ignored
 *
 * @param msg actual message
 * @param origin context to which the message belongs or NULL for default
 * @param error TRUE if @a msg is an error message
 * @param debug TRUE if @a msg is a debug message
 * @param newline TRUE if @a msg is to be printed on a separate line
 * @param first_tag one or more tags to apply, terminated by NULL
 *
 * @return 
 **/
//CHECKME: any good reason to _immediately_ convert msg to utf8
// (if it wasn't already) and use only utf8 functions to process it
void e2_output_print (gchar *msg, gchar *origin, gboolean error,
	gboolean debug, gboolean newline, const gchar *first_tag, ...)
{
	//early exit
	if (msg == NULL) return;

	if (error)
	{  //re-entrant
//		e2_output_print (msg, origin, FALSE, debug, newline, "red", "bold", NULL);
		//re-entrance abandoned, this should never happen, now
		printd (WARN, "Fix display of error message:\n%s", msg);
		return;
	}
//	printd (DEBUG, "e2_ouput_print (msg:,origin:%s,error:%d,debug:%d,newline:%d,first_tag:)",
//		origin, error, debug, newline);

	//if this is not a debug message show the output pane
	//if the user wishes that
	if (!debug && !app.output.visible && e2_option_bool_get ("show-output-window-on-output"))
	{
//		gdk_threads_enter ();
		e2_window_output_show (NULL, NULL);
//		gdk_threads_leave ();
	}
/* do we need to do this ASAP?
	gchar *utf;
	gboolean wasutf;	//for remembering whether to free utf
	//if message is already utf8
	if ((wasutf=g_utf8_validate (msg, -1, NULL)))
		//just work with the original
		utf = msg;
	else
		//convert it to utf8
		utf = e2_utf8_from_locale (msg); 	//freeme later
*/

	//parse the msg string to handle special characters

	gint len = strlen (msg);
	//process leading backspaces
	//these delete characters from the last message of this context
	gint back = 0;
	while (((gint) msg[0]) == '\b')
	{
		back++;
		msg++;
		len--;
	}

	//FIXME: handle \r correctly
	//flag whether there's a carriage return '\r' in the message
	gboolean line_back = FALSE;
	//flag for backspaces not at the beginning of the message
	gboolean backspaces_left = FALSE;
	//flag whether message variable should be freed
	gboolean free_message = FALSE;
	gint i;

	//FIXME make this faster !!
	for (i = 0; i < len; i++)
	{
//		printd (DEBUG, "%c - %d", msg[i], (gint) msg[i]);
		//escape
		if (((gint) msg[i]) == 27)
		{
			//clear sequence: ESC[2J
			if (((i + 3) < len) && (msg[i + 1] == '[') &&
				(msg[i + 2] == '2') && (msg[i + 3] == 'J'))
			{
				_e2_output_clear ();
				return;
			}
			
		}
		//carriage return
		else if (((gint) msg[i]) == '\r')
			line_back = TRUE;
		//backspace
		else if (((gint) msg[i]) == '\b')
		{
			//yes, there are backspaces in the middle
			backspaces_left = TRUE;
			gint j = 1;
			//find a character back in the string that is not
			//already a backspace
			while ((j <= len) && (msg[i - j] == '\b'))
				j++;
			//and overwrite it with backspace
			if (j <= i)
				msg[i - j] = '\b';
			//if there is no non-backspace character delete one 
			//from the previous message
			else
				back++;
		}
	}

	if (backspaces_left)
	{
		free_message = TRUE;
		//replace backspaces with nothing
		//FIXME: is there a faster way to do that?
		msg = e2_utils_str_replace (msg, "\b", NULL);
	}

	//ensure that there's a context set
	if (origin == NULL)
		origin = "default";	//no need to translate

	//name of the mark that saves the end of the context's output
	gchar *mark_end_name = g_strconcat (origin, "-end", NULL);	//no need to translate
	//is this context new
	gboolean is_new = FALSE;

	//get the text buffer for the output pane
	GtkTextBuffer *buffer = app.output.buffer;
	GtkTextIter start, end;

	//DEBUG thread protection certainly helps to reduce crashes
	//when there's a lot of fast output
//	gdk_threads_enter ();
	//DEBUG this helps to reduce crashes, but not enough by itself
//	WAIT_FOR_EVENTS;

	//try to get insert mark
	//it exists if there has been output in this context before
	GtkTextMark *mark = gtk_text_buffer_get_mark (buffer, origin);
	//if not, create it at the end
	if (mark == NULL)
	{
		is_new = TRUE;
		gtk_text_buffer_get_end_iter (buffer, &end);
		mark = gtk_text_buffer_create_mark (buffer, origin, &end, TRUE);
	} else
	{
		//the default context always outputs to the end of the textview
		//(ie the output is not "glued together"
		if (g_str_equal (origin, "default"))	//no need to translate
		{
			gtk_text_buffer_get_end_iter (buffer, &end);
			gtk_text_buffer_move_mark (buffer, mark, &end);
		}
		else
			gtk_text_buffer_get_iter_at_mark (buffer, &end, mark);
	}
	//move the scroll helper mark
	//(it is used by the scroll callback to find out if the user is
	//following output)
	gtk_text_buffer_move_mark (buffer, app.output.mark, &end);

	gtk_text_buffer_place_cursor (buffer, &end);

	//do we have to overwrite a previous message's characters in front of us?
	//(if a previous message had a carriage return '\r' in it)
	GtkTextMark *mark_del = gtk_text_buffer_get_mark (buffer, mark_end_name);
	if (mark_del != NULL)
	{
		GtkTextIter iter_del;
		gtk_text_buffer_get_iter_at_mark (buffer, &iter_del, mark_del);
		gtk_text_buffer_delete (buffer, &end, &iter_del);
	}
	//do we have to delete a previous message's characters behind us?
	//(from backspaces at the start of this message)
	if (back > 0)
	{
		gint _offset = gtk_text_iter_get_offset (&end);
		GtkTextIter backi;
		gtk_text_buffer_get_iter_at_offset (buffer, &backi, _offset - back);
		gtk_text_buffer_delete (buffer, &backi, &end);
		gtk_text_buffer_get_iter_at_offset (buffer, &end, _offset - back);
	}
	//delete the delete mark, it's only used one time
	if (mark_del != NULL)
		gtk_text_buffer_delete_mark (buffer, mark_del);

	//find out if it is necessary to insert a newline character before
	//the message because it is has a different context from the last one
	//and the last one hasn't printed one yet;
	//save the origin of the last message
	static gchar *origin_buffer = NULL;
	if (origin_buffer != origin)
	{
		if ((!gtk_text_iter_starts_line (&end)) &&
			(is_new || (g_str_equal (origin, "default"))))	//no need to translate
		{
			gtk_text_buffer_insert  (buffer, &end, "\n", 1);
			GtkTextMark *mark_end = gtk_text_buffer_get_insert (buffer);
			gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
		}
	}

	if (newline && (!gtk_text_iter_starts_line (&end)))
	{
		printd (DEBUG, "inserted additional newline because the message requested it");
		gtk_text_buffer_insert  (buffer, &end, "\n", 1);
		GtkTextMark *mark_end = gtk_text_buffer_get_insert (buffer);
		gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
	}
	origin_buffer = origin;

	//now we're done parsing

	//check if message is already utf8
	if (g_utf8_validate (msg, -1, NULL))
		//if so, insert it now
		gtk_text_buffer_insert  (buffer, &end, msg, -1);
	else
	{
		//convert to utf before inserting
		gchar *utf = e2_utf8_from_locale (msg); 
		gtk_text_buffer_insert  (buffer, &end, utf, -1);
		g_free (utf);
	}

	if (free_message)
		g_free (msg);

	//get end of last insert
	GtkTextMark *mark_end = gtk_text_buffer_get_insert (buffer);
	gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
	if (newline)
	{
		gtk_text_buffer_insert  (buffer, &end, "\n", 1);
		mark_end = gtk_text_buffer_get_insert (buffer);
		gtk_text_buffer_get_iter_at_mark (buffer, &end, mark_end);
	}
	gtk_text_buffer_get_iter_at_mark (buffer, &start, mark);
	
	//if there was a carriage return, save a delete mark for the next message
	if ((line_back) && (!is_new))
		gtk_text_buffer_create_mark (buffer, mark_end_name, &end, TRUE);
	else
		gtk_text_buffer_move_mark (buffer, mark, &end);

	g_free (mark_end_name);

	//scroll down if necessary
	if (_e2_option_bool_get (app.output.opt_jump) //scrolling to new output is in effect
		&& (app.output.onscreen //manual scrolling has not moved the insert mark off-screen
				|| !_e2_option_bool_get (app.output.opt_jump_follow)) //scroll regardless of whether manually scrolled away
		)
	{
		GtkTextIter iter;
		gtk_text_buffer_get_iter_at_mark (buffer, &iter, mark_end);
		if (
			//print is at the end of the buffer
			gtk_text_iter_is_end (&iter)
			//don't care whether or not this print is at the end of the buffer
			|| !_e2_option_bool_get (app.output.opt_jump_end))
		{
			if (_e2_output_iter_offscreen (app.output.text, &iter))
				gtk_text_view_scroll_to_mark (app.output.text, mark_end,
					0.0, TRUE, 0.1, 1.0);
		}
	}

/*	FIXME when there is a context after the current one, a font like that of
	the later one (eg bold) is sometimes (eg current has error then normal)
	used instead of the proper font (normal)
	might be gtk bug as the font color is correct, but its weight is not */

	if (first_tag == NULL)
	return;

	//process and apply style-tags (if any) for this string
	va_list args;
	va_start (args, first_tag);
	const gchar *tag_name;
	tag_name = first_tag;
	while (tag_name)
	{
		GtkTextTag *tag = gtk_text_tag_table_lookup (buffer->tag_table,
			tag_name);

		if (tag == NULL)
		{
			g_warning ("%s: no tag with name '%s'!", G_STRLOC, tag_name);
			tag_name = va_arg (args, const gchar*);
			continue;
		}

		gtk_text_buffer_apply_tag (buffer, tag, &start, &end);

		tag_name = va_arg (args, const gchar*);
	}
	va_end (args);

//	gdk_threads_leave ();
}
/**
 * @brief
 *
 *
 *
 * @param rt runtime struct to work on
 *
 * @return
 **/
void e2_output_update_style (E2_OutputRuntime *rt)
{
	gtk_text_view_set_wrap_mode (rt->text,
		e2_option_int_get ("output-wrap-mode")); 
	gtk_text_view_set_left_margin (rt->text,
		e2_option_int_get ("output-left-margin"));
	gtk_text_view_set_right_margin (rt->text,
		e2_option_int_get ("output-right-margin"));
	//set font
	PangoFontDescription *font_desc = pango_font_description_from_string
		(e2_option_str_get ("output-font"));
	rt->font_size = pango_font_description_get_size (font_desc);
	GtkTextTagTable *table = gtk_text_buffer_get_tag_table (rt->buffer);
	GtkTextTag *tag = gtk_text_tag_table_lookup (table, "small");
	if (tag != NULL)
	{
		g_object_set (G_OBJECT (tag), "size",
			(gint) (PANGO_SCALE_SMALL * rt->font_size), NULL);
	} else
	{
		gtk_text_buffer_create_tag (rt->buffer, "small", "size",
			(gint) (PANGO_SCALE_SMALL * rt->font_size), NULL);
	}
	gtk_widget_modify_font (GTK_WIDGET (rt->text), font_desc);
	pango_font_description_free (font_desc);
	tag = gtk_text_tag_table_lookup (table, "red");
	if (tag != NULL)
	{
		g_object_set (G_OBJECT (tag), "foreground",
			e2_option_str_get ("color-negative"), NULL);
	} 
	tag = gtk_text_tag_table_lookup (table, "grey");
	if (tag != NULL)
	{
		g_object_set (G_OBJECT (tag), "foreground",
			e2_option_str_get ("color-unimportant"), NULL);
	} 
}
/**
 * @brief create output pane
 *
 * Create output pane with textview and buffer and some basic tags.
 *
 * @param rt runtime struct to work on
 *
 * @return scrolled window with a textview
 **/
GtkWidget *e2_output_create (E2_OutputRuntime *rt)
{
	//visible flag is set elswhere, depending on cache data
	//init some vars
	//at the outset, assume that the current content is on-screen
	rt->onscreen = TRUE;
	rt->mark = NULL;

	//create scrolled window
	rt->scroll = e2_widget_get_sw
		(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (rt->scroll),
		GTK_SHADOW_OUT);

	//create text view
	rt->text = GTK_TEXT_VIEW (gtk_text_view_new ());
	gtk_container_add (GTK_CONTAINER (rt->scroll), GTK_WIDGET (rt->text));
	gtk_text_view_set_editable (rt->text, FALSE);
	gtk_text_view_set_cursor_visible (rt->text, FALSE);
//	allow focus so popup menu signal can happen, & can select text in the pane
//	GTK_WIDGET_UNSET_FLAGS (rt->text, GTK_CAN_FOCUS);

	//signal used for "links" in the output pane
//	g_signal_connect (G_OBJECT (rt->text), "motion-notify-event",
//		G_CALLBACK (test_cb), NULL);
	gtk_widget_set_events (GTK_WIDGET (rt->text),
		gtk_widget_get_events (GTK_WIDGET (rt->text))
		| GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK);
	g_signal_connect (G_OBJECT (rt->text), "popup_menu",
		G_CALLBACK (_e2_output_popup_menu_cb), rt);
	g_signal_connect (G_OBJECT (rt->text), "button_press_event",
		G_CALLBACK (_e2_output_output_pane_clicked_cb), rt);
	g_signal_connect_after (G_OBJECT (rt->text->vadjustment), "value-changed",
		G_CALLBACK (_e2_output_scrolled_cb), rt);
	gtk_widget_show (GTK_WIDGET (rt->text));
	
	rt->buffer = gtk_text_view_get_buffer (rt->text);

	e2_output_update_style (rt);

	//create some default output tags
	gtk_text_buffer_create_tag (rt->buffer, "blue", "foreground", "blue", NULL);
	gtk_text_buffer_create_tag (rt->buffer, "red", "foreground",
		e2_option_str_get ("color-negative"), NULL);
	gtk_text_buffer_create_tag (rt->buffer, "grey", "foreground", 
		e2_option_str_get ("color-unimportant"), NULL);
	gtk_text_buffer_create_tag (rt->buffer, "bold", "weight", PANGO_WEIGHT_BOLD, NULL);
	gtk_text_buffer_create_tag (rt->buffer, "ul", "underline", PANGO_UNDERLINE_SINGLE, NULL);
//	gtk_text_buffer_create_tag (rt->buffer, "small", "size", , NULL);
	gtk_text_buffer_create_tag (rt->buffer, "link", "foreground", "blue",
		"underline", PANGO_UNDERLINE_NONE, NULL);
	gtk_text_buffer_create_tag (rt->buffer, "link-active", "foreground", "blue",
		"underline", PANGO_UNDERLINE_SINGLE, NULL);

	GtkTextIter end;
	gtk_text_buffer_get_end_iter (rt->buffer, &end);
	rt->mark = gtk_text_buffer_create_mark (rt->buffer,
		"internal-end-mark", &end, FALSE);

	//attach options
	rt->opt_wrap = e2_option_attach_value_changed ("output-wrap-mode",
		GTK_WIDGET (rt->text), _e2_output_set_op_wrap, rt);
	rt->opt_show_on_new = e2_option_get ("show-output-window-on-output");
	rt->opt_show_on_focus_in = e2_option_get ("command-line-show-output-on-focus-in");
	rt->opt_hide_on_focus_out = e2_option_get ("command-line-hide-output-on-focus-out");
	rt->opt_jump = e2_option_get ("output-jump-new");
	rt->opt_jump_follow = e2_option_get ("output-jump-new-following");
	rt->opt_jump_end = e2_option_get ("output-jump-new-end");

	return rt->scroll;
}
/**
 * @brief register actions related to output pane
 *
 * @return
 **/
void e2_output_actions_register (void)
{
	gchar *action_name = g_strconcat(_A(8),".",_A(29),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_clear, NULL, FALSE);
	action_name = g_strconcat(_A(8),".",_A(58),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_print_action, NULL, TRUE);
	action_name = g_strconcat(_A(8),".",_A(45),NULL);  //output.help
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_help_action, NULL, TRUE);
	action_name = g_strconcat(_A(8),".",_A(61),NULL);  //output.scrolldown
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_scroll_step, GINT_TO_POINTER (TRUE), TRUE);
	action_name = g_strconcat(_A(8),".",_A(62),NULL);  //output.scrollup
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_scroll_step, GINT_TO_POINTER (FALSE), TRUE);
	action_name = g_strconcat(_A(8),".",_A(55),NULL); //output.pagedown
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_scroll_page, GINT_TO_POINTER (TRUE), TRUE);
	action_name = g_strconcat(_A(8),".",_A(56),NULL); //output.pageup
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,	
		_e2_output_scroll_page, GINT_TO_POINTER (FALSE), TRUE);
	action_name = g_strconcat(_A(8),".",_A(43),NULL); //output.goto bottom
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_scroll_all, GINT_TO_POINTER (TRUE), TRUE);
	action_name = g_strconcat(_A(8),".",_A(44),NULL); //output.goto top
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM,
		_e2_output_scroll_all, GINT_TO_POINTER (FALSE), TRUE);
}
/**
 * @brief register options related to output pane
 *
 * @return
 **/
void e2_output_options_register (void)
{
//CHECKME = strdup's for group, to support session-end free's
	gchar *group_name = g_strconcat(_C(8),".",_C(25),":",_C(23),NULL); //_("commands.output:miscellaneous"
	e2_option_bool_register ("show-output-window-on-output", group_name,
		_("show output pane when a new message appears"),
		_("This will ensure you don't miss any messages"), NULL, FALSE,
		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP);
	e2_option_bool_register ("command-line-show-output-on-focus-in", group_name,
		_("show output pane if the command line is focused"),
		_("This causes the output pane to be opened when you are about enter a command"),
		NULL, FALSE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("command-line-hide-output-on-focus-out", group_name,
		_("hide output pane if the command line is unfocused"), 
		_("This causes the output pane to be closed when you move focus away from the command line"),
		NULL, FALSE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("output-jump-new", group_name, _("scroll to new output"),
		_("This will automatically scroll the output pane content, to show new message(s)"), NULL, TRUE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("output-jump-new-following", group_name, _("only scroll when really following"),
		_("This stops automatic pane scrolling to new output if you've manually scrolled away"),
		"output-jump-new", TRUE,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_bool_register ("output-jump-new-end", group_name, _("only scroll when new output is at the end"),
		_("This stops pane scrolling if a different process has displayed text after the current insert position"),
  		"output-jump-new", FALSE,
		E2_OPTION_FLAG_ADVANCED);
	gchar *opt_wrap_mode[] = {_("none"), _("everywhere"), _("words"), NULL};
	group_name = g_strconcat(_C(8),".",_C(25),":",_C(35),NULL);	//_("commands.output:style"
	e2_option_sel_register ("output-wrap-mode", group_name, _("line wrap mode"),
		_("If mode is 'none', a horizontal scrollbar will be available. Mode 'words' will only break the line between words"),
		NULL, 2, opt_wrap_mode,
		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREEGROUP | E2_OPTION_FLAG_BUILDALL);
	e2_option_int_register ("output-left-margin", group_name, _("left margin (pixels)"),
		_("This is the left margin between the output-pane edge and the text in it"),
		NULL, 6, 0, 1000,
		E2_OPTION_FLAG_ADVANCED);
	e2_option_int_register ("output-right-margin", group_name, _("right margin (pixels)"),
		_("This is the right margin between the output-pane edge and the text in it"),
		 NULL, 2, 0, 1000,
		E2_OPTION_FLAG_ADVANCED);
}
