/* $Id: e2_output.h 338 2005-03-04 09:05:37Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef E2_OUTPUT_H
#define E2_OUTPUT_H

#include "e2_option.h"

typedef enum
{
	E2_OUTPUT_NEWLINE           = 1<<1,
	E2_OUTPUT_PREPEND_NEWLINE   = 1<<2,
	E2_OUTPUT_STYLE_ERROR       = 1<<3,
	E2_OUTPUT_STYLE_LINK        = 1<<4,
	E2_OUTPUT_STYLE_IMPORTANT   = 1<<5,
	E2_OUTPUT_STYLE_UNIMPORTANT = 1<<6,
	E2_OUTPUT_DEBUG             = 1<<7,
	E2_OUTPUT_LINE_FEED         = 1<<8,
} E2_OUTPUT_PRINT_FLAGS;

typedef struct _E2_OutputRuntime
{
	GtkWidget *scroll;	//scrolled window containing the textview
	GtkTextView *text;
	GtkTextBuffer *buffer;
	gboolean visible;	//whether output pane is visible
	gint font_size;
	E2_OptionSet *opt_wrap;
	E2_OptionSet *opt_show_on_new;
	E2_OptionSet *opt_show_on_focus_in;
	E2_OptionSet *opt_hide_on_focus_out;
	gboolean showhide_blocked;
	E2_OptionSet *opt_jump;	//option for scrolling to new output
	E2_OptionSet *opt_jump_follow;	//option for scrolling to new output when not manually scrolled away
	E2_OptionSet *opt_jump_end;	//option for scrolling only when new output is into the last context in the textbuffer
	GtkTextMark *mark;	//scroll helper mark in the text buffer
	gboolean onscreen;	//whether scroll mark is presently visible in the textview
	GHashTable *origins;	//table of textbuffer contexts & data
} E2_OutputRuntime;

typedef struct _E2_OutputOrigin
{
	gchar *name;
	GtkTextMark *mark_start;
	GtkTextMark *mark_insert_start;
	GtkTextMark *mark_insert_end;
	GtkTextMark *mark_end;
	GString *str;
	glong num_in_str;
	glong num_in_buffer;
} E2_OutputOrigin;

#define E2_ERRORTAGS "red", "bold"
#define status_message(msg) e2_output_print (msg, NULL, FALSE, FALSE, FALSE, NULL);
#define status_error(msg) e2_output_print (msg, NULL, FALSE, FALSE, TRUE, E2_ERRORTAGS, NULL);
//#define status_errno() e2_output_print ((gchar *) g_strerror (errno), NULL, FALSE, FALSE, TRUE, E2_ERRORTAGS, NULL);
#define e2_output_print_errno() e2_output_print ((gchar *) g_strerror (errno), NULL, FALSE, FALSE, TRUE, E2_ERRORTAGS, NULL);
#define status_messagef(msg, first_tag, args...) e2_output_print (msg, NULL, FALSE, FALSE, FALSE, first_tag, ## args);
#define status_message2(msg, debug, first_tag, args...) e2_output_print (msg, NULL, FALSE, debug, FALSE, first_tag, ## args);
#define e2_output_print_error(msg) e2_output_print (msg, NULL, FALSE, FALSE, FALSE, E2_ERRORTAGS, NULL);
#define e2_output_print_default(msg) e2_output_print (msg, NULL, FALSE, FALSE, FALSE, NULL);
//#define status_error(msg) e2_output_print (msg, NULL, TRUE, FALSE, TRUE, NULL);
//#define status_errno(msg) e2_output_print ((gchar *) g_strerror (errno), NULL, TRUE, FALSE, TRUE, NULL);
//#define e2_output_print_errno() e2_output_print ((gchar *) g_strerror (errno), NULL, TRUE, FALSE, TRUE, NULL);
//#define e2_output_print_error(msg) e2_output_print (msg, NULL, TRUE, FALSE, FALSE, NULL);

void e2_output_print (gchar *msg, gchar *origin, gboolean error,
	gboolean debug, gboolean newline, const gchar *first_tag, ...);
void e2_output_print2 (const gchar *msg, const gchar *origin,
	E2_OUTPUT_PRINT_FLAGS flags, const gchar *first_tag, ...);
void toggle_output_window_fs (GtkWidget *widget);
gboolean set_output_window_fs (GtkWidget *widget);
gboolean unset_output_window_fs (GtkWidget *widget);

void e2_output_update_style (E2_OutputRuntime *rt);
GtkWidget *e2_output_create (E2_OutputRuntime *rt);
void e2_output_actions_register (void);
void e2_output_options_register (void);

#endif //ndef E2_OUTPUT_H
