/* $Id: e2_pane.c 1064 2005-08-22 04:52:22Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 /**
 * @file src/e2_pane.c 
 * @brief pane creation and action functions
 *
 * includes actions on pane contents, but not on panes per se
 *
 **/

#include "emelfm2.h"
//#include <unistd.h>
#include "e2_pane.h"
#include "e2_action.h"
#include "e2_dialog.h"
#include "e2_context_menu.h"
#include "e2_filelist.h"

static void _e2_pane_change_dir (E2_PaneRuntime *rt,
	gchar *path, gboolean history, gboolean hook);

  /*****************/
 /***** utils *****/
/*****************/
/**
 * @brief setup some properties for the current pane view, to signal to the user that it's current
 * This works on curr_view.
 * The "active-pane-signal" config option is checked, and either the column headers will be
 * re-coloured, or the filename column title will be set to bold, or the treeview will be made sensitive
 * 
 * @return
**/
static void _e2_pane_flag_active (void)
{
	E2_OptionSet *set = e2_option_get ("active-pane-signal");
	gint htype = _e2_option_int_get (set);
	switch (htype)
	{
		case 1:
		{  //set bold Filename-column header title
			gchar *new_title = g_strconcat ("<b>", gettext (e2_all_columns[FILENAME].title),
				"</b>", NULL);
			GtkLabel *label = curr_view->name_label;
			gtk_label_set_markup (label, new_title);
			g_free (new_title);
			break;
		}
		case 2:
		{  //make active pane sensitive
			gtk_widget_set_sensitive (curr_view->treeview, TRUE);
			break;
		}
		default:
		{
			//set all columns' header color
			GList *cols = gtk_tree_view_get_columns (GTK_TREE_VIEW (curr_view->treeview));
			GList *base = cols;
			GtkTreeViewColumn *col;
			GdkColor *active_btncolor = e2_option_color_get ("color-active-pane");
			for (; cols !=NULL; cols=cols->next)
			{
				col = cols->data;
				gtk_widget_modify_bg (col->button, GTK_STATE_NORMAL, active_btncolor);
			}
			g_list_free (base);
			break;
		}
	}
}
/**
 * @brief setup some properties for the other pane view, to signal to the user that it's not current
 * This works on other_view, but gets the new color from a button in the current_view, 
 * so it needs to be called before the color there is updated
 * 
 * @return
**/
static void _e2_pane_flag_inactive (void)
{
	E2_OptionSet *set = e2_option_get ("active-pane-signal");
	gint htype = _e2_option_int_get (set);
	switch (htype)
	{
		case 1:
		{  //set normal Filename-column header title
			gchar *new_title = gettext (e2_all_columns[FILENAME].title);
			GtkLabel *label = other_view->name_label;
			gtk_label_set_text (label, new_title);
			break;
		}
		case 2:
		{  //make inactive pane insensitive
			gtk_widget_set_sensitive (other_view->treeview, FALSE);
			break;
		}
		default:
		{
			//revert all columns' header color
			GList *cols = gtk_tree_view_get_columns (GTK_TREE_VIEW (curr_view->treeview));
			GtkTreeViewColumn *col = cols->data;
			GdkColor inactive_panecolor = col->button->style->bg[GTK_STATE_NORMAL];
			g_list_free (cols);
			cols = gtk_tree_view_get_columns (GTK_TREE_VIEW (other_view->treeview));
			GList *base = cols;
			for (; cols !=NULL; cols=cols->next)
			{
				col = cols->data;
				gtk_widget_modify_bg (col->button, GTK_STATE_NORMAL, &inactive_panecolor);
			}
			g_list_free (base);
			break;
		}
	}
}
/**
 * @brief set filepanes' properties to indicate which one is active
 * 
 * @return
**/
void e2_pane_flag_active (void)
{
	//inactive pane must be processed before active, to get the recoloring righg
	_e2_pane_flag_inactive ();
	_e2_pane_flag_active ();
}

/**
 * @brief timeout function for checking completion of fileview creation
 *
 * @param userdata data specified when the timer was initialised, here, always NULL, unused
 *
 * @return TRUE until update is completed, then FALSE
 */
/*gboolean _e2_pane_views_complete (gpointer userdata)
{
	if (curr_view->completed && other_view->completed)
	{
		e2_fileview_switch_views ();
		gtk_widget_grab_focus (app.pane_current->focus_widget);
		e2_hook_list_run (&app.hook_pane_focus_changed, app.pane_current);
		return FALSE;
	}
	printd (DEBUG, "finished waiting for fileview completion");
	return TRUE;  //continue waiting
}
*/
  /*******************/
 /***** actions *****/
/*******************/

/**
 * @brief change active pane from the current one to the other one
 *
 * @return
**/
void e2_pane_activate_other (void)
{
//	printd (DEBUG, "e2_pane_activate_other ()");
	E2_PaneRuntime *tmp = app.pane_other;
	app.pane_other = app.pane_current;
	app.pane_current = tmp;
	//DEBUG try this
//	if (curr_view->completed && other_view->completed)
//	{
//		printd (DEBUG, "NOT waiting for fileview completion");
		e2_fileview_switch_views ();
		gtk_widget_grab_focus (tmp->focus_widget);
		e2_hook_list_run (&app.hook_pane_focus_changed, tmp);
/*	}
	else
	{  //wait until all is done
		printd (DEBUG, "start waiting for fileview completion");
		g_timeout_add (200, _e2_pane_views_complete, NULL);
	} */
}
/**
 * @brief change the directory for a specified pane
 *
 * This is a wrapper for _e2_pane_change_dir (), with updqte of
 * pane history list and hooklist
 *
 * @param rt pointer to pane data struct, or NULL for current pane
 * @param path utf string containing path to directory to be opened
 *
 * @return
**/
void e2_pane_change_dir (E2_PaneRuntime *rt, gchar *path)
{
	_e2_pane_change_dir (rt, path, TRUE, TRUE);
}
/**
 * @brief change directory displayed in a specified pane
 *
 * ~ at start of path is interpreted to $HOME
 * redundant separators are removed, and one is added to the end,
 * if not already there
 * Does nothing if the path string is empty, shows error msg if
 * path is unreachable
 * The supplied path string is unchanged.
 * New path is stored in heap @ rt->path
 * New path is also stored in array @ rt->view->dir
 * History list is updated if @a no-update is FALSE
 *
 * @param rt  pointer to pane data struct, or NULL for current pane
 * @param path utf-8 string with the path to open
 * @param history boolean TRUE to update pane history list
 * @param hook boolean TRUE to update hooklist
 *
 * @return
 */
static void _e2_pane_change_dir (E2_PaneRuntime *rt, 
	gchar *path, gboolean history, gboolean hook)
{
	printd (DEBUG, "_e2_pane_change_dir (rt:,path:%s,update:%d,hook:%d)",
		path, history, hook);

	if ((path == NULL) || (*path == '\0'))
		return;

	//work on currently active file pane if argument is NULL
	if (rt == NULL)
		rt = app.pane_current;

	path = g_strdup (path);

	g_strstrip (path);

	//quick exit
	if (*path == '\0')
	{
		g_free (path);
		return;
	}

	gchar *freeme;

	//home dir check
	if (path[0] == '~')
	{
		freeme = path;
		if (path[1] == '\0')
		{
			path = g_strdup (g_get_home_dir ());
			g_free (freeme);
		}
		else
			if (path[1] == G_DIR_SEPARATOR)
		{
			path = g_strconcat (g_get_home_dir (), path + 1, NULL);
			g_free (freeme);
		}
	}

#ifndef FILES_UTF8ONLY
	if (!cfg.utf8_filenames)
	{
		//this test not needed with only-utf8 file coding
		if (! g_utf8_validate (path, -1, NULL))
		{
			freeme = path;
			path = S_FILENAME_FROM_LOCALE (path);
			g_free (freeme);
		}
	}
#endif
	//these fns both ensure a trailing /, which gets into rt->path
	//(see below) and onto dir lines
	if (_e2_option_bool_get (rt->opt_transparent))
	{	//relative path segment(s) conversion
		//clean and convert the path string, if needed
		freeme = path;
		path = e2_utils_translate_relative_path (rt->path, path);
		g_free (freeme);
	}  // otherwise, just make sure the user didn't enter a 'dirty' path string
	else
		path = e2_utils_path_clean (path);

	if (! e2_fs_cd_isok (path) //can't go there
		|| ! e2_fs_chdir (path)) //actual go-there failed
	{
		g_free (path);
		return;
	}

	//update the path store
#ifdef E2_FAM
	gchar *oldpath = rt->path;
#else
	g_free (rt->path);
#endif
	rt->path = path;

	//create the new filelist
	e2_filelist_change_dir (rt->view, path);

#ifdef E2_FAM
	//startup change monitoring for the new dir
	e2_fs_FAM_change (oldpath, rt);
	g_free (oldpath);
#endif

	//update the pane forward-back list, if appropriate
	if (history)
	{
		if (rt->history_cur < g_list_length (rt->history))
		{
			GList *free = g_list_nth (rt->history, rt->history_cur);
			GList *last = g_list_nth (rt->history, rt->history_cur - 1);
			if (last == NULL)
			{
				rt->history = NULL;
			} else
				last->next = NULL;
			e2_list_free_with_data (&free);
		}
		e2_list_update_history (path, &rt->history,
			&rt->history_cur, 30, TRUE);
	}
	//update the pane hooklist, if appropriate
	if (hook)
		e2_hook_list_run (&rt->hook_change_dir, rt->path);
}
/**
 * @brief for a specified pane, open the directory @a number places forward in the history list
 *
 * The last entry in the list will be used if we try to go forward too far
 *
 * @param rt  pointer to pane data struct
 * @param number integer no.of places in the history list to go forward
 * 
 * @return
**/
static void _e2_pane_go_forward (E2_PaneRuntime *rt, gint number)
{
	printd (DEBUG, "_e2_pane_go_forward (rt:,number:%d", number);
	gint old = rt->history_cur;
	gint len = g_list_length (rt->history);
	rt->history_cur += number;
	if (rt->history_cur > g_list_length (rt->history))
		rt->history_cur = len;
	if (old != rt->history_cur)
	{
//		F_DECLARE;
//		gchar *dir = F_FILENAME_TO_LOCALE (
		gchar *dir =
			g_list_nth_data (rt->history, rt->history_cur - 1);
		_e2_pane_change_dir (rt, dir, FALSE, TRUE);
//		F_FREE (dir);
	}
}
/**
 * @brief goto the next dir in the history list for a specified pane
 *
 * @param rt  pointer to pane data struct, or NULL for active pane
 * 
 * @return
**/
static void e2_pane_go_forward (E2_PaneRuntime *rt)
{
	if (rt == NULL)
		rt = app.pane_current;
	_e2_pane_go_forward (rt, 1);
}
/**
 * @brief for a specified pane, open the directory @a number places back in the history list
 *
 * The first entry in the list will be used if we try to go back too far
 *
 * @param rt  pointer to pane data struct
 * @param number integer no.of places in the history list to go back
 * 
 * @return
**/
static void _e2_pane_go_back (E2_PaneRuntime *rt, gint number)
{
	printd (DEBUG, "_e2_pane_go_back (rt:,number:%d", number);
	gint old = rt->history_cur;
	rt->history_cur -= number;
	if (rt->history_cur < 1)
		rt->history_cur = 1;
	if (old != rt->history_cur)
	{
//		F_DECLARE;
//		gchar *dir = F_FILENAME_TO_LOCALE (
		gchar *dir =
			g_list_nth_data (rt->history, rt->history_cur - 1);
		_e2_pane_change_dir (rt, dir, FALSE, TRUE);
//		F_FREE (dir);
	}
}
/**
 * @brief goto the previous dir in the history list for a specified pane
 *
 * @param rt  pointer to pane data struct, or NULL for current pane
 * 
 * @return
**/
static void e2_pane_go_back (E2_PaneRuntime *rt)
{
	if (rt == NULL)
		rt = app.pane_current;
	_e2_pane_go_back (rt, 1);
}
/**
 * @brief for a specified pane, open the parent of the current directory
 *
 * @param rt pointer to pane data struct, or NULL for current pane
 * 
 * @return
**/
static void _e2_pane_go_up (E2_PaneRuntime *rt)
{
	if (rt == NULL)
		rt = app.pane_current;
	e2_pane_change_dir (rt, "..");
}
/**
 * @brief show a chosen directory in a pane
 *
 * @param rt pointer to pane data struct, or NULL to determine the pane
 * @param actionrt_data UNUSED
 * @param from pointer to widget where the action was initiated, or NULL
 *
 * @return
**/
void e2_pane_goto_choice (E2_PaneRuntime *rt, gpointer actionrt_data, GtkWidget *from)
{
	//make sure we know the relevant pane runtime and commandline runtime
	E2_CommandLineRuntime *clrt;
	GList *line;
	for (line = cfg.command_lines; line != NULL ; line = g_list_next (line))
	{
		clrt = line->data;
		if (! clrt->original	//this is a dir line
			&& (GTK_BIN (clrt->cl)->child == from //processing a keybinding, from != NULL
					|| clrt->pane == rt) //processing a mouse-click, rt != NULL
		)
		{
			rt = clrt->pane;
			break;
		}
	}
	if (line == NULL)	//OOPS !!
		return;	//can't do anything

	GtkWidget *entry = GTK_BIN (clrt->cl)->child;
	//create relevant path string
	gchar *dir = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
	if (!g_str_has_prefix (dir, G_DIR_SEPARATOR_S))
	{
		gchar *freeme = dir;
		dir = g_strconcat (rt->path, dir, NULL);
		g_free (freeme);
	}
	e2_fs_get_valid_path (&dir, FALSE);

	gchar *newdir = NULL;
	e2_opendir_dialog_create (dir, &newdir);
	if (newdir != NULL)
	{	//something was selected
		e2_pane_change_dir (rt, newdir);
		g_free (newdir);
	}
	else
	{	//user cancelled
		gtk_widget_grab_focus (entry);
		gtk_editable_select_region (GTK_EDITABLE (entry),
			-1, -1);	//unselect the entry contents
	}
	g_free (dir);
}
/**
 * @brief open the last-used trash directory
 *
 * @return
**/
void _e2_pane_goto_trash (void)
{
	const gchar *tp = e2_utils_last_trash_path ();
	if (tp != NULL)
		e2_pane_change_dir (app.pane_current, (gchar *) tp);	//CHECKME tp utf8
}
/**
 * @brief switch panes if the specified pane is not the active one
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
static void _e2_pane_focus_action (E2_PaneRuntime *rt)
{
	if (app.pane_current != rt)
		e2_pane_activate_other ();
}
/**
 * @brief toggle focus between a dir line and the file pane
 *
 * @param data UNUSED pointer to ??
 * @param arg string with "1" (actually, anything but "2") or "2" which specifies the dir line to focus
 * 
 * @return
**/
static void _e2_pane_focus_dirline_action (gpointer data, gchar *arg)
{
	//decide which dirline
	if (arg != NULL)
	{
		g_strstrip (arg);
		if (!g_str_equal (arg, "2"))	//CHECKME no translation ??
			arg = NULL;
	}
	E2_PaneRuntime *pane = (arg==NULL) ? &app.pane1 : &app.pane2;
	//find the first-registered command-line for the pane
	E2_CommandLineRuntime *cl_rt;
	GList *list;
	for (list = cfg.command_lines; list != NULL ; list = g_list_next (list))
	{
		cl_rt = (E2_CommandLineRuntime *) list->data;
		if (!cl_rt->original && cl_rt->pane==pane)
			break;
	}
	if (list == NULL)
		return;	//no dirline found for the pane
	if (GTK_WIDGET_HAS_FOCUS (GTK_BIN (cl_rt->cl)->child))
		gtk_widget_grab_focus (app.pane_current->focus_widget);
	else
	{
		gtk_widget_grab_focus (GTK_BIN (cl_rt->cl)->child);
		gtk_editable_set_position (GTK_EDITABLE (GTK_BIN (cl_rt->cl)->child), -1);
	}
}
/**
 * @brief select all items in specified pane
 * The pane is made active, if it wasn't already
 * @param rt pointer to pane data struct for the pane whose content is to be selected
 * 
 * @return
**/
static void _e2_pane_select_all (E2_PaneRuntime *rt)
{
	if (rt != app.pane_current)
		e2_pane_activate_other ();
	e2_fileview_select_all (NULL, curr_view);
	return;
}
/**
 * @brief toggle selection of all items in active pane
 *
 * This is used for a key-binding
 * 
 * @return
**/
static void _e2_pane_current_select_all (void)
{
	e2_fileview_select_all (NULL, curr_view);
}
/**
 * @brief open pane other pane directory in this pane
 *
 * @param rt pointer to pane data struct for the pane which is to mirror the other one
 * 
 * @return
**/
static void _e2_pane_mirror (E2_PaneRuntime *rt)
{
	E2_PaneRuntime *other = (rt == &app.pane1) ?
		&app.pane2 : &app.pane1;
	e2_pane_change_dir (rt, other->path);
}
/**
 * @brief helper for toggle display of hidden items
 * @param label action-name label for the pane, _A(9) or _A(10)
 * @param view data struct for the view whose content is to be updated
 * @return
**/
static void _e2_pane__toggle_hidden (gchar *label, FileView *view)
{
	view->show_hidden = !view->show_hidden;
	e2_filelist_refresh_list (view);
	//toggle the button
	gchar *action_name = g_strconcat (label,".",_A(72),NULL);
	e2_toolbar_button_toggle (action_name);
	g_free (action_name);
	return;
}
/**
 * @brief toggle display of hidden items in specified pane
 * The toggle-button state is also changed
 * @param rt pointer to pane data struct for the pane to be altered
 * 
 * @return
**/
static void _e2_pane_toggle_hidden (E2_PaneRuntime *rt)
{
	gchar *label = (rt == &app.pane1) ? _A(9) : _A(10) ;
	_e2_pane__toggle_hidden (label, rt->view);
}
/**
 * @brief toggle display of hidden items in active pane
 *
 * This is used for a key-binding
 * 
 * @return
**/
static void _e2_pane_current_toggle_hidden (void)
{
	gchar *label = (app.pane_current == &app.pane1) ? _A(9) : _A(10);
	_e2_pane__toggle_hidden (label, curr_view);
}
/**
 * @brief create and pop up a filters menu for a specified pane
 * The toggle-button state will be changed if appropriate
 * Cannot use a button-click callback to do the menu, as we would 
 * need to connect the same menu to 2 buttons, which is very bad
 * for bar/window refreshes	 
 *
 * @param rt pointer to pane data struct for the pane to be altered
 * 
 * @return
**/
static void  _e2_pane_filter_menu_create (E2_PaneRuntime *rt)
{
	FileView *view = (rt == &app.pane1) ? 
		&app.pane1_view : &app.pane2_view;
	GtkWidget *menu = gtk_menu_new ();
	view->check_name = e2_menu_add_check (menu, _("_Name filter"),
		view->filename_filter.active, e2_name_filter_dialog_create_cb, view);
	view->check_size = e2_menu_add_check (menu, _("_Size filter"),
		view->size_filter.active, e2_size_filter_dialog_create_cb, view);
	view->check_date = e2_menu_add_check (menu, _("_Date filter"),
		view->date_filter.active, e2_date_filter_dialog_create_cb, view);
	e2_menu_add_separator (menu);
	view->check_dirs = e2_menu_add_check (menu, _("_Directories too"),
		view->filter_directories, e2_filelist_filter_dirs_cb, view);
	if (view->filename_filter.active 
		|| view->size_filter.active
		|| view->date_filter.active)
	{
		e2_menu_add_separator (menu);
		e2_menu_add (menu, _("_Remove all filters"), NULL, NULL, 
			e2_filelist_remove_filters_cb, view);
	}

	g_signal_connect (G_OBJECT (menu), "selection-done",
		G_CALLBACK (e2_menu_destroy_cb), NULL);

	//to determine the menu's popup position:
	//first, get the button that was pressed
	gchar *label = (rt == &app.pane1) ? _A(9) : _A(10);
	label = g_strconcat (label,".",_A(22),NULL);
	E2_Action *action = e2_action_get (label);
	E2_ToggleActionEx *ex = action->data2;
	GtkWidget *button = (ex->current_state) ?
		ex->true_button : ex->false_button;

	GtkAllocation menu_position;
	e2_utils_get_abs_pos (button, &(menu_position.x), &(menu_position.y));
	e2_toobar_menu_position_set (button, menu, &menu_position,
		&rt->toolbar);

	gtk_menu_popup (GTK_MENU (menu), NULL, NULL,
		(GtkMenuPositionFunc) e2_toolbar_menu_position, &menu_position, 1, 0);
}

  /******************/
 /***** public *****/
/******************/

/**
 * @brief grab pointers to options previously registered
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
void e2_pane_create_option_data (E2_PaneRuntime *rt)
{
	gchar *option_name = (rt == &app.pane1) ?
		"panebar1" : "panebar2" ;	//do not translate
	rt->set = e2_option_get (option_name);
	rt->opt_transparent = e2_option_get ("transparent-dir-links");
	option_name = g_strconcat (rt->name,"-use-startup-dir",NULL);
	rt->opt_use_startup = e2_option_get (option_name);
	g_free (option_name);
	option_name =  g_strconcat(rt->name,"-use-startup-dir-startup-dir",NULL);
	rt->opt_startup = e2_option_get (option_name);
	g_free (option_name);
	//create dependency
	rt->opt_startup->depends = rt->opt_use_startup->name;
}
/**
 * @brief create file pane and all its contents
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
void e2_pane_create (E2_PaneRuntime *rt)
{
	gint num = (rt == &app.pane1) ? 1 : 2;
	rt->name = g_strdup_printf ("pane%d", num);
	rt->pane = NULL;

	e2_pane_create_option_data (rt);

	e2_cache_str_register (rt->name, &rt->path, G_DIR_SEPARATOR_S);
	//do we need to over-ride the cached startup dir ?
	gchar *panedir = (num == 1) ? e2_cl_options.pane1_path :
		e2_cl_options.pane2_path ;
	if (panedir == NULL 
		&& _e2_option_bool_get (rt->opt_use_startup))
			panedir = _e2_option_str_get (rt->opt_startup);
	if (panedir != NULL)
	{
		g_free (rt->path);
		rt->path = g_strdup (panedir);
	}
	//make sure initial path is valid
	e2_fs_get_valid_path (&(rt->path), TRUE);

	E2_BarType thisbar = (num == 1) ? E2_BAR_PANE1 : E2_BAR_PANE2;
	e2_toolbar_initialise (thisbar);

	//register pane-specific actions (all name strings freed in action creation function)
	gchar *pstr = (num == 1) ? _A(9) : _A(10);
	gchar *action_name = g_strconcat (pstr,".",_A(38), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_focus_action, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(24), NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_COMMAND_LINE,
		e2_command_line_directory_cb, "dir line", FALSE,  //internal default name, not translated
		E2_ACTION_EXCLUDE_MENU | E2_ACTION_EXCLUDE_ACCEL, rt);
	action_name = g_strconcat (pstr,".",_A(41), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_forward, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(40), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_back, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(42), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_go_up, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(51), NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_goto_choice, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(77),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_mirror, rt, FALSE);
	action_name = g_strconcat(pstr,".",_A(72),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_TOGGLE, _e2_pane_toggle_hidden, rt, FALSE);
	action_name = g_strconcat(pstr,".",_A(73),NULL);
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_select_all, rt, FALSE);
	action_name = g_strconcat (pstr,".",_A(21), NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_BOOKMARKS, e2_bookmark_open,
		GINT_TO_POINTER (num), TRUE, E2_ACTION_EXCLUDE_ACCEL, NULL);
	action_name = g_strconcat (pstr,".",_A(22),NULL);
	// NULL function for this, actions handled by filters submenu items with individual callbacks
// 	e2_action_register_simple (action_name, E2_ACTION_TYPE_FILTERS, NULL, NULL, FALSE);
//converted filters item to a toggle
 	e2_action_register_simple (action_name, E2_ACTION_TYPE_TOGGLE,
		_e2_pane_filter_menu_create, rt, FALSE);

	//after the actions are in place, fill in the pane contents
	e2_pane_create_part (rt);
}
/**
 * @brief part of pane creation, used also in re-creation
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
void e2_pane_create_part (E2_PaneRuntime *rt)
{
	g_hook_list_init (&rt->hook_change_dir, sizeof (GHook));

	gchar *history_name = g_strconcat (rt->name, "-history", NULL);
	e2_cache_list_register (history_name, &rt->history);
	if (rt->history != NULL)  //if history list found in cache
		rt->history_cur = g_list_length (rt->history);
	g_free (history_name);

	rt->pane = e2_fileview_create_sw (rt);
	rt->focus_widget = rt->view->treeview;
	rt->outer_box = gtk_hbox_new (FALSE, 0);
	rt->inner_box = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start_defaults (GTK_BOX (rt->inner_box), rt->pane);
	gtk_box_pack_start_defaults (GTK_BOX (rt->outer_box), rt->inner_box);

	gchar *category = g_strconcat(_C(17),".",_C(30),NULL);
	e2_keybinding_register (category, rt->focus_widget);  //_("general.panes
	g_free (category);

	gtk_widget_show (rt->inner_box);
	gtk_widget_show (rt->outer_box);

	e2_toolbar_create (&rt->toolbar);

	if (_e2_option_bool_get (rt->toolbar.show))
	{
		//show full/split window button according to cached pane ratio
		gchar *action_name;
		if (rt == &app.pane1 && app.window.panes_paned_ratio > 0.999)
		{
			action_name = g_strconcat(_A(9),".",_A(74),NULL);
			e2_toolbar_toggle_button_set_state (action_name, TRUE);
			g_free (action_name);
		}
		else if (rt == &app.pane2 && app.window.panes_paned_ratio < 0.001)
		{
			action_name = g_strconcat(_A(10),".",_A(74),NULL);
			e2_toolbar_toggle_button_set_state (action_name, TRUE);
			g_free (action_name);
		}
		//show (un)hide toggle button which matches cached state
		action_name = (rt == &app.pane1) ? _A(9) : _A(10);
		action_name = g_strconcat (action_name,".",_A(72),NULL);
		e2_toolbar_toggle_button_set_state (action_name, rt->view->show_hidden);
		g_free (action_name);
	}
}
/**
 * @brief destroy pane data
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
/*static void _e2_pane_destroy (E2_PaneRuntime *rt)
{
	gchar *history_name = g_strconcat (rt->name, "-history", NULL);
	e2_cache_list_unregister (history_name);
	g_free (history_name);
//	e2_toolbar_destroy (&rt->toolbar);
	//pane toolbar may be outside rt->outer_box, so destroy it independently
	gtk_widget_destroy (rt->toolbar.toolbar_container);
//	gtk_widget_destroy (rt->pane);
	gtk_widget_destroy (rt->outer_box);
	g_hook_list_clear (&rt->hook_change_dir);
}*/
/**
 * @brief destroy and re-create specified pane
 *
 * @param rt pointer to pane data struct
 * 
 * @return
**/
/*void e2_pane_recreate (E2_PaneRuntime *rt)
{
	_e2_pane_destroy (rt);
	e2_pane_create_part (rt);
}*/
/**
 * @brief register pane-related actions that do have a pane-specifc action
 *
 * Actions with pane-specific function are registered in e2_pane_create()
 *
 * @return
**/
void e2_pane_actions_register (void)
{  //these strings are freed during the registration process
	gchar *action_name = g_strconcat(_A(11),".",_A(41),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_forward, NULL, FALSE);
	action_name = g_strconcat(_A(11),".",_A(40),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_go_back, NULL, FALSE);
	action_name = g_strconcat(_A(11),".",_A(42),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_go_up, NULL, FALSE);
	action_name = g_strconcat(_A(11),".",_A(51),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, e2_pane_goto_choice, NULL, FALSE);
//	action_name = g_strconcat(_A(11),".",_A(72),NULL); 
//	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, toggle_show_hidden, NULL, FALSE);
//	action_name = g_strconcat(_A(11),".",_A(64),NULL);
// 	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, fileview_select_all, NULL, FALSE);
	action_name = g_strconcat(_A(11),".",_A(72),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_toggle_hidden, NULL, FALSE);
	action_name = g_strconcat(_A(11),".",_A(73),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_current_select_all, NULL, FALSE);
	action_name = g_strconcat(_A(11),".",_A(66),NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM, e2_context_menu_show_menu_action, NULL, TRUE,
		E2_ACTION_EXCLUDE_MENU, NULL);
	action_name = g_strconcat(_A(6),".",_A(81),NULL);
	e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_goto_trash, NULL, FALSE);
	action_name = g_strconcat(_A(4),".",_A(39),NULL);
	e2_action_register (action_name, E2_ACTION_TYPE_ITEM, _e2_pane_focus_dirline_action, NULL, TRUE,
		E2_ACTION_EXCLUDE_MENU, NULL);
}
/**
 * @brief register pane-related config options
 * Panebar default contents are set here, too
 * @param num integer pane number (1 or 2)
 *
 * @return
**/
void e2_pane_options_register (gint num)
{
	gchar *parent = _C(30);	//_("panes
	gchar *internal_name = g_strdup_printf ("panebar%d", num);
	gchar *public_name = (num == 1) ? _C(26) : _C(27);
	E2_OptionSet *set = e2_option_tree_register (internal_name, public_name, public_name,
		NULL, NULL, NULL, E2_OPTION_TREE_UP_DOWN | E2_OPTION_TREE_ADD_DEL,
		E2_OPTION_FLAG_ADVANCED | E2_OPTION_FLAG_FREENAME | E2_OPTION_FLAG_BUILDBARS);
	e2_option_tree_add_column (set, _("Label"), E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Icon"), E2_OPTION_TREE_TYPE_ICON, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Tooltip"), E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Action"), E2_OPTION_TREE_TYPE_SEL, 0, _A(17),
		e2_actions_fill_store, GINT_TO_POINTER (E2_ACTION_EXCLUDE_TOOLBAR), 0, NULL, NULL);
	e2_option_tree_add_column (set, _("Argument"), E2_OPTION_TREE_TYPE_STR, 0, "",
		NULL, NULL, 0, NULL, NULL);

	e2_option_tree_sync_columns (set);

	if (num == 1)
	{
		_e2_option_tree_prepare_default (set,
			g_strconcat("<",internal_name,NULL), // panebar1
		//NOTE - these 2 btns and their corresponding T/F flags are expected to be present
		// (text/icons can vary) see code for selecting appropriate one of the pair at session start
//			g_strconcat(_("_Panes"),"|gtk-zoom-fit|",_("Toggle other-pane display"),"|",_A(11),".",_A(28),"|","*,1",NULL),	//no arg translation
			g_strconcat(_("_Panes"),"|gtk-zoom-fit|",_("Hide other pane"),"|",_A(9),".",_A(74),"|false",NULL),	//no arg translation
#ifdef E2_PANES_HORIZONTAL
			g_strconcat(_("_Panes"),"|"ICON_DIR"split_vert_"E2IP".png|",
				_("Show other-pane"),"|",_A(9),".",_A(74),"|true",NULL),
#else
			g_strconcat(_("_Panes"),"|"ICON_DIR"split_horiz_"E2IP".png|",
				_("Show other-pane"),"|",_A(9),".",_A(74),"|true",NULL),
#endif
			g_strconcat(_("Show _hidden"),"|"ICON_DIR"hidden_show_"E2IP".png|",
				_("Display hidden items in this directory"),"|",_A(9),".",_A(72),"|false",NULL),
			g_strconcat(_("Hide _hidden"),"|"ICON_DIR"hidden_noshow_"E2IP".png|",
				_("Do not display hidden items in this directory"),"|",_A(9),".",_A(72),"|true",NULL),

			g_strconcat(_("Fil_ters"),"|"ICON_DIR"filter_"E2IP".png|",
//				_("Set/remove rules for the items to be displayed"),"|", _A(18),"|",NULL),
				_("Set rules for the items to be displayed"),"|",_A(9),".",_A(22),"|true",NULL),
			//the 'primary' needs a submenu entry, action name needs to be same as for button item
//			g_strconcat("\t|||",_A(9),".",_A(22),"|",NULL),
			g_strconcat(_("Fil_ters"),"|"ICON_DIR"filter_off_"E2IP".png|",
				_("Set/remove rules for the items to be displayed"),"|",_A(9),".", _A(22),"|false",NULL),
			//the 'alternate' does not need a submenu entry, it uses the same values as the 'primary'

// inconsistent with other toolitems here
//			g_strconcat(_("_Select all"),"|"ICON_DIR"star_"E2IP".png|",
//				_("Toggle select all items"),"|",_A(9),".",_A(73),"|",NULL),  //current pane, not per pane

			g_strconcat("|||",_A(9),".",_A(24),"|,",_A(89),NULL),

			g_strconcat(_("_Marks"),"|"ICON_DIR"bookmark_"E2IP".png|",
				_("Bookmarks"),"|",_A(18),"|",NULL),
			g_strconcat("\t",_("Add _top"),"||",_("Add the current directory to the top of the bookmarks list"),
				"|",_A(0),".",_A(27),"|",_A(9),",",_A(93),NULL),
			g_strconcat("\t|||",_A(9),".",_A(21),"|",NULL),
			g_strconcat("\t",_("Add _bottom"),"||",_("Add the current directory to the bottom of the bookmarks list"),
				"|",_A(0),".",_A(27),"|",_A(9),NULL),
			g_strconcat("\t|||",_A(17),"|",NULL),
			g_strconcat("\t",_("_Edit bookmarks"),"||",
			_("Open the bookmarks configuration dialog"),"|",_A(2),".",_C(1),"|",NULL),

#ifdef E2_PANES_HORIZONTAL
			g_strconcat(_("Mi_rror"),"|"ICON_DIR"mirror_vert_"E2IP".png|",
				_("Go to directory shown in other pane"),"|",_A(9),".",_A(77),"|",NULL),
#else
			g_strconcat(_("Mi_rror"),"|"ICON_DIR"mirror_horiz_"E2IP".png|",
				_("Go to directory shown in other pane"),"|",_A(9),".",_A(77),"|",NULL),
#endif
			g_strconcat(_("_Back"),"|gtk-go-back|",_("Go to previous directory in history"),"|",_A(9),".",_A(40),"|",NULL),
			g_strconcat(_("_Up"),"|gtk-go-up|",_("Go up to parent directory"),"|",_A(9),".",_A(42),"|",NULL),
			g_strconcat(_("_Forward"),"|gtk-go-forward|",_("Go to next directory in history"),"|",_A(9),".",_A(41),"|",NULL),
			g_strdup(">"),
			NULL);

		e2_toolbar_options_register (E2_BAR_PANE1);  //set general toolbar options
	}
	else if (num == 2)
	{
		_e2_option_tree_prepare_default (set, 
			g_strconcat("<",internal_name,NULL),

#ifdef E2_PANES_HORIZONTAL
		//build with 2nd bar also left to right
//			g_strconcat(_("_Panes"),"|gtk-zoom-fit|",_("Toggle other-pane display"),"|",
//				_A(11),".",_A(28),"|","*,0",NULL), //no arg translation
		//NOTE - these 2 btns and their corresponding T/F flags are expected to be present
		// (text/icons can vary) see code for selecting appropriate one of the pair at session start
			g_strconcat(_("_Panes"),"|gtk-zoom-fit|",_("Hide other pane"),"|",_A(10),".",_A(74),"|false",NULL),	//no translation
			g_strconcat(_("_Panes"),"|"ICON_DIR"split_vert_"E2IP".png|",
				_("Show other-pane"),"|",_A(10),".",_A(74),"|true",NULL),
//			g_strconcat(_("Show _hidden"),"|"ICON_DIR"hidden_show2_"E2IP".png|",
//				_("Toggle display of hidden items in this directory"),"|",_A(10),".",_A(72),"|",NULL),
		
			g_strconcat(_("Show _hidden"),"|"ICON_DIR"hidden_show_"E2IP".png|",
				_("Display hidden items in this directory"),"|",_A(10),".",_A(72),"|false",NULL),	//no translation
			g_strconcat(_("Hide _hidden"),"|"ICON_DIR"hidden_noshow_"E2IP".png|",
				_("Do not display hidden items in this directory"),"|",_A(10),".",_A(72),"|true",NULL),	//no translation

			g_strconcat(_("Fil_ters"),"|"ICON_DIR"filter_"E2IP".png|",
//				_("Set/remove rules for the items to be displayed"),"|", _A(18),"|",NULL),
				_("Set rules for the items to be displayed"),"|",_A(10),".",_A(22),"|true",NULL),
//			g_strconcat("\t|||",_A(10),".",_A(22),"|",NULL),
			g_strconcat(_("Fil_ters"),"|"ICON_DIR"filter_off_"E2IP".png|",
				_("Set/remove rules for the items to be displayed"),"|",_A(10),".", _A(22),"|false",NULL),

// inconsistent with other toolitems here
//			g_strconcat(_("_Select all"),"|"ICON_DIR"star_"E2IP".png|",
//				_("Toggle select all items"),"|",_A(10),".",_A(73),"|",NULL),

			g_strconcat("|||",_A(10),".",_A(24),"|,",_A(89),NULL),

			g_strconcat(_("_Marks"),"|"ICON_DIR"bookmark_"E2IP".png|",
				_("Bookmarks"),"|",_A(18),"|",NULL),
			g_strconcat("\t",_("Add _top"),"||",_("Add the current directory to the top of the bookmarks list"),
				"|",_A(0),".",_A(27),"|",_A(10),",",_A(93),NULL),
			g_strconcat("\t|||",_A(10),".",_A(21),"|",NULL),
			g_strconcat("\t",_("Add _bottom"),"||",_("Add the current directory to the bottom of the bookmarks list"),
				"|",_A(0),".",_A(27),"|",_A(10),NULL),
			g_strconcat("\t|||",_A(17),"|",NULL),
			g_strconcat("\t",_("_Edit bookmarks"),"||",
			_("Open the bookmarks configuration dialog"),"|",_A(2),".",_C(1),"|",NULL),

			g_strconcat(_("Mi_rror"),"|"ICON_DIR"mirror_vert_"E2IP".png|",
				_("Go to directory shown in other pane"),"|",_A(10),".",_A(77),"|",NULL),

			g_strconcat(_("_Back"),"|gtk-go-back|",_("Go to previous directory in history"),"|",_A(10),".",_A(40),"|",NULL),
			g_strconcat(_("_Up"),"|gtk-go-up|",_("Go up to parent directory"),"|",_A(10),".",_A(42),"|",NULL),
			g_strconcat(_("_Forward"),"|gtk-go-forward|",_("Go to next directory in history"),"|",_A(10),".",_A(41),"|",NULL),
#else  //vertical panes
	//build 2nd bar with buttons in reverse order (more or less)
			g_strconcat(_("_Back"),"|gtk-go-back|",_("Go to previous directory in history"),"|",_A(10),".",_A(40),"|",NULL),
			g_strconcat(_("_Up"),"|gtk-go-up|",_("Go up to parent directory"),"|",_A(10),".",_A(42),"|",NULL),
			g_strconcat(_("_Forward"),"|gtk-go-forward|",_("Go to next directory in history"),"|",_A(10),".",_A(41),"|",NULL),

			g_strconcat(_("Mi_rror"),"|"ICON_DIR"mirror_horiz_"E2IP".png|",
				_("Go to directory shown in other pane"),"|",_A(10),".",_A(77),"|",NULL),

			g_strconcat(_("_Marks"),"|"ICON_DIR"bookmark_"E2IP".png|",
				_("Bookmarks"),"|",_A(18),"|",NULL),
			g_strconcat("\t",_("Add _top"),"||",_("Add the current directory to the top of the bookmarks list"),
				"|",_A(0),".",_A(27),"|",_A(10),",",_A(93),NULL),
			g_strconcat("\t|||",_A(10),".",_A(21),"|",NULL),
			g_strconcat("\t",_("Add _bottom"),"||",_("Add the current directory to the bottom of the bookmarks list"),
				"|",_A(0),".",_A(27),"|",_A(10),NULL),
			g_strconcat("\t|||",_A(17),"|",NULL),
			g_strconcat("\t",_("_Edit bookmarks"),"||",
			_("Open the bookmarks configuration dialog"),"|",_A(2),".",_C(1),"|",NULL),

			g_strconcat("|||",_A(10),".",_A(24),"|,",_A(89),NULL),

// inconsistent with other toolitems here
//			g_strconcat(_("_Select all"),"|"ICON_DIR"star_"E2IP".png|",
//				_("Toggle select all items"),"|",_A(10),".",_A(73),"|",NULL),

			g_strconcat(_("Fil_ters"),"|"ICON_DIR"filter_"E2IP".png|",
//				_("Set/remove rules for the items to be displayed"),"|", _A(18),"|",NULL),
				_("Set rules for the items to be displayed"),"|",_A(10),".",_A(22),"|true",NULL),
//			g_strconcat("\t|||",_A(10),".",_A(22),"|",NULL),
			g_strconcat(_("Fil_ters"),"|"ICON_DIR"filter_off_"E2IP".png|",
				_("Set/remove rules for the items to be displayed"),"|",_A(10),".", _A(22),"|false",NULL),

			g_strconcat(_("Show _hidden"),"|"ICON_DIR"hidden_show_"E2IP".png|",
				_("Display hidden items in this directory"),"|",_A(10),".",_A(72),"|false",NULL),	//no translation
			g_strconcat(_("Hide _hidden"),"|"ICON_DIR"hidden_noshow_"E2IP".png|",
				_("Do not display hidden items in this directory"),"|",_A(10),".",_A(72),"|true",NULL),	//no translation

			g_strconcat(_("_Panes"),"|gtk-zoom-fit|",_("Hide other pane"),"|",_A(10),".",_A(74),"|false",NULL),	//no translation
			g_strconcat(_("_Panes"),"|"ICON_DIR"split_horiz_"E2IP".png|",
				_("Show other-pane"),"|",_A(10),".",_A(74),"|true",NULL),

#endif  //E2_PANES_HORIZONTAL
			g_strdup(">"),
			NULL);

		e2_toolbar_options_register (E2_BAR_PANE2);  //set general toolbar options
	}

	gint i; gchar *option_name, *desc, *tip, *grey_me;
	E2_OptionFlags flags;
	desc = (num == 1) ? _A(9) : _A(10);
	gchar *group_name = g_strconcat (parent, ".", _C(5), ":", desc, NULL);    // _("panes.columns:pane N"
	for (i = 0; i < MAX_COLUMNS; i++)
	{
		option_name = g_strdup_printf ("pane%d-show-column%d", num, i);
		desc = g_strdup_printf (_("show %s column"), gettext(e2_all_columns[i].title));
		tip = g_strdup_printf (_("This causes the the named column to be displayed in pane %d (duh ...)"), num);
		flags = E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREENAME | E2_OPTION_FLAG_FREEDESC
			| E2_OPTION_FLAG_FREETIP | E2_OPTION_FLAG_BUILDPANES;
		if (i > 0)
			grey_me = NULL;
		else
		{
			flags |= E2_OPTION_FLAG_FREEGROUP;
			//disable hiding of col 0 = filename
			grey_me = g_strconcat ("!",option_name,NULL); //a 'depends' entry = !<self> disables any change
		}
		e2_option_bool_register (option_name, group_name, desc, tip, grey_me,
		TRUE, flags);
	}
	option_name = g_strdup_printf ("pane%d-use-startup-dir", num);
//	group_name = g_strconcat (parent, ":", _C(34), NULL);    // _("panes:startup"
	group_name = g_strconcat (_C(17), ":", _C(34), NULL);    // _("general:startup"
 	desc = g_strdup_printf (_("at startup, show a specific directory in pane %d"), num);
	e2_option_bool_register (option_name, group_name,
		desc, _("This causes the directory named below, instead of the last-opened directory, to be shown at session start"),
		NULL, FALSE,
		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREEGROUP 
		| E2_OPTION_FLAG_FREENAME | E2_OPTION_FLAG_FREEDESC );
	option_name = g_strconcat (option_name,"-startup-dir", NULL);
	desc = g_strdup_printf (_("pane %d startup directory:"), num);
//FREEME = non-constant name, desc
	e2_option_str_register (option_name, group_name,
		desc,  _("This is the directory to show in this pane, at session start"),
		"",  //= no dependency, changed to rt->opt_use_startup->name after pane rt is created
		(num == 1) ? "~" : G_DIR_SEPARATOR_S,   //pane1 default = home, pane2 default = root directory
		E2_OPTION_FLAG_BASIC | E2_OPTION_FLAG_FREENAME | E2_OPTION_FLAG_FREEDESC);
}
