/* $Id: e2_task.c 1128 2005-09-14 02:15:03Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/* TODO 25/10/04
some functions do not need TreeRowReferences
 (as of now, none do - no on-the-fly store content changes)
when stability confirmed, move surrounding dis/enable refresh to e2_dialog_ow_check() itself ?
*/

#include "e2_task.h"
//#include <unistd.h>
#include <string.h>
#include "e2_dialog.h"
#include "e2_option.h"
#include "e2_ownership_dialog.h"
#include "e2_filelist.h"

gboolean _e2_task_move (gpointer action_data, gpointer rt_data,
	GtkWidget *widget, gchar *trashpath);

/**
 * @brief copy selected items from active to inactive pane
 *
 * Overwrite checking is performed, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_copy (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
//	printd (DEBUG, "task: copy");
	if (g_str_equal (curr_view->dir, other_view->dir))
	{
		//display some message
		return FALSE;
	}
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	GString *src = g_string_sized_new (PATH_MAX+NAME_MAX);
	GString *dest = g_string_sized_new (PATH_MAX+NAME_MAX);
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	//setup for tailoring over-write dialog
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	OW_ButtonFlags extras = (multisrc)? BOTHALL : NONE;

	//no window desensitize ...
	e2_widget_set_cursor (GDK_WATCH);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "disable refresh, copy task");
#endif
	e2_filelist_disable_refresh ();  //prevent pauses in the copy process
	gchar *itemname, *slocal, *dlocal;
	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, "..")) continue;
		itemname = (*iterator)->filename;
		//curr_view->dir, other_view->dir are utf-8
		g_string_printf(src, "%s%s", curr_view->dir, itemname);  //separator comes with dir
		g_string_printf(dest, "%s%s", other_view->dir, itemname);
		slocal = S_FILENAME_TO_LOCALE (src->str);
		dlocal = S_FILENAME_TO_LOCALE (dest->str);

		if (e2_fs_access2 (slocal) == 0)
		{	//item really still exists
			if (check && e2_fs_access2 (dlocal) == 0)
			{
#ifdef E2_REFRESH_DEBUG
		printd (DEBUG, "enable refresh, copy dialog");
#endif
				e2_filelist_enable_refresh ();  //allow updates while we wait
				DialogButtons result = e2_dialog_ow_check (dest->str, extras);
#ifdef E2_REFRESH_DEBUG
		printd (DEBUG, "disable refresh, copy dialog");
#endif
				e2_filelist_disable_refresh ();
				switch (result)
				{
					case YES_TO_ALL:
						check = FALSE;
					case OK:
						e2_task_backend_copy (slocal, dlocal);
					case CANCEL:
						break;
					default:
						result = NO_TO_ALL;
						break;
				}
				if (result == NO_TO_ALL)
				{
					S_FREE (slocal);
					S_FREE (dlocal);
					break;
				}
			}
			else  //file doesn't exist, or don't care
			{
				e2_task_backend_copy (slocal, dlocal);
			}
		}
		S_FREE (slocal);
		S_FREE (dlocal);
	}
	e2_fileview_clean_selected (names);
	g_string_free (src, TRUE);
	g_string_free (dest, TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif

	e2_widget_set_cursor (GDK_LEFT_PTR);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, copy task");
#endif
	e2_filelist_enable_refresh ();
//	printd (DEBUG, "task: copy FINISHED");
	return TRUE;
}
/**
 * @brief copy selected items from active to inactive pane, with renaming
 *
 * Overwrite checking is performed, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_copy_as (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gchar *new_name, *slocal, *dlocal;
	GString *prompt = g_string_sized_new (NAME_MAX+64);
	GString *src = g_string_sized_new (PATH_MAX+NAME_MAX);
	GString *dest = g_string_sized_new (PATH_MAX+NAME_MAX);
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	//setup for tailoring over-write dialog
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	// setup for showing stop button in rename dialog
	OW_ButtonFlags extras = (multisrc)? NOALL : NONE;

//	prepare_dialog();
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "disable refresh, copy as task");
#endif
	e2_filelist_disable_refresh ();
	e2_widget_set_cursor (GDK_WATCH);

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, "..")) continue;
		g_string_assign (src, (*iterator)->filename);
		g_string_printf (prompt, "%s: <b>%s</b>", _("Enter new name for"), src->str);
		DialogButtons result = 0;
	//FIXME = SPEED UP DIALOG CREATION OUTSIDE LOOP ?
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, copy as dialog");
#endif
		e2_filelist_enable_refresh ();  //allow updates while we wait
		DialogButtons result2 = e2_dialog_line_input (_("copy"), prompt->str, src->str, extras, FALSE, &new_name);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "disable refresh, copy as dialog");
#endif
		e2_filelist_disable_refresh ();

		if (result2 == OK)
		{
			g_string_printf (src, "%s%s", curr_view->dir, (*iterator)->filename);  //separator comes with dir
			g_string_printf (dest, "%s%s", other_view->dir, new_name);
			g_free (new_name);

			if (strcmp(src->str, dest->str) == 0) continue;
			slocal = S_FILENAME_TO_LOCALE (src->str);
			dlocal = S_FILENAME_TO_LOCALE (dest->str);

			if (check && e2_fs_access2 (dlocal) == 0)
			{
				e2_filelist_enable_refresh ();  //allow updates while we wait
				result = e2_dialog_ow_check (dest->str, extras);
				e2_filelist_disable_refresh ();
				if (result == OK)
				{
					e2_task_backend_copy (slocal, dlocal);
				}
/*				else if (result == YES_TO_ALL)
				{
					do something smart about multiple-renames
				} */
				else if (result == NO_TO_ALL)
				{
					S_FREE (slocal);
					S_FREE (dlocal);
					break;
				}
			}
			else
			{
				e2_task_backend_copy (slocal, dlocal);
			}
			S_FREE (slocal);
			S_FREE (dlocal);
		}
		else if (result2 == NO_TO_ALL)
		{
//			result = NO_TO_ALL;
			break;
		}
	}
	e2_fileview_clean_selected (names);
	g_string_free (prompt,TRUE);
	g_string_free (src,TRUE);
	g_string_free (dest,TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif

	gtk_widget_set_sensitive(app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, copy as task");
#endif
	e2_filelist_enable_refresh();
	return TRUE;
}
/**
 * @brief move selected items from active pane to trash location
 *
 * Selected item(s) are moved to the last-used trash dir, which will be:
 * if it exists, folder named '.Trash' in current dir, or
 * if it exists, ~/.Trash, or
 * if it exists, config dir/Trash, or
 * not moved, and error msg is printed
 * No overwrite checking is performed.
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, "" to act on selection
 * @param widget 
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_trashit (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
//	printd (DEBUG, "task: trash");
	//save pointer for other fn to use
	const gchar *last_trashpath = e2_utils_last_trash_path ();
	if (last_trashpath != NULL)
	{
		if (!g_str_equal (last_trashpath, curr_view->dir))  //do nothing if we're in the trash place already
			return (_e2_task_move (action_data, rt_data, widget, (gchar *) last_trashpath));
	}
	//error message displayed by the utils fn
	return FALSE;
}
/**
 * @brief move selected items from active to inactive pane
 *
 * Overwrite checking is performed, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_move (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
//	printd (DEBUG, "task: move");
	if (g_str_equal (curr_view->dir, other_view->dir))
	{
		//display some message
		return FALSE;
	}
	return (_e2_task_move (action_data, rt_data, widget, NULL));
}
/**
 * @brief move selected items
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 * @param trashpath string with path to trash dir, or NULL if moving to inactive pane
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean _e2_task_move (gpointer action_data, gpointer rt_data,
	GtkWidget *widget, gchar *trashpath)
{
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gchar *slocal, *dlocal;
#ifndef E2_INCLIST
	//handle case of specific data instead of selection??
	GtkTreeSelection *selection = gtk_tree_view_get_selection
		(GTK_TREE_VIEW (curr_view->treeview));
#endif
	GString *src = g_string_sized_new (PATH_MAX+NAME_MAX);
	GString *dest = g_string_sized_new (PATH_MAX+NAME_MAX);
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean trash = (trashpath != NULL);
	gboolean check = (trash) ? FALSE : e2_option_bool_get ("confirm-overwrite");
	//setup for tailoring over-write dialog
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	OW_ButtonFlags extras = (multisrc) ? BOTHALL : NONE;

	//no window de-sensitize
	e2_widget_set_cursor (GDK_WATCH);
	e2_filelist_disable_refresh ();  //avoid pauses in the move process
//	e2_window_disable_status_update ();  //prevent changes to the selected files count
	gchar *itemname;

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, "..")) continue;
		itemname = (*iterator)->filename;
		g_string_printf (src, "%s%s", curr_view->dir, itemname);  //separator comes with dir
		gchar *p = (trash) ? trashpath : other_view->dir ;	//CHECKME trashpath = utf-8, not localised
		g_string_printf (dest, "%s%s", p, itemname);
		slocal = S_FILENAME_TO_LOCALE (src->str);
		dlocal = S_FILENAME_TO_LOCALE (dest->str);

		if (e2_fs_access2 (slocal) == 0)
		{	//item really still exists
			if (check && e2_fs_access2 (dlocal) == 0)
			{
				e2_filelist_enable_refresh ();  //allow updates while we wait
				DialogButtons result = e2_dialog_ow_check (dest->str, extras);
				e2_filelist_disable_refresh ();
				switch (result)
				{
					case YES_TO_ALL:
						check = FALSE;
					case OK:
#ifdef E2_INCLIST
						e2_task_move_backend (slocal, dlocal);
#else
						if (e2_task_backend_move (slocal, dlocal))
							//unselect item only if move is successful
							gtk_tree_selection_unselect_path (selection, (*iterator)->path);
#endif
					case CANCEL:
						break;
					default:
						result = NO_TO_ALL;
						break;
				}
				if (result == NO_TO_ALL)
				{
					S_FREE (slocal);
					S_FREE (dlocal);
					break;
				}
			}
			else  //file doesn't exist, or don't care
			{
#ifdef E2_INCLIST
				e2_task_backend_move (slocal, dlocal);
#else
				if (e2_task_backend_move (slocal, dlocal))
					gtk_tree_selection_unselect_path (selection, (*iterator)->path);
#endif
			}
		}
		S_FREE (slocal);
		S_FREE (dlocal);
	}
	e2_fileview_clean_selected (names);
	g_string_free (src, TRUE);
	g_string_free (dest, TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
//	e2_fileview_clean_selected (names);

	e2_widget_set_cursor (GDK_LEFT_PTR);
	e2_filelist_enable_refresh();
//	e2_window_enable_status_update (-1);

	return TRUE;
}
/**
 * @brief move selected items from active to inactive pane, with renaming
 *
 * Overwrite checking is performed, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_move_as (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gchar *new_name, *slocal, *dlocal;
#ifndef E2_INCLIST
	//handle case of specific data instead of selection??
	GtkTreeSelection *selection = gtk_tree_view_get_selection
		(GTK_TREE_VIEW (curr_view->treeview));
#endif
	GString *prompt = g_string_sized_new (NAME_MAX+64);
	GString *src = g_string_sized_new (PATH_MAX+NAME_MAX);
	GString *dest = g_string_sized_new (PATH_MAX+NAME_MAX);
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	OW_ButtonFlags extras = (multisrc)? NOALL : NONE;

//	prepare_dialog();  //disable refresh needed ??
	e2_widget_set_cursor (GDK_WATCH);
	e2_filelist_disable_refresh ();
//	e2_window_disable_status_update ();  //prevent changes to the selected files count

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, "..")) continue;
		g_string_assign (src, (*iterator)->filename);
		g_string_printf (prompt, "%s: <b>%s</b>", _("Enter new name for"), src->str);
		DialogButtons  result = 0;
		e2_filelist_enable_refresh ();  //allow updates while we wait
		DialogButtons result2 = e2_dialog_line_input (_("move"), prompt->str, src->str, extras, FALSE, &new_name);
		e2_filelist_disable_refresh ();

		if (result2 == OK)
		{
			g_string_printf(src, "%s%s", curr_view->dir, (*iterator)->filename);  //separator comes with dir
			g_string_printf(dest, "%s%s", other_view->dir, new_name);
			if (strcmp (src->str, dest->str) == 0) continue;
			slocal = S_FILENAME_TO_LOCALE (src->str);
			dlocal = S_FILENAME_TO_LOCALE (dest->str);

			if (check && e2_fs_access2 (dlocal) == 0)
			{
				e2_filelist_enable_refresh ();  //allow updates while we wait
				result = e2_dialog_ow_check (dest->str, extras);
				e2_filelist_enable_refresh ();
				if (result == OK)
				{
#ifdef E2_INCLIST
					e2_task_move_backend (slocal, dlocal);
#else
					if (e2_task_backend_move (slocal, dlocal))
						gtk_tree_selection_unselect_path (selection, (*iterator)->path);
#endif
				}
/*				else if (result == YES_TO_ALL)
				{
					do something smart about multiple-renames ??
				} */
				else if (result == NO_TO_ALL)
				{
					S_FREE (slocal);
					S_FREE (dlocal);
					break;
				}
			}
			else
			{
#ifdef E2_INCLIST
				e2_task_move_backend (slocal, dlocal);
#else
				if (e2_task_backend_move (slocal, dlocal))
					gtk_tree_selection_unselect_path (selection, (*iterator)->path);
#endif
			}
			S_FREE (slocal);
			S_FREE (dlocal);
		}
		else if (result2 == NO_TO_ALL)
		{
//			result = NO_TO_ALL;
			break;
		}
	}
	e2_fileview_clean_selected (names);
	g_string_free (prompt,TRUE);
	g_string_free (src,TRUE);
	g_string_free (dest,TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
	gtk_widget_set_sensitive(app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	e2_filelist_enable_refresh();
//	e2_window_enable_status_update (-1);

	return TRUE;
}

//FIXME where to put this ??
static void _e2_task_response_decode_cb (GtkWidget *button,
	gint response, gint *dialog_result)
{
	*dialog_result = response;
	gtk_main_quit ();
	return;
}
/**
 * @brief deleted selected items in the active pane
 *
 * Confirmation is sought, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_delete (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
//	printd (DEBUG, "task: delete");
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gchar *slocal;
#ifndef E2_INCLIST
	//handle case of specific data instead of selection??
	GtkTreeSelection *selection = gtk_tree_view_get_selection
		(GTK_TREE_VIEW (curr_view->treeview));
#endif
	GString *src = g_string_sized_new (NAME_MAX);
	GString *prompt = g_string_sized_new (NAME_MAX +64);
	gboolean check = e2_option_bool_get ("confirm-delete");
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	gint result = 0;

//	prepare_dialog ();
	e2_filelist_disable_refresh ();
	e2_widget_set_cursor  (GDK_WATCH);
//	e2_window_disable_status_update ();  //prevent changes to the selected files count

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, ".."))continue;
		gchar *filename = g_build_filename (curr_view->dir, (*iterator)->filename, NULL);
		slocal = S_FILENAME_TO_LOCALE (filename);

		if (e2_fs_access2 (slocal) == 0)
		{	//item really still exists
			if (check)
			{
				g_string_assign (src, (*iterator)->filename);
				g_string_printf (prompt, _("Are you sure you want to delete <b>%s</b>?"), src->str);
				GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
					prompt->str, _("confirm"), _e2_task_response_decode_cb, &result);	
	//FIXME make this support single-setup when multisrc is FALSE
				//setup buttons states
				if (!multisrc)
				{
					E2_BUTTON_NOTOALL.showflags |= E2_BTN_GREY;
					E2_BUTTON_YESTOALL.showflags |= E2_BTN_GREY;
				}
				E2_BUTTON_CANCEL.showflags |= E2_BTN_DEFAULT;
				E2_BUTTON_OK.showflags &= !E2_BTN_DEFAULT;
	#ifdef E2_REFRESH_DEBUG
		printd (DEBUG, "enable refresh, delete dialog");
	#endif
				e2_filelist_enable_refresh ();  //allow updates while we wait
	/* modal dialog
				result = e2_dialog_show (dialog, app.main_window,
					E2_DIALOG_RUN | E2_DIALOG_FREE,
					&E2_BUTTON_NOTOALL, &E2_BUTTON_YESTOALL,
					&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
	*/
	//non-modal dialog
				e2_dialog_show (dialog, app.main_window,
					0,
					&E2_BUTTON_NOTOALL, &E2_BUTTON_YESTOALL,
					&E2_BUTTON_CANCEL, &E2_BUTTON_OK, NULL);
				gtk_main ();
				gtk_widget_destroy (dialog);
	#ifdef E2_REFRESH_DEBUG
		printd (DEBUG, "disable refresh, delete dialog");
	#endif
				e2_filelist_disable_refresh ();
	
				switch (result)
				{
					case E2_RESPONSE_YESTOALL:
						check = FALSE;
					case GTK_RESPONSE_OK:
	#ifdef E2_INCLIST
						e2_task_backend_delete (slocal);
	#else
						if (e2_task_backend_delete (slocal))
							gtk_tree_selection_unselect_path (selection, (*iterator)->path);
	#endif
					case GTK_RESPONSE_CANCEL:
						break;
	//				case E2_RESPONSE_NOTOALL:
					default:
						result = E2_RESPONSE_NOTOALL;
						break;
				}
			}
			else  //no checking
			{
	#ifdef E2_INCLIST
				e2_task_delete_backend (slocal);
	#else
				if (e2_task_backend_delete (slocal))
					gtk_tree_selection_unselect_path (selection, (*iterator)->path);
	#endif
			}
		}
		g_free (filename);
		S_FREE (slocal);
		if (result == E2_RESPONSE_NOTOALL)
			break;
	}
	e2_fileview_clean_selected (names);
	g_string_free (src, TRUE);
	g_string_free (prompt, TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
//	gtk_widget_set_sensitive(app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	e2_filelist_enable_refresh();
//	e2_window_enable_status_update (-1);

	return TRUE;
}
/**
 * @brief delete all items from trash location
 *
 * The last-used trash dir, which will be:
 * if it exists, folder named '.Trash' in current dir, or
 * if it exists, ~/.Trash, or
 * if it exists, config dir/Trash, 
 * will be opened, and all item(s) there will be selected and deleted
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, "" to act on selection
 * @param widget 
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_trashempty (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
//	printd (DEBUG, "task: empty trash");
	const gchar *last_trashpath = e2_utils_last_trash_path ();
	if (last_trashpath != NULL)
	{
		gchar *pathsave = NULL;
		//make sure we're in the trash place
		if (!g_str_equal (last_trashpath, curr_view->dir))
		{
			pathsave = g_strdup (curr_view->dir);
			e2_pane_change_dir (NULL, (gchar *)last_trashpath);
		}
		
		e2_fileview_select_all (NULL, curr_view);
		gboolean retval = e2_task_delete (action_data, rt_data, widget);

		if (pathsave != NULL)
		{
			e2_pane_change_dir (NULL, pathsave);
			g_free (pathsave);
		}
		return retval;
	}
	//error message displayed by the utils fn
	return FALSE;
}
/**
 * @brief symlink selected items in active pane to the inactive pane
 *
 * Overwrite checking is performed, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_symlink (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	if (g_str_equal (curr_view->dir, other_view->dir))
	{
		//display some message
		return FALSE;
	}
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	GString *src = g_string_sized_new (PATH_MAX);	//used for target path
	GString *dest = g_string_sized_new (PATH_MAX);	//used for link path
	gboolean use_rel_links = e2_option_bool_get ("relative-symlinks");
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	OW_ButtonFlags extras = (multisrc)? BOTHALL : NONE;

	e2_widget_set_cursor (GDK_WATCH);
	e2_filelist_disable_refresh();  //avoid any interruption to linking
	gchar *itemname, *slocal, *dlocal;

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, "..")) continue;
		itemname = (*iterator)->filename;
		g_string_printf (src, "%s%s", curr_view->dir, itemname);  //separator comes with dir
		g_string_printf (dest, "%s%s", other_view->dir, itemname);
		if (use_rel_links)
			e2_utils_create_relative_path (src, dest);
		slocal = S_FILENAME_TO_LOCALE (src->str);	//target, possibly relative
		dlocal = S_FILENAME_TO_LOCALE (dest->str);	//link name

//		if (e2_fs_access2 (slocal) == 0)
//		{	//item really still exists
			if (check && e2_fs_access2 (dlocal) == 0)
			{
				e2_filelist_enable_refresh ();  //allow updates while we wait
				DialogButtons result = e2_dialog_ow_check (dest->str, extras);
				e2_filelist_disable_refresh ();
				switch (result)
				{
					case YES_TO_ALL:
						check = FALSE;
					case OK:
						e2_task_backend_link (slocal, dlocal);
					case CANCEL:
						break;
					default:
						result = NO_TO_ALL;
						break;
				}
				if (result == NO_TO_ALL) break;
			}
			else  //file not present or don't care
				e2_task_backend_link (slocal, dlocal);
//		}
		S_FREE (slocal);
		S_FREE (dlocal);
	}
	e2_fileview_clean_selected (names);
	g_string_free (src, TRUE);
	g_string_free (dest, TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
	e2_widget_set_cursor (GDK_LEFT_PTR);
	e2_filelist_enable_refresh();

	return TRUE;
}
/**
 * @brief symlink selected items in active pane to the inactive pane, with renaming
 *
 * Overwrite checking is performed, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_symlink_as (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gchar *new_name, *slocal, *dlocal;
	GString *prompt = g_string_sized_new (NAME_MAX+64);
	GString *src = g_string_sized_new (PATH_MAX);	//used for target path
	GString *dest = g_string_sized_new (PATH_MAX);	//used for link path
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gboolean use_rel_links = e2_option_bool_get ("relative-symlinks");
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	OW_ButtonFlags extras = (multisrc) ? NOALL : NONE;
	F_DECLARE;
	
//	prepare_dialog();
	e2_widget_set_cursor (GDK_WATCH);
	e2_filelist_disable_refresh ();

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, "..")) continue;
		g_string_assign (src, (*iterator)->filename);
		g_string_printf (prompt, "%s: <b>%s</b>", _("Enter new name for"), src->str);
		DialogButtons result = 0;
		e2_filelist_enable_refresh ();  //allow updates while we wait
		DialogButtons result2 = e2_dialog_line_input (_("symlink"), 
			prompt->str, src->str, extras, FALSE, &new_name);
		e2_filelist_disable_refresh ();

		if (result2 == OK)
		{
			g_string_printf(src, "%s%s", curr_view->dir, (*iterator)->filename);  //separator comes with dir
			g_string_printf(dest, "%s%s", other_view->dir, new_name);
			g_free (new_name);
			if (strcmp (src->str, dest->str) == 0) continue;
			if (use_rel_links)
				e2_utils_create_relative_path (src, dest);
			slocal = S_FILENAME_TO_LOCALE (src->str);	//target, possibly relative
			dlocal = S_FILENAME_TO_LOCALE (dest->str);	//link name

			if (check && (e2_fs_access2 (dlocal) == 0))
			{
				e2_filelist_enable_refresh ();  //allow updates while we wait
				result = e2_dialog_ow_check (dest->str, extras);
				e2_filelist_disable_refresh ();
					if (result == OK)
					e2_task_backend_link (slocal, dlocal);
/*				if (result == YES_TO_ALL)
				{
					do something smart about multiple-renames ??
				} */
				else if (result == NO_TO_ALL)
					break;
			}
			else
				e2_task_backend_link (slocal, dlocal);
			S_FREE (slocal);
			S_FREE (dlocal);
		}
		else if (result2 == NO_TO_ALL)
		{
//			result = NO_TO_ALL;
			break;
		}
	}
	e2_fileview_clean_selected (names);
	g_string_free (prompt,TRUE);
	g_string_free (src,TRUE);
	g_string_free (dest,TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
//	gtk_widget_set_sensitive(app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	e2_filelist_enable_refresh();
	return TRUE;
}
/**
 * @brief show/change permission of selected items in active pane
 *
 * The actual operation is performed by a separate back-end function
 * Directories are treated separately, to support recursive changes to
 * directories only
 * View is not updated immediately as the change does not trigger an auto-refresh  FIXME
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_permissions (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
//	printd (DEBUG, "task: permissions");
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	GString *path = g_string_sized_new (PATH_MAX+NAME_MAX);
	GString *mode = g_string_sized_new (16);
	GtkWidget *dialog=NULL;
	gint count;
	gint result;
	gint myuid = -1;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean multisrc = names->len > 1;
	gboolean all = FALSE;
	F_DECLARE;

/* out-of-loop setup = FIXME
	GtkWidget *dialog;
	dialog = e2_permissions_dialog_setup (info, &recurse);
	e2_dialog_add_buttons_simple (dialog, buttons ..., NULL);
*/	
//	prepare_dialog();
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "disable refresh, permissions task");
#endif
	e2_filelist_disable_refresh ();
	e2_widget_set_cursor (GDK_WATCH);

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, ".."))continue;
		E2_RecurseType recurse;
		gboolean permission;
//FIXME for single-setup: instead of the following, adjust file details in dialog, reset default button

		if (all)
		{
			//check if we have permission to change this item
			//NB this tests _all_ w permissions, chmod is not so sophisticated ...
//			gchar *itempath = g_strconcat (curr_view->dir, (*iterator)->filename, NULL);
// 			permission = e2_utils_check_write_permission (myuid, itempath);
//			g_free (itempath);
			//path is to current dir
 			permission = e2_fs_check_write_permission (myuid, (*iterator)->filename);
			if (permission)
				result = GTK_RESPONSE_OK;
			else
			{
				result = GTK_RESPONSE_CANCEL;
				gchar *msg = g_strdup_printf ("%s %s\n", 
					_("You do not have authority to change permission(s) for"),
					(*iterator)->filename);
				e2_output_print_error(msg);
				g_free (msg);
			}
		}
		else
		{
			recurse=E2_RECURSE_NONE;
			permission=TRUE;
			g_string_assign (mode, "");
			dialog = e2_permissions_dialog_setup (*iterator, &recurse, 
				&permission, &mode, multisrc);
/*			result = _e2_dialog_show (dialog, app.main_window, 
				E2_DIALOG_RUN | E2_DIALOG_FREE);
*/
			g_signal_connect (G_OBJECT (dialog), "response",
				G_CALLBACK (_e2_task_response_decode_cb), &result);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, permissions dialog");
#endif
			e2_filelist_enable_refresh ();  //allow updates while we wait
			_e2_dialog_show (dialog, app.main_window, 0);
			gtk_main ();
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "disable refresh, permissions dialog");
#endif
			e2_filelist_disable_refresh ();
		}

		switch (result)
		{
		  case E2_RESPONSE_APPLYTOALL:
			all = TRUE;
			myuid = getuid ();  //do this once, for speed
			result = GTK_RESPONSE_OK;
		  case GTK_RESPONSE_OK:
			if (permission)
			{
				g_string_printf (path, "%s%s", curr_view->dir, (*iterator)->filename); //separator comes with dir
				gchar *slocal = F_FILENAME_TO_LOCALE (path->str);
#ifdef E2_INCLIST
				e2_task_backend_chmod (slocal, mode->str, recurse);
#else
#ifdef E2_FAM
				e2_task_backend_chmod (slocal, mode->str, recurse);
#else
				if (e2_task_backend_chmod (slocal, mode->str, recurse))
					//make the file-list refresher notice successful change
					e2_fs_utime (curr_view->dir, NULL);
#endif
#endif
				F_FREE (slocal);
			}
		  case GTK_RESPONSE_CANCEL:
			break;
		  default:
			result = E2_RESPONSE_NOTOALL;  // break flag;
			break;
		}
		if (result == E2_RESPONSE_NOTOALL)
			break;
		if (!all && (count < names->len - 1))
			//destroy dialog, ready for next one
			gtk_widget_destroy (dialog);
	}
	e2_fileview_clean_selected (names);
	g_string_free (path, TRUE);
	g_string_free (mode, TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
	if ((count == names->len) || (result == E2_RESPONSE_NOTOALL))
		gtk_widget_destroy (dialog);
//	gtk_widget_set_sensitive (app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, permissions task");
#endif
	e2_filelist_enable_refresh ();
	gtk_widget_grab_focus (curr_view->treeview);

	return TRUE;
}
/**
 * @brief show/change user and/or group of selected items in active pane
 *
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_ownership (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	GtkWidget *dialog=NULL;
	GString *path = g_string_sized_new (PATH_MAX+NAME_MAX);
	gint count;
	gint result;
	gint myuid = -1;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean multisrc = names->len > 1;
	gboolean all = FALSE;
	F_DECLARE;

//	prepare_dialog();
	e2_filelist_disable_refresh ();
	e2_widget_set_cursor (GDK_WATCH);

	for (count=0; count < names->len; count++, iterator++)
	{
		gboolean permission;
		uid_t owner_id;
		gid_t group_id;
		gboolean recurse;
//		if (g_str_equal((*iterator)->filename, ".."))continue;

		if (all)
		{
			//check if we have permission to change this item
//			gchar *itempath = g_strconcat (curr_view->dir, (*iterator)->filename, NULL);
// 			permission = e2_utils_check_write_permission (myuid, itempath);
//			g_free (itempath);
			//path is to current dir
 			permission = e2_fs_check_write_permission (myuid, (*iterator)->filename);
			if (permission)
				result = GTK_RESPONSE_OK;
			else
			{
				result = GTK_RESPONSE_CANCEL;
				gchar *msg = g_strdup_printf ("%s  %s\n", 
					_("You do not have authority to change owner(s) for"),
					(*iterator)->filename);
				e2_output_print_error (msg);
				g_free (msg);
			}
		}
		else
		{
			owner_id=-1;  //CHECKME = should these be set outside loop ?
			group_id=-1;
			permission=TRUE;
			recurse=FALSE;
			dialog = e2_ownership_dialog_setup (*iterator, &owner_id, 
				&group_id, &recurse, &permission, multisrc);
			//leave dialog open as long as possible to prevent 'shadow' prior to screen refresh
			g_signal_connect (G_OBJECT (dialog), "response",
				G_CALLBACK (_e2_task_response_decode_cb), &result);
			e2_filelist_enable_refresh ();  //allow updates while we wait
			_e2_dialog_show (dialog, app.main_window, 0);
			gtk_main ();
			e2_filelist_disable_refresh ();
		}

		switch (result)
		{
		  case E2_RESPONSE_APPLYTOALL:
			all = TRUE;
			myuid = getuid ();
			result = GTK_RESPONSE_OK;
		  case GTK_RESPONSE_OK:
			if (permission)
			{
//				printd (DEBUG, "ownership task: ok");
//				printd (DEBUG, "new owner id is: %d",  owner_id);
				g_string_printf (path, "%s%s", curr_view->dir, (*iterator)->filename); //separator comes with dir
				gchar *slocal = F_FILENAME_TO_LOCALE (path->str);
#ifdef E2_INCLIST
				e2_task_backend_chown (slocal, owner_id, group_id, recurse);
#else
#ifdef E2_FAM
				e2_task_backend_chown (slocal, owner_id, group_id, recurse);
#else
				if (e2_task_backend_chown (slocal, owner_id, group_id, recurse))
					//make the refresher notice successful change
					e2_fs_utime (curr_view->dir, NULL);
#endif
#endif
				F_FREE (slocal);
			}
		  case GTK_RESPONSE_CANCEL:
			break;
		  default:
			result = E2_RESPONSE_NOTOALL;  // break flag;
			break;
		}
		if (result == E2_RESPONSE_NOTOALL)
			break;
		if (!all && (count < names->len - 1))
			//destroy dialog, ready for next one
			gtk_widget_destroy (dialog);
	}
	e2_fileview_clean_selected (names);
	g_string_free (path,TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
	if ((count == names->len) || (result == E2_RESPONSE_NOTOALL))
		gtk_widget_destroy (dialog);
//	gtk_widget_set_sensitive(app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	e2_filelist_enable_refresh ();
	gtk_widget_grab_focus (curr_view->treeview);

	return TRUE;
}
/**
 * @brief show info about each select item in active pane
 *
 * NB the active pane directory is not necessarily the current file system
 * dir, so the path is added to the filename
 * FIXME on this ?
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_file_info (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	//handle case of specific data instead of selection??
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	GtkWidget *dialog;
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;

//	prepare_dialog ();
	e2_widget_set_cursor (GDK_WATCH);
//	gtk_widget_set_sensitive (app.main_window, FALSE);

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, "..")) continue;
		dialog = e2_file_info_dialog_setup (*iterator);
		_e2_dialog_show (dialog, app.main_window, 0);
		gtk_main ();
	}
	e2_fileview_clean_selected (names);

//	postpare_dialog (FALSE, NULL);
	e2_widget_set_cursor (GDK_LEFT_PTR);
//	gtk_widget_set_sensitive (app.main_window, TRUE);

	//get focus back to where we started
	gtk_widget_grab_focus (curr_view->treeview);
//	e2_fileview_focus_row (curr_view, curr_view->row, FALSE, FALSE, FALSE, TRUE);
	return TRUE;
}
/**
 * @brief rename selected items in active pane to the inactive pane
 *
 * Overwrite checking is performed, if that option is enabled
 * The actual operation is performed by a separate back-end function
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_rename (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gchar *new_name, *slocal, *dlocal;
#ifndef E2_INCLIST
	//handle case of specific data instead of selection??
	GtkTreeSelection *selection = gtk_tree_view_get_selection
		(GTK_TREE_VIEW (curr_view->treeview));
#endif
	GString *prompt = g_string_sized_new (NAME_MAX+64);
	GString *src = g_string_sized_new (PATH_MAX+NAME_MAX);
	GString *dest = g_string_sized_new (PATH_MAX+NAME_MAX);
	gboolean check = e2_option_bool_get ("confirm-overwrite");
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	gboolean multisrc =  (check) ? names->len > 1 : FALSE;
	OW_ButtonFlags extras = (multisrc) ? NOALL : NONE;

//	prepare_dialog();
	e2_filelist_disable_refresh ();
	e2_widget_set_cursor (GDK_WATCH);
//	e2_window_disable_status_update ();  //prevent changes to the selected files count

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal((*iterator)->filename, ".."))continue;
		g_string_assign (src, (*iterator)->filename);
		g_string_printf (prompt, "%s: <b>%s</b>", _("Enter new name for"), src->str);
		DialogButtons result = 0;
		e2_filelist_enable_refresh ();  //allow updates while we wait
		DialogButtons result2 = e2_dialog_line_input (_("rename"), prompt->str, src->str, extras, FALSE, &new_name);
		e2_filelist_disable_refresh ();

		if (result2 == OK)
		{
			g_string_printf(src, "%s%s", curr_view->dir, (*iterator)->filename);  //separator comes with dir
			g_string_printf(dest, "%s%s", curr_view->dir, new_name);
			g_free (new_name);
			if (strcmp(src->str,dest->str)== 0) continue;
			slocal = S_FILENAME_TO_LOCALE (src->str);
			dlocal = S_FILENAME_TO_LOCALE (dest->str);

			if (check && e2_fs_access2 (dlocal) == 0)
			{
				e2_filelist_enable_refresh ();  //allow updates while we wait
				result = e2_dialog_ow_check (dest->str, extras);
				e2_filelist_disable_refresh ();
				if (result == OK)
				{
#ifdef E2_INCLIST
					e2_task_rename_backend (slocal, dlocal);
#else
					if (e2_task_backend_rename (slocal, dlocal))
						gtk_tree_selection_unselect_path (selection, (*iterator)->path);
#endif
				}
				else if (result == NO_TO_ALL)
				{
					S_FREE (slocal);
					S_FREE (dlocal);
					break;
				}
			}
			else
			{
#ifdef E2_INCLIST
				e2_task_rename_backend (slocal, dlocal);
#else
				if (e2_task_backend_rename (slocal, dlocal))
					gtk_tree_selection_unselect_path (selection, (*iterator)->path);
#endif
			}
			S_FREE (slocal);
			S_FREE (dlocal);
		}
		else if (result2 == NO_TO_ALL)
			break;
	}
	e2_fileview_clean_selected (names);
	g_string_free (prompt,TRUE);
	g_string_free (src,TRUE);
	g_string_free (dest,TRUE);

#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
//	gtk_widget_set_sensitive(app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	e2_filelist_enable_refresh();
//	e2_window_enable_status_update (-1);

	return TRUE;
}
/**
 * @brief execute the operation specified from the drag menu, on all dragged items
 *
 * Item overwrite checks are performed if that option is in force.
 * @a uri_list is created in e2_filelist_drag_data_get () 
 *	For e2-sourced items, it is a string of the format:
 *	file://path/to/one_file\r\nfile://path/to/second_file\r\n
 * and so on for each selected item. The "file://" at the start of
 *	each element is ascii, the path/name data was converted to
 *	utf-8 if necessary
 * For externally-sourced items, it may be different eg
 *	file:/path/to/one_file\r\nfile:/path/to/second_file\r\n
 * It assumes all items have the same source path.
 * Nothing will be done if src and dest dirs are the same
 *
 * Processing of dropped items stops if any problem occurs
 *
 * @param func pointer to backend function to be performed on dragged items
 * @param uri_list string, uri list of items to process with @a func
 * @param dest_dir utf-8 string, destination for items in @a uri_list, with trailing /
 *
 * @return TRUE if the items were all processed successfully
**/
gboolean e2_task_drop (gpointer func, gchar *uri_list, gchar *dest_dir)
{
//	printd (DEBUG, "callback: drop task");
	GString *dest = g_string_sized_new (PATH_MAX+NAME_MAX);
	gchar *s, *p, *filename, *slocal, *dlocal;

	gboolean retval;
	gboolean (*op_func)(gchar *, gchar *) = func;

	gchar **uris = g_strsplit (uri_list, "\r\n", -1);

	gint skip_len;
	//get length to ignore at start of each uri
	s = uris[0];
/* internally-sourced directory items will not have a trailing /, because
	e2_fileview_drag_data_get_cb() translates the names afresh
	and externally sourced ones will probably not have one ??
	from the corresponding info->filename
	if (g_str_has_suffix (s, G_DIR_SEPARATOR_S))
	{
		skip_len = strlen (s) - 1;
		*(s+skip_len) = '\0';
			filename = strrchr (s, G_DIR_SEPARATOR);
		*(s+skip_len) = G_DIR_SEPARATOR;
	}
	else */
		filename = strrchr (s, G_DIR_SEPARATOR);
	NCHR(filename);
	//check for src path = dest path
	gchar save = *(filename);
	*(filename) = '\0';
	p = s;
	//look for the '/' that really starts the path, in context where the
	//no. of leading '/'s is not certain (various apps create different URIs)
	while (TRUE)
	{
		p = strchr (p, G_DIR_SEPARATOR);	//if always ascii /, don't need g_utf8_strchr()
		p++;	//NCHR(p);
		if (*p != G_DIR_SEPARATOR)
		{
			p--;	//PCHR(p);
			break;
		}
	}
	if (g_str_equal (p, dest_dir))
		return FALSE;
	*(filename) = save;
	//remember the length to strip from each source path, to get item name
	skip_len = filename - s;	//assumes single-byte separator

	//count items, to allow appropriate dialog buttons display
	gchar **tmp = uris;
	gint ctr = 0;
	while (*tmp != NULL && **tmp != '\0')	//last string is empty
	{
		if (++ctr > 1) break;
		tmp++;
	}
	OW_ButtonFlags extras = (ctr > 1) ? BOTHALL : NONE;

	gboolean check = e2_option_bool_get ("confirm-overwrite");
	DialogButtons result;

	//walk the array of items
	tmp = uris;
	while (*tmp != NULL && **tmp != '\0')
	{
		//get src & dest paths
		s = *tmp + 5;	//skip over "file:" (ascii)
		filename = *tmp + skip_len;
		g_string_printf (dest, "%s%s", dest_dir, filename); //dest_dir comes with /
		//get localised paths
		slocal = S_FILENAME_FROM_LOCALE (s);
		dlocal = S_FILENAME_FROM_LOCALE (dest->str);

		if (check && (e2_fs_access2 (dlocal) == 0))
		{
//			gtk_widget_set_sensitive (app.main_window, FALSE);
			result = e2_dialog_ow_check (dest->str, extras);
//			gtk_widget_set_sensitive (app.main_window, TRUE);
			switch (result)
			{
				case YES_TO_ALL:
					check = FALSE;
				case OK:
					//can't unselect moved items on-the-fly, there is no path data here
					retval = op_func (slocal, dlocal);
					if (!retval)
						result = NO_TO_ALL; //abort on failure
				case CANCEL:
					break;
				default:
					result = NO_TO_ALL;
					break;
			}
			if (result == NO_TO_ALL)
			{
				S_FREE (slocal);
				S_FREE (dlocal);
				break;
			}
		}
		else  //file doesn't exist, or don't care
		{
			retval = op_func(slocal, dlocal);
		}
		S_FREE (slocal);
		S_FREE (dlocal);
		tmp++;
	}
	//cleanups
	g_string_free (dest, TRUE);
	g_strfreev (uris);
#ifdef E2_INCLIST
	//FIXME
#else
	e2_filelist_check_dirty (
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
		GINT_TO_POINTER(1)
#else
		NULL
#endif
		);
#endif
	return retval;
}
/**
 * @brief change 'other' pane to be the same as 'current' pane
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_sync_dirs (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	e2_pane_change_dir (app.pane_other, app.pane_current->path);
	return TRUE;
}
/**
 * @brief view content of 1st selected item in active pane
 *
 * The actual operation is performed by internal or extenal viewer,
 * depending on the option specfied
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget menu item ? UNUSED
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_view (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	gchar *view_this = (gchar *) rt_data;	//CHECKME utf-8 needed
	if (view_this != NULL && *view_this != '\0')
	{  //we arrived here with a single file to be viewed
		//it could be a path or just a filename, and it might be in parentheses
		//which are a problem for building absolute paths if need be, so strip them
		if (*view_this == '"')
			view_this++;
		gchar *lng = view_this + strlen (view_this) - 1;
		if (*lng == '"')
			*lng = '\0';

		return (e2_task_backend_view (view_this));
	}
	//otherwise, process the active pane selection
	GPtrArray *names = e2_fileview_get_selected (curr_view);
	if (names == NULL) return FALSE;	//nothing selected
	gint count;
	E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;

	e2_filelist_disable_refresh();
	gtk_widget_set_sensitive (app.main_window, FALSE);

	for (count=0; count < names->len; count++, iterator++)
	{
//		if (g_str_equal(**iterator, "..")) continue;
		//this assumes that the current system dir is curr_view->dir
		e2_task_backend_view ((*iterator)->filename);
		break;	//int/ext viewer(s) don't have an 'abort' function, so stop after 1st view
	}
	e2_fileview_clean_selected (names);

	gtk_widget_set_sensitive (app.main_window, TRUE);
	gtk_widget_grab_focus (curr_view->treeview);
	e2_filelist_enable_refresh();
	return TRUE;
}
/**
 * @brief toggle tag on selected item in active pane
 * 
 * Deleted after discussion with tooar
 *
 * @return
 */
/*void __e2_task_toggle_tag_cb (void)
{
//FIXME = logic of this is silly !!
	GList *tmp = e2_fileview_get_selection (curr_view);
	//deprecated
	FileInfo *info;
	gint row;

//	g_signal_emit_by_name (G_OBJECT (curr_view->clist), "end-selection");

	for (; tmp != NULL; tmp = tmp->next)
	{
/ *		row = GPOINTER_TO_INT (tmp->data);
		info = gtk_clist_get_row_data(GTK_CLIST(curr_view->clist), row);
		if (g_list_find (curr_view->tagged, info) != NULL)
		{
			curr_view->tagged = g_list_remove(curr_view->tagged, info);
			gtk_clist_set_background(GTK_CLIST(curr_view->clist), row, &LIST_COLOR);
		}
		else
		{
			curr_view->tagged = g_list_append(curr_view->tagged, info);
			gtk_clist_set_background(GTK_CLIST(curr_view->clist), row, e2_option_color_get ("color-tag"));
		}  * /
	//FIXME
	}
	e2_fileview_focus_row(curr_view, curr_view->row+1, FALSE, TRUE, TRUE, TRUE);
} */
/**
 * @brief refresh content of both pane views
 *
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_refresh (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	e2_widget_set_cursor (GDK_WATCH);
	e2_filelist_disable_refresh ();
#ifdef E2_INCLIST
	//FIXME
#else
//#ifndef E2_FAM
	e2_filelist_refresh_list (other_view);
	e2_filelist_refresh_list (curr_view);
//#ifndef E2_FAM
#endif
	e2_filelist_enable_refresh ();
	e2_widget_set_cursor (GDK_LEFT_PTR);
	return TRUE;
}
/**
 * @brief revert all configuration settings to defailt values
 *
 *
 * @param widget always NULL, unused
 *
 * @return TRUE always
 */
gboolean e2_task_configure_default (GtkWidget *widget)
{
	e2_option_restore_defaults ();
	return TRUE;
}
/**
 * @brief initiate configuration change
 *
 *
 * @param widget always NULL, unused
 * @param page name string of 'page' in config dialog, or NULL
 *
 * @return TRUE, always
 */
gboolean e2_task_configure (GtkWidget *widget, gchar *page)
{  //put this in config dialog file
	gtk_widget_set_sensitive (app.main_window, FALSE);
	e2_config_dialog_create (page);
	gtk_widget_set_sensitive (app.main_window, TRUE);
	return TRUE;
}
/**
 * @brief find named item in current pane
 *
 * The search starts from the current line, forward to end, then cycles
 * back to the start
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data assigned when actionrt struct created
 * @param widget the button pressed to activate this task, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_find (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	gchar *s, *pattern;
	gint i, last, rowcount;

	//find the no of rows in the store
	rowcount = gtk_tree_model_iter_n_children 
		(GTK_TREE_MODEL (curr_view->store), NULL);
	if (rowcount < 1)
		return FALSE;  //no message ...

//	prepare_dialog();
	e2_widget_set_cursor (GDK_WATCH);
//	gtk_widget_set_sensitive (app.main_window, FALSE);

	DialogButtons result2 = e2_dialog_line_input (_("find"), _("Enter a name or partial name to find:"),
		curr_view->last_find, 0, TRUE, &s);

	if (result2 == OK)
	{
		g_strlcpy (curr_view->last_find, s, sizeof(curr_view->last_find));
		pattern = g_strdup_printf("*%s*", s);

		GtkTreePath *path;
		GtkTreeIter iter;
		if (gtk_tree_model_iter_nth_child
			(GTK_TREE_MODEL (curr_view->store), &iter, NULL, curr_view->row))
		{
			path = gtk_tree_model_get_path
				(GTK_TREE_MODEL (curr_view->store), &iter);
		}
		else
		{
			//try fallback to the row of the current cursor position
			gtk_tree_view_get_cursor (GTK_TREE_VIEW (curr_view->treeview),
				&path, NULL);
		}

		if (path == NULL)
		{
			last = rowcount - 1;
			//no defined start pos, start from beginning
			i = 0;
		}
		else
		{
			last = *gtk_tree_path_get_indices (path);
			if (last == rowcount - 1)
				i = 0;	//roll around to the start
			else
				i = last + 1;	//start at the next line
			
			gtk_tree_path_free (path);
		}

		//iterate through the rows
		gchar *thisrowname;
		while (i != last)
		{
			thisrowname = e2_fileview_get_row_name (curr_view, i);
			if (g_pattern_match_simple (pattern, thisrowname))
			{
				e2_fileview_focus_row (curr_view, i, TRUE, TRUE, TRUE, TRUE);
				g_free (thisrowname);
				break;
			}
			g_free (thisrowname);
			if (++i >= rowcount)
				i = 0;  //cycle to start
		}
		if (i == last)
			//nothing found (or maybe found in the last row)
			e2_fileview_focus_row (curr_view, last, TRUE, TRUE, FALSE, TRUE);
		g_free(pattern);
		g_free(s);
	}
	else	//user cancelled
		e2_fileview_focus_row (curr_view, curr_view->row, FALSE, FALSE, FALSE, TRUE);

//	postpare_dialog (FALSE, NULL);
//	gtk_widget_set_sensitive (app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	gtk_widget_grab_focus (curr_view->treeview);
	return TRUE;
}
/**
 * @brief perform the default action for the first selected item in the current pane
 *
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_open (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
//	printd (DEBUG, "task: open");
	//handle case of specific data instead of selection??
	FileInfo *info;
	if ((info = e2_fileview_get_selected_first_local (curr_view)) != NULL)
	{
		return (e2_task_backend_open (info));
	}
	return FALSE;
}
/**
 * @brief open item(s) in the current pane with a chosen application
 *
 * If the specified command does not include '%', all selected items
 * are concatenated, and sent as a single argument to the application.
 * Otherwise, the entered command is simply executed
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_open_with (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	//handle case of specific data instead of selection??
	gchar *command;

//	prepare_dialog();
	e2_widget_set_cursor (GDK_WATCH);
//	gtk_widget_set_sensitive (app.main_window, FALSE);

	DialogButtons result = e2_dialog_combo_input (_("open with"),
		_("Enter command:"), 0, &open_history, &command);

	gboolean retval = FALSE;
	if (result == OK)
	{
		retval = TRUE;
		//FIXME support macros other than with english letters
		if (strstr (command, "%f") == NULL)
		{
			GString *command_line = g_string_new (command);
			GPtrArray *names = e2_fileview_get_selected (curr_view);
			gint count;
			E2_SelectedItemInfo **iterator = (E2_SelectedItemInfo **) names->pdata;
	
			for (count=0; count < names->len; count++, iterator++)
			{
				g_string_append_printf (command_line , " \"%s\"",
					(*iterator)->filename);
			}
			e2_command_run (command_line->str, E2_COMMAND_RANGE_DEFAULT);
			g_string_free(command_line, TRUE);
			e2_fileview_clean_selected (names);
		}
		else
			e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);

		g_free(command);
	}

//	postpare_dialog (FALSE, NULL);
//	gtk_widget_set_sensitive (app.main_window, TRUE);
	e2_widget_set_cursor (GDK_LEFT_PTR);
	gtk_widget_grab_focus (curr_view->treeview);
	
	return retval;
}
/**
 * @brief open directory which is currently selected, in the other pane
 * 
 * There is no check whether the item is actually directory
 * The last-activated row in the other pane is focused - this is not
 * necessarily sensible
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget always NULL, unused
 *
 * @return TRUE if task completed successfully, else FALSE
 */
gboolean e2_task_open_in_other_pane (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
	//handle case of specific data instead of selection??
	e2_filelist_disable_refresh();
	gchar *name = e2_fileview_get_row_name (curr_view, curr_view->row);
	if (name == NULL) return FALSE;  //the model got mixed up, somehow
	gchar *path = g_strdup_printf("%s%s", curr_view->dir, name); //separator comes with dir
	e2_pane_change_dir (app.pane_other, path);
	g_free (name);
	g_free (path);
	//CHECKME a more sensible row to goto ??
	e2_fileview_focus_row (other_view, other_view->row, FALSE, FALSE, FALSE, TRUE);
	e2_filelist_enable_refresh();
	return TRUE;
}

static E2_Action e2_actions[] =
{
	{ "", E2_ACTION_TYPE_ITEM, e2_task_open, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_open_in_other_pane, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_open_with, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_view, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_file_info, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_permissions, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_ownership, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_rename, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_copy, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_copy_as, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_move, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_move_as, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_symlink, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_symlink_as, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_delete, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_find, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_trashit, FALSE, 0 },
	{ "", E2_ACTION_TYPE_ITEM, e2_task_trashempty, FALSE, 0 }
};

void e2_task_actions_register (void)
{
	e2_actions[0].name = g_strconcat(_A(5),".",_A(51),NULL);
	e2_actions[1].name = g_strconcat(_A(5),".",_A(52),NULL);
	e2_actions[2].name = g_strconcat(_A(5),".",_A(53),NULL);
	e2_actions[3].name = g_strconcat(_A(5),".",_A(75),NULL);
	e2_actions[4].name = g_strconcat(_A(5),".",_A(37),NULL);
	e2_actions[5].name = g_strconcat(_A(5),".",_A(57),NULL);
	e2_actions[6].name = g_strconcat(_A(5),".",_A(54),NULL);
	e2_actions[7].name = g_strconcat(_A(5),".",_A(60),NULL);
	e2_actions[8].name = g_strconcat(_A(5),".",_A(33),NULL);
	e2_actions[9].name = g_strconcat(_A(5),".",_A(34),NULL);
	e2_actions[10].name = g_strconcat(_A(5),".",_A(48),NULL);
	e2_actions[11].name = g_strconcat(_A(5),".",_A(49),NULL);
	e2_actions[12].name = g_strconcat(_A(5),".",_A(68),NULL);
	e2_actions[13].name = g_strconcat(_A(5),".",_A(69),NULL);
	e2_actions[14].name = g_strconcat(_A(5),".",_A(35),NULL);
	e2_actions[15].name = g_strconcat(_A(5),".",_A(6),NULL);
	e2_actions[16].name = g_strconcat(_A(5),".",_A(81),NULL);
	e2_actions[17].name = g_strconcat(_A(5),".",_A(82),NULL);

	gint actions_count = sizeof (e2_actions) / sizeof (e2_actions[0]);
	gint i;
	for (i = 0; i < actions_count; i++)
	{
		e2_action_register (e2_actions[i].name, e2_actions[i].type,
			e2_actions[i].func, NULL, e2_actions[i].has_arg,
			e2_actions[i].exclude, NULL);
	}
}
