/* $Id: e2_toolbar.h 1034 2005-08-15 02:12:13Z tpgww $
 * 
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef E2_TOOLBAR_H
#define E2_TOOLBAR_H

#include "emelfm2.h"
#include "e2_action.h"
#include "e2_option.h"

typedef enum
{	//these are ordered for decreasing visual impact when changes
	//are made
	//if order is changed, correspondingly change order of array rows
	//in e2_toolbar_options_register()
	E2_BAR_TASK,
	E2_BAR_PANE1,
	E2_BAR_PANE2,
	E2_BAR_COMMAND,
	E2_BAR_COUNT	//this is the no. of bars, not a 'type'
} E2_BarType;

typedef struct _E2_ToolbarRuntime
{
	gchar *name;	//'internal' name of bar, for referencing (constant or allocated)
	gchar *public_name;	//translated form of name (constant)
	GtkToolbar *toolbar;
	GtkWidget *toolbar_container;
	GtkWidget *toolbar_container_box;
	gboolean blocked;  //TRUE when bar's context menu is being created
	gboolean hidden;
	E2_OptionSet *set;
	E2_OptionSet *show;
	E2_OptionSet *tooltips;
	E2_OptionSet *space;
	E2_OptionSet *hori;
	E2_OptionSet *type;
	E2_OptionSet *priority;
	E2_OptionSet *relief;
	E2_OptionSet *same;
	E2_OptionSet *style;
	E2_OptionSet *isize;
	GtkIconSize icon_size;	//working copy of size enumerator (supports 'default')
	GtkAllocation *alloc;
	GtkRequisition req;
	gboolean has_toggle;
	gboolean has_bookmarks;
	gboolean has_command_line;
	gboolean is_too_small;  //this is actually a flag for visibilty of the rest-menu button
	GtkWidget *button_rest;
//#ifdef GTK26
	GList *bar_items_list;  //list of toolbar buttons (not spacers etc), including hidden toggles
//#endif
	GtkWidget *menu_starter;	//bar item that is first in the overflow menu
} E2_ToolbarRuntime;

typedef struct _E2_ToolbarData
{
	E2_BarType type;
	gchar *name;	//internal name, constant, not translated
	gchar *public_name;	//translated name, constant
	E2_ToolbarRuntime *rt;	//data struct for the bar
} E2_ToolbarData;

//#ifdef GTK26
void e2_toolbar_initialise_space_handler (E2_ToolbarRuntime *rt);
void e2_toolbar_initialise_tearoffs (void);
//#endif
void e2_toobar_menu_position_set (GtkWidget *button, GtkWidget *menu, 
	GtkAllocation *menu_position, E2_ToolbarRuntime *rt);
void e2_toolbar_menu_position (GtkMenu *menu, gint *x, gint *y,
	gboolean *push_in, GtkAllocation *menu_position);
gboolean e2_toolbar_button_toggle (gchar *action_name);
gboolean e2_toolbar_toggle_button_get_state (gchar *action_name);
void e2_toolbar_toggle_button_set_state (gchar *action_name, gboolean state);
void e2_toolbar_toggle_buttons_set_destroyed (E2_ToolbarRuntime *rt);
void e2_toolbar_initialise (E2_BarType barnum);
void e2_toolbar_create (E2_ToolbarRuntime *rt);
void e2_toolbar_recreate (E2_ToolbarRuntime *rt);
void e2_toolbar_recreate_all (void);
void e2_toolbar_destroy (E2_ToolbarRuntime *rt);

void e2_toolbar_data_create (void);
void e2_toolbar_actions_register (void);
void e2_toolbar_options_register (E2_BarType barnum);
void e2_toolbar_toolbaroptions_register (void);
void e2_toolbar_commandbaroptions_register (void);

#endif //ndef E2_TOOLBAR_H
