/* $Id: emelfm2.h 1122 2005-09-13 08:38:32Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#ifndef __EMELFM2_H__
#define __EMELFM2_H__
//see also: Makefile has build-time conditional #defines
#define _GNU_SOURCE
//default 64-bit functions (handle files > 2GB)
// also in makefile
#define _FILE_OFFSET_BITS 64

#include <sys/types.h>
//needed for various stafbuf declarations
#include <sys/stat.h>
#include <limits.h>
//some general things ...
#include <stdio.h>
#include <stdlib.h>

//#define GTK_DISABLE_DEPRECATED see Makefile
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gmodule.h>

#include "build.h"
#define GETTEXT_PACKAGE BINNAME
#include <glib/gi18n-lib.h>

//include code valid only for gtk versions > 2.4, according to the
//version in use when compiling
//#define E2_CURRENTGTK_VERSION see makefile

//backport Gtk 2.6 enumerator, for 2.4 users
#ifndef GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID
#define GTK_TREE_SORTABLE_UNSORTED_SORT_COLUMN_ID -2
#endif
//implement run-time recoloring of selected items' text for Gtk 2.4 users
//#if GTK_MINOR_VERSION == 4
//#define E2_SELTXT_RECOLOR
//#endif

//setup for kernel-based file monitoring, as opposed to fam/gamin
//see Makefile & build.h for definition of these variables
#if defined(E2_FAM_INOTIFY) || defined(E2_FAM_DNOTIFY) ||defined (E2_FAM_KQUEUE)
#define E2_FAM
#define E2_FAM_KERNEL
#else
//include code for file-change detection using FAM or gamin
//#define E2_FAM see Makefile & build.h
#ifdef E2_FAM
#include <fam.h>
#endif
#endif
#ifdef E2_FAM
//usec interval after file events before FAM is polled
//needs to be long enough for most/all changes to be noticed
//by the monitor, but not so long that the user perceives a delay
#define E2_FAMWAIT 100000
#endif

//include development code for making incremental changes to 
//file lists instead of dump & refill
//#define E2_INCLIST
#ifdef E2_INCLIST
//use treerowreferences in selection data structs
#define E2_ROWREFS
#endif

//include debug messages about refreshing
//#define E2_REFRESH_DEBUG
//include debug messages about mkdir dialog shutdown
//#define RACE_CHECK

//flag for toolbar code relevant to Gtk 2.4 & 2.6
//#define GTK26

//flag for code which uses internal, enhanced, substitute for nftw()
#define E2_TREEWALK

//bytes, NOT characters this is a linux define
//#define NAME_MAX 255
//length of various buffers
#define E2_MAX_LEN 1024
//msec interval between open-directory polls
//(long enough to avoid too many refreshes during
//e.g. archive creation)
#define E2_FILESCHECK_INTERVAL 3000
//msec interval between config-directory polls
//this is only used if E2_FAM not defined
#define E2_CONFIGCHECK_INTERVAL 5000
//msec interval between status line updates
#define E2_STATUSREPORT_INTERVAL 1500

//padding for various widgets
#define E2_PADDING_LARGE 8
#define E2_PADDING 5
#define E2_PADDING_SMALL 3
#define E2_PADDING_XSMALL 2

#define E2_COMMAND_PREFIX "."G_DIR_SEPARATOR_S
#define PLUGIN action_labels[12]
// in these, "home" is a pointer determined at runtime
#define E2_CONFIG_DIR home,"."BINNAME
#define E2_TRASH_DIR E2_CONFIG_DIR, "Trash"
#define SYSTEM_TRASH_DIR home, ".Trash"
//#define E2_CONFIG_FILE "config"  no good for gettext ?? do not translate

//default build with horizontal panes - see Makefile
//#define E2_PANES_HORIZONTAL
//default icon pointsize - see Makefile
//#define E2IP "24"

// this is for config/button/menu image-cacheing, post-0.1
#define E2_IMAGECACHE
#ifdef E2_IMAGECACHE
typedef enum
{
	E2_IMAGE_BMP,
	E2_IMAGE_SVG
} E2Imagetype;

typedef struct _E2_Image
{
//	E2Imagetype type;	//not used
	gint size;	//pixel size
//	gint refcount;	//not used
	GdkPixbuf *pixbuf;
} E2_Image;
#endif

#include "debug.h"
#include "e2_alias.h"
#include "e2_bookmark.h"
#include "e2_button.h"
#include "e2_cache.h"
#include "e2_cl_option.h"
#include "e2_combobox.h"
#include "e2_command.h"
#include "e2_command_line.h"
#include "e2_fileview.h"
#include "e2_fs.h"
#include "e2_hook.h"
#include "e2_keybinding.h"
#include "e2_list.h"
#include "e2_menu.h"
#include "e2_output.h"
#include "e2_pane.h"
#include "e2_toolbar.h"
#include "e2_tree.h"
#include "e2_utf8.h"
#include "e2_utils.h"
#include "e2_widget.h"
#include "e2_window.h"

// some nice helper/utility macros
#define MESC(d) g_markup_escape_text (d, -1)
#define WAIT_FOR_EVENTS while (gtk_events_pending ()) { gtk_main_iteration (); }
// development-phase string tagger
// search for _I( for general strings that need attention
#define _I(d) d

#define RUN_ONCE_CHECK(ret) \
	static gboolean run_once_check_init = FALSE; \
	if (run_once_check_init) \
		return ret; \
	else \
		run_once_check_init = TRUE;

typedef struct _emelFM2
{
	GtkWidget *main_window;
	//pair of main boxes that can be surrounded by toolbars
	GtkWidget *hbox_main;
	GtkWidget *vbox_main;
	FileView pane1_view;
	FileView pane2_view;
	E2_WindowRuntime window;
	E2_PaneRuntime pane1;
	E2_PaneRuntime pane2;
	E2_PaneRuntime *pane_current;
	E2_PaneRuntime *pane_other;
	GHookList hook_pane_focus_changed;
	E2_ToolbarRuntime toolbar;
	E2_ToolbarRuntime commandbar;
	E2_OutputRuntime output;
	E2_AliasRuntime aliases;
	E2_ToolbarData **bars;	//null-terminated array of pointers to toolbar data
#if defined(E2_FAM) && !defined(E2_FAM_KERNEL)
	FAMConnection *fcp;
	FAMEvent *fep;
#endif
	//pointer to context menu when it is popped up
	GtkWidget *context_menu;
	GtkWidget *status_bar_label2;
	//this is a hbox, at the right of the status bar, for general usage
	// needs to be shown when used
	GtkWidget *status_bar_box3;
	GtkWidget *drag_op_menu;
	GtkTooltips *tooltips;
//	GMutex *filelist_mutex;  //for locking threaded creation of filelists
} emelFM2;

#ifdef E2_FAM
typedef enum
{
	E2_MONITOR_DEFAULT,
	E2_MONITOR_FAM,	//unused
	E2_MONITOR_GAMIN
} E2_FAMonitor;
#endif

typedef struct _Config
{
	gchar version[20];  //store for e2 version string eg "0.0.9", used only for upgrades
	gint window_width;  //config dialog width
	gint window_height; //config dialog height
	guint auto_refresh_id;
	guint check_config_id;
	guint status_timer_id;
	time_t config_mtime;
	gboolean completed;	//set TRUE when it's ok to refresh the config data
#ifdef E2_FAM
	E2_FAMonitor monitor_type;
	gint FAMreq;	//request no used for monitoring config file changes
#endif
	GList *command_lines;	//list of rt data structs of command-lines
	GList *plugins;
	GHashTable *filetypes;
	GSList *typelist;	//list of string arrays, needed tor cleanup
	//pointer to dir used for trash storage (string with trailing /), or NULL
	gchar *trashpath;
#ifdef E2_IMAGECACHE
	GHashTable *icons;	//table of E2_Image's for cached icons
#endif
#ifndef FILES_UTF8ONLY
	gboolean utf8_filenames;	//TRUE when filesystem coding is utf-8 or ascii only
#endif
} Config;

emelFM2 app;
Config cfg;

void e2_main_user_shutdown (void);
void e2_main_winclose_shutdown (GtkWidget *widget,
	GdkEvent  *event, gpointer data);

#endif //ndef __EMELFM2_H__
