/* $Id$
 *
 * Copyright (C) 2005 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file src/filesystem/e2_fs.c
 * @brief filesystem io functions
 **/

#include "emelfm2.h"
#include <string.h>
#include <unistd.h>
#include <langinfo.h>
#include <pwd.h>
#include <grp.h>
#include "e2_fs.h"
#include "e2_utf8.h"
#include "e2_option_tree.h"

#if 0
/**
 * @brief check if any x permission flag is set in @a mode
 * @param mode
 * @return TRUE if any x flag is set
 **/
//FIXME this test should be for current user only, cf writable test
static gboolean _e2_fs_S_ISEXE (mode_t mode)
{
	if ((S_IXUSR & mode) || (S_IXGRP & mode) || (S_IXOTH & mode))
		return TRUE;
	return FALSE;
}
#endif //0

#ifndef FILES_UTF8ONLY
/**
 * @brief check whether the local filesystem encoding is utf8 or ascii, and setup for conversion accordingly
 *
 * @return 
 **/
//FIXME vfs may need to do this for each displayed dir
void e2_fs_check_coding (void)
{
	gchar *filecoding;
	cfg.utf8_filenames = FALSE;
	//1st, check what the user specified about encoding
	if (e2_cl_options.encoding != NULL)
	{
		filecoding = g_ascii_strdown (e2_cl_options.encoding, -1);
		if (strstr (filecoding, "ascii") != NULL
			|| strstr (filecoding, "utf-8") != NULL)
		cfg.utf8_filenames = TRUE;
		g_free (filecoding);
	}
	/* NO POINT IN THESE TESTS, THEY ONLY CONFIRM FALSE
	else if ((filecoding = (gchar *) g_getenv ("G_FILENAME_ENCODING")) != NULL)
	{
		if (strcmp (filecoding, "@locale"))
		{
			filecoding = strdup (filecoding);
			gchar *sep = strchr (filecoding, ',');	//CHECKME ',' for all locales ? or THOUSEP ?
			if (sep == NULL)
				*sep = '\0';
			MORE
			g_free (filecoding)
		}
	}
	else if ((filecoding = (gchar *) g_getenv ("G_BROKEN_FILENAMES")) != NULL)
	{
	} */
	else
	{
		//next, check what the locale data knows about the encoding
		filecoding = nl_langinfo (CODESET);
		if (!strcmp (filecoding, "UTF-8") ||
			strstr (filecoding, "ASCII") != NULL ||
			!strcmp (filecoding, "ANSI_X3.4-1968") ||
			!strcmp (filecoding, "646") ||
			!strcmp (filecoding, "ISO646") ||
			!strcmp (filecoding, "ISO_646.IRV"))
				cfg.utf8_filenames = TRUE;
		//next, check whether the locale language is one of the english variants
		//and that it's not being overridden by a relevant environment variable
		else
		{
			const gchar *lang =  g_getenv ("LANGUAGE");
			if (lang == NULL)
				lang = g_getenv ("LANG");
			if (lang != NULL && strstr (lang, "en_") != NULL)
			{
				filecoding = (gchar *) g_getenv ("G_BROKEN_FILENAMES");
				if (filecoding == NULL)
				{
					filecoding = (gchar *) g_getenv ("G_FILENAME_ENCODING");
					if (filecoding == NULL
			//FIXME check the 1st specified encoding is not ascii/utf
			//			|| 
					)
						cfg.utf8_filenames = TRUE;
					else if (g_str_equal (filecoding, "@locale"))
					{
//						G_CONST_RETURN char *coding2;
//						if (g_get_charset (&coding2))
							cfg.utf8_filenames = TRUE;
//						else
							//FIXME should check the name, as above
//							cfg.utf8_filenames = FALSE;
					}
				}
			}
		}
	}
	//set pointers to conversion functions
	if (cfg.utf8_filenames)
	{
 		e2_fname_to_locale = e2_fname_from_locale = e2_utf8_not_converted;
 		e2_fname_dupto_locale = e2_fname_dupfrom_locale = g_strdup;
		e2_fname_free = e2_utf8_not_freed;
	}		
	else
	{
		e2_fname_to_locale = e2_fname_dupto_locale = e2_utf8_filename_to_locale;
		e2_fname_from_locale = e2_fname_dupfrom_locale = e2_utf8_filename_from_locale;
		e2_fname_free = g_free;
	}
}
#endif

/**
 * @brief check whether @a filename exists
 * Can't use the function access(), which (for glibc at least)
 * looks through links
 * Errors other than non-readable are ignored
 * @param filename localised name of file to test
 *
 * @return -1 (TRUE) if @filename does not exist (like access(F_OK))
 **/
gint e2_fs_access2 (gchar *filename)
{
	struct stat statbuf;
	return (e2_fs_lstat (filename, &statbuf));
}
/**
 * @brief check whether @a filename can be 'used' in a manner consistent with flags in @a how
 * access() (for glibc at least) looks through links.
 * This mimics the glibc function access(), except that it does
 * not look through links
 *
 * @param filename localised name of file to test
 * @param how Or'd combination of R_OK, W_OK, X_OK and/or F_OK
 * @return -1 (TRUE) if all @how condition(s) are _not_ satisfied (this is the way access() works)
 **/
/*gint e2_fs_access3 (gchar *filename, gint how)
{
	static uid_t myuid = -1;
	static gid_t mygid = -1;
	struct stat statbuf;
	if (e2_fs_lstat (filename, &statbuf))
		return -1;
	else if (! S_ISLNK (statbuf.st_mode))
		return (access (filename, how));	//FIXME vfs

	if (myuid == -1)
		myuid = getuid ();
	if (mygid == -1)
		mygid = getgid ();
	gboolean result = FALSE;

	if (how & R_OK)
	{
		if (statbuf.st_mode & S_IROTH)
			result = TRUE;
		else if (statbuf.st_mode & S_IRUSR)
			result = ((myuid == 0) || (myuid == statbuf.st_uid));
		else if (statbuf.st_mode & S_IRGRP)
			result = ((myuid == 0) || (mygid == statbuf.st_gid));
		if (!result)
			return -1;
	}
	if (how & W_OK)
	{
		if (statbuf.st_mode & S_IWOTH)
			result = TRUE;
		else if (statbuf.st_mode & S_IWUSR)
			result = ((myuid == 0) || (myuid == statbuf.st_uid));
		else if (statbuf.st_mode & S_IWGRP)
			result = ((myuid == 0) || (mygid == statbuf.st_gid));
		if (!result)
			return -1;
	}
	if (how & X_OK)
	{
		if (statbuf.st_mode & S_IXOTH)
			result = TRUE;
		else if (statbuf.st_mode & S_IXUSR)
			result = ((myuid == 0) || (myuid == statbuf.st_uid));
		else if (statbuf.st_mode & S_IXGRP)
			result = ((myuid == 0) || (mygid == statbuf.st_gid));
		if (!result)
			return -1;
	}
	//we already know the link exists, no need for F_OK test
	return 0;
} */
/**
 * @brief execute the file (1) command and search its output for @a string
 *
 * Used in e2_fs_is_text and e2_fs_is_executable
 *
 * @param filename localised name of file to test
 * @param string ascii string for which to search, in the output from 'find'
 * @return TRUE if @a string is found
 **/
static gboolean _e2_fs_grep_file_output (gchar *filename, gchar *string)
{
	//do not use g_strdup_printf or g_snprintf because string is localised
	gchar line[PATH_MAX + 20];
	snprintf (line, sizeof (line), ("file \"%s\""), filename);	//do not translate
	E2_FILE *pipe = e2_fs_open_pipe (line);
	if (pipe == NULL)
		return FALSE;

	gboolean retval = FALSE;
	size_t bsize = E2_MAX_LEN;
	gchar *buf = (gchar *) g_malloc ((gulong) bsize);
#ifdef __USE_GNU
	ssize_t bytes_read;
	while ((bytes_read = getdelim (&buf, &bsize, '\n', pipe)) > 0)
#else
	gchar *tmpstr;
	while ((tmpstr = fgets (buf, bsize, pipe)) != NULL)
#endif
	{
		if (strstr (buf, string))
		{
			retval = TRUE;
			break;
		}
	}

	e2_fs_pipe_close (pipe);
	g_free (buf);
	return retval;
}
/**
 * @brief test whether @a filename is a text file
 * @param filename localised name of file to test
 * @return TRUE if it is a text file
 **/
gboolean e2_fs_is_text (gchar *filename)
{
	return _e2_fs_grep_file_output (filename, "text"); //do not translate
}
/**
 * @brief test whether @a info belongs to an executable item
 * If the item is a symlink, its target will be tested
 * @param info pointer to a FileInfo data struct
 * @return TRUE if system reports x permission, and the file (1) command says it's executable
 **/
gboolean e2_fs_is_executable (FileInfo *info)
{
	if (!e2_fs_access (info->filename, X_OK))
		//double check ...
		return _e2_fs_grep_file_output (info->filename, "executable");  //string not to be translated
	return FALSE;
}
/**
 * @brief check whether item in active pane is a directory or a link to one
 *
 * This assumes that the mode flags have been gathered using
 * lstat, not stat (ie no link look-through)
 *
 * Item needs to be in active pane because no path is added to info->filename
 *
 * @param info FileInfo structure
 *
 * @return TRUE if belongs to a directory or a link to a directory
 */
gboolean e2_fs_is_dir (FileInfo *info)
{
	if ( S_ISDIR (info->statbuf.st_mode) )
		return TRUE;
	if ( S_ISLNK(info->statbuf.st_mode) )
	{
		struct stat statbuf;
		if (!e2_fs_stat (info->filename, &statbuf)
			&& S_ISDIR (statbuf.st_mode))
		  return TRUE;
	}
	return FALSE;
}
/**
 * @brief inline version of func that checks whether item in active pane is a directory or a link to one
 *
 * This assumes that the mode flags have been gathered using
 * lstat, not stat (ie no link look-through)
 *
 * Item needs to be in active pane because no path is added to info->filename
 *
 * @param info FileInfo structure
 *
 * @return TRUE if belongs to a directory or a link to a directory
 */
inline gboolean e2_fs_is_dir_fast (FileInfo *info, struct stat *statbuf)
{
	if (S_ISDIR (info->statbuf.st_mode) )
		return TRUE;
	if (S_ISLNK(info->statbuf.st_mode) )
	{
		if (!e2_fs_stat (info->filename, statbuf)
			&& S_ISDIR (statbuf->st_mode))
		  return TRUE;
	}
	return FALSE;
}
/**
 * @brief check whether selected item in active pane is a directory or a link to one
 *
 * No path is added to info->filename, so must be in current dir
 *
 * @param info E2_SelectedItemInfo data structure
 *
 * @return TRUE if @a info belongs to a directory or a link to a directory
 */
gboolean e2_fs_is_dir2 (E2_SelectedItemInfo *info)
{
	struct stat statbuf;
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (info->filename);
	gboolean retval = (!e2_fs_stat (local, &statbuf) && S_ISDIR (statbuf.st_mode));
	F_FREE (local);
	return retval;
}
/**
 * @brief check whether @a path is a directory or a link to one
 *
 * @param path localised path string
 *
 * @return TRUE if @a info belongs to a directory or a link to a directory
 */
gboolean e2_fs_is_dir3 (gchar *local_path)
{
	struct stat statbuf;
	if (e2_fs_stat (local_path, &statbuf))
		return FALSE;
	return (S_ISDIR (statbuf.st_mode));
}
/**
 * @brief change system directory to @s path, if possible
 *
 * This tests @a path for existence, is-a-directory, is-executable
 *
 * @param path utf8 string with path to directory to be opened
 *
 * @return TRUE if @a path is opened ok
**/
gboolean e2_fs_chdir (gchar *path)
{
	gboolean retval = TRUE;
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (path);

	if (e2_fs_chdir_local (local))
	{	//go there failed
		e2_output_print_errno ();
		retval = FALSE;
	}
	F_FREE (local);
	return retval;
}
/**
 * @brief check whether the user is able to change to a specified directory
 *
 * This tests @a path for existence, is-a-directory, is-executable
 *
 * @param path utf8 string with path to directory to be checked
 *
 * @return TRUE if @a path is ok to cd to
 **/
gboolean e2_fs_cd_isok (gchar *path)
{
	gchar *message;
	gboolean success = TRUE;
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (path);
	if (e2_fs_access (local, F_OK))
	{
		message = g_strdup_printf (_("The directory '%s' does not exist!\n"), path);
		success = FALSE;
	}
	else if (!e2_fs_is_dir3 (local))
	{
		message = g_strdup_printf (_("The file '%s' is not a directory!\n"), path);
		success = FALSE;
	}
	else if (e2_fs_access (local, X_OK))
	{
		message = g_strdup_printf (_("No permission to access directory '%s'!\n"), path);
		success = FALSE;
	}
	if (!success)
	{
		e2_output_print_error (message);
		g_free (message);
	}
	F_FREE (local);
	return success;
}
/**
 * @brief check whether directory @a path is valid, and optionally, accessible, and if not, pick a fallback
 *
 * If test(s) are not satisfied, @a path is set to a different path string,
 * pointing to a place up the tree branch that does pass the test(s)
 * If a different path string is returned, the old one is freed, of course
 *
 * @param path utf8 path string which may be invalid, must be freeable
 * @param accessible TRUE to find a path that is accessible, FALSE if don't care about that
 *
 * @return TRUE if @a path meets the specified test(s), FALSE if a replacement has been provided
 */
gboolean e2_fs_get_valid_path (gchar **path, gboolean accessible)
{
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (*path);
	if (e2_fs_is_dir3 (local)
		&& (!accessible || !e2_fs_access (local, R_OK | X_OK)))
	{
		F_FREE (local);
		return TRUE;
	}
	else	//walk back up the tree branch until we find an acceptable dir
	{
		gchar *p;
		while ((p = strrchr (local, G_DIR_SEPARATOR)) != NULL)
		{
			*p = '\0';	//CHECKME ok to change local (maybe == path) ??
			if (e2_fs_is_dir3 (local)
				&& (!accessible || !e2_fs_access (local, R_OK | X_OK)))
			{	//replace the invalid path string with the valid one
				p = D_FILENAME_FROM_LOCALE (local);	//copy in case not changed by macro
				g_free (*path);
				*path = p;
				F_FREE (local);
				return FALSE;
			}
		}
		//whole branch bad, revert to default
		g_free (*path);
//		*path = g_strdup (G_DIR_SEPARATOR_S);
		*path = g_strconcat (g_get_home_dir (), G_DIR_SEPARATOR_S, NULL);
		F_FREE (local);
	}
	return FALSE;
}
/**
 * @brief stat item @a path, put results into @a buf
 *
 * This does not look through links (target may not exist)
 *
 * @param filename relative or absolute path of item to stat, localised string
 * @param buf pointer to buffer to hold the results
 *
 * @return 0 (FALSE) if the stat succeeded, else -1 (TRUE)
 **/
gint e2_fs_stat (const gchar *path, struct stat *buf)
{
	if (stat (path, buf))	//FIXME vfs
	{	//stat failed, but it could be just due to a relative symlink
		struct stat statbuf;	//don't disturb the caller's buffer
		if (e2_fs_lstat (path, &statbuf)	//stat link succeeded
			|| (S_ISLNK (statbuf.st_mode) && errno == ENOENT)
		)
		{
			//FIXME more checks for hanging link?
			*buf = statbuf;
			return 0;
		}
		else
			return -1;
	}
//	else
		return 0;
}
/**
 * @brief check whether the current user may modify a specified item
 *
 * This does not look through links (target may not exist)
 *
 * @param uid the current user's uid, or -1 to determine the latter
 * @param path utf string with path of item to check
 *
 * @return TRUE if the user has write authority for @a path
 **/
gboolean e2_fs_check_write_permission (gint uid, gchar *path)
{
	if (uid == -1)
		uid = getuid ();
	if (uid == 0)
		return TRUE; //user is root
	
	struct stat statbuf;
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (path);
	e2_fs_lstat (local, &statbuf);
	F_FREE (local);

	if ((uid == (gint) statbuf.st_uid)	//user is owner
//		&& (statbuf.st_mode & S_IWUSR )	//with write permission
	)
		return TRUE;

	if (statbuf.st_mode & S_IWOTH)
		return TRUE; //everyone has write permission

	struct group *groupdata = getgrgid (statbuf.st_gid);
	if (groupdata != NULL)
	{
		gchar **members = groupdata->gr_mem;
		while (*members != NULL)
		{
			struct passwd *pwbuf = getpwnam (*members);
			if ( pwbuf != NULL &&
				((gint) pwbuf->pw_uid == uid))
			{
				if (statbuf.st_mode & S_IWGRP)
						return TRUE; //user is group, with write permission
				else
					break;
			}
			members++;	
		}
	}

	return FALSE;
}
/**
 * @brief open file @a filepath for writing
 *
 * @param filepath path to file open
 *
 * @return pointer to data struct for opened file, or NULL if the open failed
 */
E2_FILE *e2_fs_open_file_to_write (gchar *filepath)
{
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (filepath);
	E2_FILE *f = fopen (local, "w");
	F_FREE (local);
	return f;
}
/**
 * @brief open pipe to read output from @a command
 *
 * @param command utf-8 command string
 * @return pointer to opened pipe, or NULL if open failed
 **/
E2_FILE *e2_fs_open_pipe (gchar *command)
{
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (command);
	E2_FILE *pipe;
	if ((pipe = popen (local, "r")) == NULL)
	{
		gchar *s;
		if ((s = strchr (command, ' ')) != NULL)	//always ascii
			*s = '\0';
		s = g_strdup_printf (_("Unable to open pipe for %s command\n"), command);
		e2_output_print_error (s);
		g_free (s);
	}
	F_FREE (local);
	return pipe;
}
/**
 * @brief read file contents into memory
 *
 * @param filepath path to file to be read
 * @param contents ptr to where the contents are to be stored
 *
 * @return TRUE if @a filepath was read successfully
 */
gboolean e2_fs_get_file_contents (gchar *filepath, gchar **contents)
{
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (filepath);
	//CHECKME ok to localise the name string  with this glib fn ?
	gboolean retval = g_file_get_contents (local, contents, NULL, NULL);
	F_FREE (local);
	return retval;
}
/**
 * @brief convert file mode_t to a readable permissions string
 * NOTE this handles only single-byte strings, and some parts are
 * not translated
 * String is 10 bytes long
 *
 * @param buf ptr to buffer where the result is to be stored
 * @param len integer length of @a buf
 * @param mode mode_t property to be parsed
 *
 * @return
 */
void e2_fs_get_perm_string (gchar *buf, gint len, mode_t mode)
{
	gchar *perm_sets[8];
	perm_sets[0] = _("---");
	perm_sets[1] = _("--x");
	perm_sets[2] = _("-w-");
	perm_sets[3] = _("-wx");
	perm_sets[4] = _("r--");
	perm_sets[5] = _("r-x");
	perm_sets[6] = _("rw-");
	perm_sets[7] = _("rwx");

	gint u, g, o;
	u = (mode & S_IRWXU) >> 6;
	g = (mode & S_IRWXG) >> 3;
	o = (mode & S_IRWXO);

	g_snprintf (buf, len, "-%s%s%s", perm_sets[u], perm_sets[g], perm_sets[o]);

	//_I( FIXME the things below do not suppport translation !!
	if (S_ISLNK(mode))
		buf[0] = 'l';
	else if (S_ISDIR(mode))
		buf[0] = 'd';
	else if (S_ISBLK(mode))
		buf[0] = 'b';
	else if (S_ISCHR(mode))
		buf[0] = 'c';
	else if (S_ISFIFO(mode))
		buf[0] = 'f';
	else if (S_ISSOCK(mode))
		buf[0] = 's';

	if (mode & S_ISVTX)
		buf[9] = (buf[9] == '-') ? 'T' : 't';
	if (mode & S_ISGID)
		buf[6] = (buf[6] == '-') ? 'S' : 's';
	if (mode & S_ISUID)
		buf[3] = (buf[3] == '-') ? 'S' : 's';
}
/**
 * @brief update m/a/c times of config dir
 *
 * This should be called after writing the config/cache files
 * to let any other instance know that it should re-read those
 * files (if that option is in force)
 *
 * @return
 **/
void e2_fs_touch_config_dir (void)
{
#ifdef E2_FAM
	if (cfg.monitor_type != E2_MONITOR_DEFAULT)
		return;
#endif
	struct stat stat_buf;

	if (e2_fs_utime (e2_cl_options.config_dir, NULL) == 0)
	{
		if (e2_fs_stat (e2_cl_options.config_dir, &stat_buf) == 0)
			cfg.config_mtime = stat_buf.st_mtime;
		else
		{
			e2_output_print_errno();
			printd (WARN, "couldn't stat the config dir");
		}
	}
	else
	{
		e2_output_print_errno();
		printd (WARN, "couldn't touch the config dir");
	}
}
