/* $Id: e2_hook.c 762 2005-05-13 04:00:38Z tpgww $
 * 
 * Copyright (C) 2004-2005 tooar <tooar@gmx.net>
 * 
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/**
 * @file src/utils/e2_hook.c 
 * @brief Hook utility functions
 *
 * This file contains hook utility functions to register hooks in a
 * hook_list and to run the hook list.
 **/

#include "emelfm2.h"
#include "e2_hook.h"
#include "e2_utils.h"

  /*********************/
 /***** callbacks *****/
/*********************/

static gboolean list_run (GHook *hook, gpointer data)
{
//	printd (DEBUG, "list_run (hook:,data:)");
	gboolean (*func) (gchar *, gpointer) = hook->func;
	return func ((gchar *) data, hook->data);
}

void e2_hook_unattach_cb (E2_Duo *duo)
{
	_e2_hook_unregister (duo->b, duo->a);
	g_free (duo);
}

  /******************/
 /***** public *****/
/******************/

GHook *e2_hook_register (GHookList *hook_list, gpointer func, gpointer data)
{
	if (hook_list->is_setup == FALSE)
		g_hook_list_init (hook_list, sizeof (GHook));
	GHook *hook = g_hook_alloc (hook_list);
	hook->data = data;
	hook->func = func;
	g_hook_append (hook_list, hook);
	return hook;
}
gboolean e2_hook_unregister (GHookList *hook_list, gpointer func, gpointer data,
	gboolean use_data)
{
	GHook *hook;
	if (use_data)
		hook = g_hook_find_func_data (hook_list, TRUE, func, data);
	else
		hook = g_hook_find_func (hook_list, TRUE, func);
	return _e2_hook_unregister (hook_list, hook); 
}

gboolean _e2_hook_unregister (GHookList *hook_list, GHook *hook)
{
	if (hook != NULL)
	{
		g_hook_destroy_link (hook_list, hook);
		return TRUE;
	}
	return FALSE;
}

void e2_hook_list_run (GHookList *hook_list, gpointer data)
{
//	printd (DEBUG, "e2_hook_list_run (hook_list:,data:)");

	g_hook_list_marshal_check (hook_list, FALSE,
		list_run, data);
}
