/* $Id: e2_utils.c 1123 2005-09-13 08:38:59Z tpgww $
 * 
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 * Potions copyright (C) 1999 Michael Clark.
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
//#include <unistd.h>
#include "e2_utils.h"
#include "e2_dialog.h"
#include "e2_filelist.h"

static gchar *last_trashpath;

/**
 * @brief setup struct with 3 NULL pointers
 * @return the struct
 **/
E2_Trio *e2_utils_trio_new (void)
{
	E2_Trio *t = g_malloc0 (sizeof (E2_Trio));
	return t;
}
/**
 * @brief setup struct with 6 NULL pointers
 * @return the struct
 **/
E2_Sextet *e2_utils_sextet_new (void)
{
	E2_Sextet *s = g_malloc0 (sizeof (E2_Sextet));
	return s;
}
/**
 * @brief setup struct with 7 NULL pointers
 * @return the struct
 **/
E2_Nontet *e2_utils_nontet_new (void)
{
	E2_Nontet *s = g_malloc0 (sizeof (E2_Nontet));
	return s;
}
/**
 * @brief convert color data to string form
 * @param color Gdk color data struct
 * @return newlay-allocated string with the color data
 **/
gchar *e2_utils_color2str (GdkColor *color)
{
	return g_strdup_printf ("#%.2X%.2X%.2X",
		color->red/256, color->green/256, color->blue/256);
}
/**
 * @brief replace all occurrences of @a old in @a str with @a new
 * This uses strsplit and strjoinv. No utf8 consideration.
 * @param str the 'haystack' string in which to search for @a old
 * @param old the 'needle' string which is to be repaced
 * @param new the replacement string for @a old
 * @return newly allocated string with the repacements made
 **/
gchar *e2_utils_str_replace (gchar *str, gchar *old, gchar *new)
{
	gchar **split = g_strsplit (str, old, -1);
	gchar *join = g_strjoinv (new, split);
	g_strfreev (split);
	return join;
}
/**
 * @brief put spaces between all characters in ascii string @a str
 * This does not handle utf-8
 * @param str the string which is to be expanded
 * @return newly-allocated expanded string
 **/
//FIXME for utf-8
gchar *e2_utils_str_stretch (gchar *str)
{
	gint len = strlen (str);
	gchar *streched = g_malloc ((len * 2) + 2);
	gint j = 0;
	streched[j++] = ' ';
	gint i;
	for (i = 0; i < len; i++)
	{
		streched[j++] = str[i];
		streched[j++] = ' ';
	}
	streched[j] = '\0';
	return streched;
}
/**
 * @brief convert utf8 string @a string to lower case
 *
 * @param string the string which is to be processed
 * @return newly allocated lc string: must be freed
 **/
gchar *e2_utils_str_to_lower (gchar *string)
{
	return g_utf8_strdown (string, -1);
}
/**
 * @brief join strings @a string1 and @a string2
 *
 * This is for localised strings, for which g_strconcat() can't be used
 *
 * @param string1 the string which will start the returned string
 * @param string2 the string which will be appended
 * @return newly allocated joined string, or NULL
 **/
gchar *e2_utils_strcat (gchar *string1, gchar *string2)
{
	gint len1 = strlen (string1);
	gint len2 = strlen (string2);
	gchar *result = g_malloc (len1 + len2 + 1);
	if (result != NULL)
	{
#ifdef __USE_GNU
		gchar *next = mempcpy (result, string1, len1);
		next = mempcpy (next, string2, len2);
		*next = '\0';
#else
		memcpy (result, string1, len1);
		memcpy (result + len1, string2, len2);
		*(result + len1 + len2) = '\0';
#endif
	}
	return result;
}
/*gchar *get_key_name (gint keyval)
{
	return gdk_keyval_name (gdk_keyval_to_lower (keyval));
} */

/**
 * @brief Replace macro(s) in string @a text with appropriate value(s)
 *
 * Supported macro codes are (%)%f, F, d, D, %{...}. These 'letters' are
 * hard-coded CHECKME should the letters be translatable, non-ascii ?
 *
 * @param text utf-8 or ascii string, possibly with macros to be expanded
 * @param for_each NULL to substitute the selected items for "%f", or filename-string to use for "%f"
 * @return newly allocated GString, or NULL if failure, or 1 if prompt macro cancelled
 **/
GString *e2_utils_expand_macros (gchar *text, gchar *for_each)
{
	GString *command_string = g_string_new ("");
	gchar *s, *free_this, *command_copy, *utf;
	F_DECLARE;
	FileInfo *info;

#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "disable refresh, expand macros");
#endif
	e2_filelist_disable_refresh ();  //prevent any change to the selected items ?
	command_copy = g_strdup (text);
	free_this = s = command_copy;
	while ((s = strchr (command_copy, '%')) != NULL)	//if always ascii %, don't need g_utf8_strchr()
	{
		*s = '\0'; s++;	//NCHR(s);
		g_string_append (command_string, command_copy);
		gboolean with_quotes = TRUE;
		command_copy = s; NCHR(command_copy);
		if (*s == '%')
		{
			s++;	//NCHR(s);
			NCHR(command_copy);
			with_quotes = FALSE;
		}
		switch (*s)
		{
			case 'f':
			{
				if (for_each != NULL)
				{
					if (with_quotes == TRUE)
						g_string_append_printf (command_string, "\"%s\"", for_each);
					else
						g_string_append_printf (command_string, "%s", for_each);
				}
				else
				{
					GList *base, *tmp;
					base = tmp = e2_fileview_get_selected_local (curr_view);
					if (tmp == NULL)
					{
						status_message (_("No item selected\n"));
						g_free (free_this);
						g_string_free (command_string, TRUE);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, expand macros");
#endif
						e2_filelist_enable_refresh();
						return NULL;
					}
					else
					{
						for (; tmp != NULL; tmp = tmp->next)
						{
							info = tmp->data;
							utf = F_FILENAME_FROM_LOCALE (info->filename);
							if (with_quotes == TRUE)
								g_string_append_printf (command_string, "\"%s\"%s", utf,
									tmp->next != NULL ? " " : "");
							else
								g_string_append_printf (command_string, "%s%s", utf,
									tmp->next != NULL ? " " : "");
							F_FREE (utf);
						}
					}
					g_list_free (base);
				}
			}
			break;
			case 'F':
			{
				GList *base, *tmp;
				base = tmp = e2_fileview_get_selected_local (other_view);
				if (tmp == NULL)
				{
					status_message (_("No item selected in other pane\n"));
					g_free (free_this);
					g_string_free (command_string, TRUE);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, expand macros 2");
#endif
					e2_filelist_enable_refresh();
					return NULL;
				}
				else
				{
					for (; tmp != NULL; tmp = tmp->next)
					{
						info = tmp->data;
						utf = F_FILENAME_FROM_LOCALE (info->filename);
						if (with_quotes == TRUE)
							g_string_append_printf (command_string, "\"%s%s\"%s", other_view->dir, utf,
									tmp->next != NULL ? " " : "");
						else
							g_string_append_printf (command_string, "%s%s%s", other_view->dir, utf,
									tmp->next != NULL ? " " : "");
						F_FREE (utf);
					}
				}
				g_list_free (base);
			}
			break;
			case 'd':
				if (with_quotes == TRUE)
					g_string_append_printf (command_string, "\"%s\"", curr_view->dir);
				else
					g_string_append_printf (command_string, "%s", curr_view->dir);
				break;
			case 'D':
				if (with_quotes == TRUE)
					g_string_append_printf (command_string, "\"%s\"", other_view->dir);
				else
					g_string_append_printf (command_string, "%s", other_view->dir);
				break;
			case '{':
				if ((s = strchr (command_copy, '}')) == NULL)	//if always ascii }, don't need g_utf8_strchr()
				{
					status_message (_("No matching '}' found in action text.\n"));
					g_free (free_this);
					g_string_free (command_string, TRUE);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, expand macros3");
#endif
					e2_filelist_enable_refresh();
					return NULL;
				}
				else
				{
					gchar *user_input;
					*s = '\0'; s++;	//NCHR(s);
					DialogButtons result = e2_dialog_line_input (NULL, command_copy, "", 0, FALSE, &user_input); 
					command_copy = s;
					if (result == OK)
					{
						g_string_append (command_string, user_input);
						g_free (user_input);
					}
					else
					{
						g_free (free_this);
						g_string_free (command_string, TRUE);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, expand macros4");
#endif
						e2_filelist_enable_refresh();
						return GINT_TO_POINTER (1);	//1 is cancel signal
					}
				}
				break;
			default:
				g_string_append_c (command_string, '%');
				g_string_append_c (command_string, *s);
				break;
		}
	}
	g_string_append (command_string, command_copy);
	g_free (free_this);
#ifdef E2_REFRESH_DEBUG
	printd (DEBUG, "enable refresh, expand macros5");
#endif
	e2_filelist_enable_refresh();
	return command_string;
}
/**
 * @brief ensure path string @a path has appropriate separators
 *
 * Removes superfluous separators from inside @a path,
 * adds separator to end if not one there already.
 * Unless @a path is just "/", the returned string will be newly
 * allocated, and the old one will have been freed.
 *
 * @param path utf8 string, path to be checked
 *
 * @return pointer to cleaned path string (utf8)
 **/
// (what a clunker this is, with utf parsing !!)
gchar *e2_utils_path_clean (gchar *path)
{
	gchar *freeme = path;
	//remove leading and trailing whitespace
	path = g_strstrip (path);

	if (!g_str_equal (path, G_DIR_SEPARATOR_S))
	{ 	//if path is not just the file system root, 
		//remove any superfluous separator from within it
		gunichar c;
		gint path_len = strlen (path);
		gchar clean[path_len + 4];  //space for an extra trailer if needed
		gchar *po = path, *pp = path, *pn = clean;
		while ((pp = strchr (pp, G_DIR_SEPARATOR)) != NULL ) //if always ascii /, don't need g_utf8_strchr()
		{
			//check for a repeated separator, or 0
			pp++;	//NCHR(pp);
			c = g_utf8_get_char (pp);
			if (c == '\0')
				break;
			if (c != G_DIR_SEPARATOR)
			{
				NCHR (pp);
				continue;
			}
			//found at least 2 separators,
			//copy old to clean, with the first separator
			while (po < pp)
				*(pn++) = *(po++);
			//skip any more separators
			while ((c = g_utf8_get_char (pp)) == G_DIR_SEPARATOR)
				NCHR (pp);
			po = pp;  //point to the next start for copying
		}
		//copy rest, to the terminator
		while (*po != '\0')
			*(pn++) = *(po++);
		//check for trailing /
		PCHR (pn);
		c = g_utf8_get_char (pn);
		NCHR (pn);
		if (c != G_DIR_SEPARATOR)
		{	//we need to add a trailer
			//g_strlcpy (pn, G_DIR_SEPARATOR_S, path_len-pn+4);
			//since separator is ascii, just shove it into place
			*pn = G_DIR_SEPARATOR;
			NCHR (pn);
		}
		*(pn) = '\0';
		path = strdup (clean); //get a copy we can keep
		g_free (freeme);	//and cleanup the one supplied
	}
	return path;
}
/**
 * @brief translate relative path
 *
 * Creates an absolute new-path string from a 
 * relative one, or if the new one is already absolute,
 * just makes sure it is 'clean'.
 * Works with UTF (if splitting on G_DIR_SEPARATOR_S is ok)
 * and with non-UTF strings.
 * Returned string is newly allocated, the old one needs
 * to be freed by caller.
 *
 * @param current_dir current-path string (probably UTF) 
 * @param new_dir absolute or relative new-path string (probably UTF)
 *
* @return newly-allocated, cleaned, absolute path
 **/
gchar *e2_utils_translate_relative_path (gchar *current_dir, gchar *new_dir)
{
	//work with a copy, esp so that
	// e2_utils_path_clean doesn't free anything essential
	gchar *retval = g_strdup (new_dir);
	//clean new path, add trailer if need be
	retval = e2_utils_path_clean (retval);
	//do we need to do anything?
	if (g_path_is_absolute (retval))
		return retval;  //no, send back the copy

	//split current path into its 'segments' (plus leading & trailing nulls)
	gchar **split_old = g_strsplit (current_dir, G_DIR_SEPARATOR_S, -1);
	gint split_old_count = 0;
	while (split_old[split_old_count] != NULL) split_old_count++;

	//likewise, split new dir path
	//note: new dir path has trailing /, & may have leading / or ./
	gchar **split = g_strsplit (retval, G_DIR_SEPARATOR_S, -1);
	gint split_count = 0;
	while (split[split_count] != NULL) split_count++;

	//suitable size for the segment ptr  array
	gint join_max_count = split_old_count + split_count + 2;
	//this is used to point to final path segments
	gchar *join[join_max_count];
	gint i, join_count = 0;

	//to get the point of reference for relative cd(s),
	//prepend the current path segments
	for (i = 0; i < split_old_count; i++)
		join[join_count++] = split_old[i];
	//point back to trailer, to suit updir at start of new path
	//also = place to store any / at start of new path
	//CHECKME what if new path is relative, with no leading ./
	join_count--;
	//append the new path segments (including trailer)
	gchar *p;
	for (i = 0; i < split_count; i++)
	{
		if (g_utf8_get_char_validated (split[i], 1) == '.')
		{
			p = split[i];
			NCHR(p);
			if (*p == '\0' && i == 0)
			{ //ignore leading reference to current dir
				continue;
			}
			else if (g_utf8_get_char_validated (p, 1) == '.')
			{
				NCHR(p);
				if (*p == '\0')
				{	//up dir, move back 1 segment if we can
					//ottherwise ignore it
					if ( join_count > 0)
						join_count--;
					continue;
				}
			}
		}
		join[join_count++] = split[i];
	}

	if (join_count == 0)  //we've updir'ed past the root !
		join[join_count++] = "";  //setup for joined path = root
	
	//fill the leftover (empty) element field with NULL
//	no, 1 is enough to terminate the concat
//		for (; join_count < join_max_count; join_count++)
	join[join_count] = NULL;

//	g_free (retval); FIXME = this bombs, regardless of the variable name !!
	//concat the retval
	retval = g_strjoinv (G_DIR_SEPARATOR_S, join);
//	}
	//clean up
	g_strfreev (split);
	g_strfreev (split_old);
	return retval;
}
/**
 * @brief create relative path
 * FIXME this only works for single-byte file/path names
 * The path of @a src relative to @a dest is stored in @a src
 * @param src GString containing the target path
 * @param dest GString containing the link path
 * @return
 **/
void e2_utils_create_relative_path (GString *src, GString *dest)
{
	gchar *sp = src->str, *dp = dest->str, *tsp=NULL;
	GString *rel = g_string_sized_new (PATH_MAX+NAME_MAX);

	/* strip common path */
	while (*sp && (*sp == *dp))
	{
		if (*sp == G_DIR_SEPARATOR)
		{
			NCHR(sp);
			tsp = sp; /* save as latest short path */
		}
		else
			NCHR(sp);
		NCHR(dp);
	}
	/* insert non-common parent dirs (if any) as uplinks */
	while (*dp)
	{
		if (*dp == G_DIR_SEPARATOR)
			g_string_append (rel, ".."G_DIR_SEPARATOR_S);  //.. applies in all languages ?
		NCHR(dp);
	}
	/* if not an updir ref. make it relative to current */
	if (rel->len == 0)
		g_string_append (rel, "."G_DIR_SEPARATOR_S);
	/* and add short src path */
	g_string_append (rel, tsp);

	g_string_assign (src, rel->str);
	g_string_free (rel, TRUE);
}
/**
 * @brief get pointer to last-used trash directory
 * Returned string should NOT be freed elsewhere
 * @return pointer to path-string, or NULL if there is none
 **/
const gchar *e2_utils_last_trash_path (void)
{
	if (g_str_has_suffix (curr_view->dir, "Trash"G_DIR_SEPARATOR_S))
	//already at a trash place, just use that
		return curr_view->dir;
	//check if there is a local trashfolder
	//needs separator at end, for comparisons when simply open a trash dir
	gchar *trash = g_build_filename (curr_view->dir, ".Trash"G_DIR_SEPARATOR_S, NULL);
	F_DECLARE;
	gchar *local = F_FILENAME_TO_LOCALE (trash);

	if (!e2_fs_is_dir3 (local))
	{	//if not, use the default, set at session start
		g_free (trash);
		if (last_trashpath != NULL)
			trash = g_strdup (last_trashpath);  //copy, as it will be freed later
		else if (cfg.trashpath != NULL)
			trash = g_strdup (cfg.trashpath);
		else
		{
			e2_output_print_error(_("No trash directory\n"));
			return NULL;
		}
	}
	//use the local dir instead of the global
	//remember the last-known trash path
	if (last_trashpath != NULL)
		g_free (last_trashpath);
	last_trashpath = trash;
	F_FREE (local);
	return last_trashpath;
}
#ifdef E2_IMAGECACHE
gint iconsizes [GTK_ICON_SIZE_DIALOG+1];
/**
 * @brief determine all icon pixel-sizes
 *
 * assumes width = height
 * @return
 **/
void e2_utils_init_icon_sizes (void)
{
	GtkIconSize size;
/*	0 invalid/theme, 1 menu, 2 toolbar small, 3 toolbar large, 4 button, 5 dnd, 6 dialog
	GTK_ICON_SIZE_INVALID, GTK_ICON_SIZE_MENU, GTK_ICON_SIZE_SMALL_TOOLBAR,
	GTK_ICON_SIZE_LARGE_TOOLBAR, GTK_ICON_SIZE_BUTTON, GTK_ICON_SIZE_DND,
	GTK_ICON_SIZE_DIALOG
*/
	gint wide, high;
	gint defsizes[] = {18, 16, 18, 24, 20, 32, 48};	//default sizes
	GtkSettings* defs = gtk_settings_get_default ();
	gchar *sizestr, *this;
	g_object_get (G_OBJECT (defs), "gtk-icon-sizes", &sizestr, NULL);
	//sizestr = NULL or "gtk-menu=16,16;gtk-button=20,20 ..."
/*	if (sizestr == NULL)
	{ //this is one way to work around strange behaviour of gtk_icon_size_get_name ();
		gchar *names[] = {
			"", "menu", "toolbar-small", "toolbar-large", "button", "dnd", "dialog"
			};	//CHECKME
	} */

	//get the pixel-size corresponding to each GtkIconSize
	for (size = GTK_ICON_SIZE_MENU; size <= GTK_ICON_SIZE_DIALOG; size++)
	{
		if (sizestr == NULL)
		{
			if (gtk_icon_size_lookup_for_settings (defs, size, &wide, &high))
				iconsizes [size] = high;
			else
				iconsizes [size] = defsizes [size];
		}
		else
		{
			gchar *s, *p;
//			this = g_strconcat ("gtk-", names [size], "=", NULL);
			this = (gchar *) gtk_icon_size_get_name (size);
			if (this == NULL)
			{  //could be a malformed theme-descriptor string
				//and for gtk 2.6.7/8 at least - strange behaviour - GTK_ICON_SIZE_MENU
				//doesn't match here, but does match if we call the fn later!
				if (gtk_icon_size_lookup_for_settings (defs, size, &wide, &high))
					iconsizes [size] = high;
				else
					iconsizes [size] = defsizes [size];
			}
			else
			{
				s = strstr (sizestr, this);
				if (s != NULL)
				{	//the theme sets this icon size
					s += strlen (this) + 1;	//skip the descriptor and "="
					p = strchr (s, ',');	//always ascii
					s = g_strndup (s, (p-s));
					iconsizes [size] = atoi (s);
					g_free (s);
				}
				else
				{
					if (gtk_icon_size_lookup_for_settings (defs, size, &wide, &high))
						iconsizes [size] = high;
					else
						iconsizes [size] = defsizes [size];
				}
	//			g_free (this);
			}
		}
	}

	//now set the default
/* gtk 2.6.8 spits warning about gtk-toolbar-icon-size property
	not existing, tho' API doco says it does !
	size = GTK_ICON_SIZE_INVALID;
	g_object_get (G_OBJECT (defs), "gtk-toolbar-icon-size", &size, NULL);
	if (size == GTK_ICON_SIZE_INVALID)
		size = GTK_ICON_SIZE_LARGE_TOOLBAR;
	iconsizes [0] = iconsizes [size]; */
	iconsizes [0] = iconsizes [GTK_ICON_SIZE_LARGE_TOOLBAR];
}
/**
 * @brief get icon pixel size for icon size enumerator @a size
 *
 * @param size GtkIconSize value
 *
 * @return icon pixelsize
 **/
gint e2_utils_get_icon_size (GtkIconSize size)
{
	return iconsizes [size];
}
/**
 * @brief get icon size enumerator for icon which is closest-below @a psize
 *
 * @param psize icon pixel size
 *
 * @return iconsize
 **/
GtkIconSize e2_utils_get_best_icon_size (gint psize)
{
	GtkIconSize i, isz = GTK_ICON_SIZE_MENU;
	gint psz = 0;
	for (i = GTK_ICON_SIZE_MENU; i <= GTK_ICON_SIZE_DIALOG; i++)
	{
		if (iconsizes [i] > psz && iconsizes [i] <= psize)
		{
			isz = i;
			psz = iconsizes [i];
		}
	}

	return isz;
}
#endif //def E2_IMAGECACHE
/**
 * @brief dtermine whether @a name reprresents a gtk stock icon
 *
 * @param name string with icon name, a file path/name or "gtk-*"
 *
 * @return TRUE if @a name represents a stock-icon
 **/
gboolean e2_utils_check_stock_icon (gchar *name)
{
/* gtk_stock_lookup is badly unreliable !!
	GtkStockItem item;
	return (gtk_stock_lookup (rt->icon, &item)) */
	//quick n dirty check
//	return (g_str_has_prefix (name, "gtk-"));
	gboolean retval = FALSE;
	if (g_str_has_prefix (name, "gtk-"))
	{
		GSList *tmp, *ids = gtk_stock_list_ids ();
		for (tmp = ids; tmp != NULL; tmp = g_slist_next (tmp))
		{
			if (!retval && g_str_equal ((gchar *)tmp->data, name))
				retval = TRUE;
			g_free (tmp->data);
		}
		g_slist_free (ids);
	}
	return retval;
}
/**
 * @brief update gtk properties like menu delays
 *
 * update gtk internal properties. these are currently only menu
 * popup and popdown delays. this function is usually called
 * after configuration changes to update the gtk properties.
 *
 * @return 
 **/
void e2_utils_update_gtk_settings (void)
{
	GtkSettings *s = gtk_settings_get_default ();
	gint delay_up = e2_option_int_get ("submenu-up-delay");
	gint delay_down = e2_option_int_get ("submenu-down-delay");
	//be on the safe side
	if (delay_up < 0) delay_up = 0;
	if (delay_down < 0) delay_down = 0;
	gtk_settings_set_long_property (s, "gtk-menu-popup-delay",
		delay_up, "XProperty");
	gtk_settings_set_long_property (s, "gtk-menu-popdown-delay",
		delay_down, "XProperty");
	gtk_settings_set_long_property (s, "gtk-menubar-popup-delay",
		delay_up, "XProperty");
//	gtk_rc_parse_string ("style \"e2_default\" { GtkComboBoxEntry::appears-as-list = 1 } class \"*\" style \"e2_default\"");
}
//no. of single-parenthesis characters found in the replacement string,
//up to the end of the current segment
static gint p_count;
static gchar *dollar = "$";
/**
 * @brief add segment to the replacement string
 *
 * Empty strings are ignored. The index for the next segment
 * is updated.
 * The count of single-parentheses is updated. This uses
 * single-byte ascii scanning.
 *
 * @param string string to record
 * @param join pointer to the array of segments of the replacement string
 * @param join_count pointer to index of the next segment
 *
 * @return
 **/
static void _e2_utils_replace_vars_add (gchar *string, gchar *join[], gint *join_count)
{
	if (*string == '\0')
		return;	//no point in adding empty strings
	join[*join_count] = string;
	(*join_count)++;
	//update count of ' characters sofar detected
	gchar *p = string;
	while (*p != '\0')
	{
		if (*p++ == '\'')
			p_count++;
	}
}
/**
 * @brief add original segment to the replacement string
 *
 * To avoid leaks, this adds 2 segments - one with just the
 * separator "$" and the second with the ignored segment 
 * of the original string.
 * The count of single-parentheses is updated. This uses
 * single-byte ascii scanning.
 *
 * @param string string to record
 * @param join pointer to the array of segments of the replacement string
 * @param join_count pointer to index of the next segment
 *
 * @return
 **/
static void _e2_utils_replace_vars_ignore (gchar *string, gchar *join[], gint *join_count)
{
	join[*join_count] = dollar;
	(*join_count)++;
	_e2_utils_replace_vars_add (string, join, join_count);
}
/**
 * @brief replace variables in a string
 *
 * Replaces all relevant '~' characters, and all valid environment
 * variables and option variables, in @a raw.
 * ~ will be replaced if preceeded by nothing or whitespace,
 * and followed by nothing or whitespace or '/'
 * Environment references must have the form @c \$VAR or
 * @c \${VAR} where @c VAR is the name of the environment
 * variable.
 * Option references must have the form @c \$[VAR] where 
 * @c VAR is the 'internal' name of the option.
 * Unrecognised variables are ignored. Environment variables
 * inside single parentheses are ignored. Strings prefaced by
 * "$$" or "\$" are ignored.
 *
 * @code
 * gchar *str = e2_utils_replace_vars ("my home is \${HOME} and my e2 terminal is \$[command-xterm]");
 * @endcode
 *
 * @param raw ascii string maybe with variable(s) to replace
 *
 * @return newly allocated string, with variables (if any) replaced as appropriate
 **/
gchar *e2_utils_replace_vars (gchar *raw)
{
//	printd (DEBUG, "e2_utils_replace_vars (raw:%s)", raw);
	//replace all relevant occurrences of ~ with ${HOME}
	gchar *s = g_strdup (raw), *p = s, *h = "${HOME}", *freeme;
	while ((p = strchr (p, '~')) != NULL) //if always ascii ~, don't need g_utf8_strchr()
	{
		if (
				(p == s || *(p-1) == ' ' || *(p-1) == '\t' ) //preceeded by nothing or whitespace
			&& (*(p+1) == G_DIR_SEPARATOR //and followed by '/'
					|| *(p+1) == '\0' || *(p+1) == ' ' || *(p+1) == '\t' )	//or lone char in argument
		)
		{
			*p = '\0';
			freeme = s;
			s = g_strconcat (s, h, p+1, NULL);
			p = s + (p - freeme);
			g_free (freeme);
		}
		else
			p++;
	}

	//there may be no $... replacements to handle
	if (strchr (s, '$') == NULL)	//if always ascii $, don't need g_utf8_strchr()
	{
		return s;
	}
	//break into pieces and count them
	gchar **split = g_strsplit (s, "$", -1);
	gint split_count = 0;
	while (split[split_count] != NULL)
		split_count++;
	//init the stack-store for the pieces of the replacement string
	gint join_count = 0;
	gchar *join[split_count * 2 + 1];
	//init the count of ' chars
	p_count = 0;
	gchar *rest;

	_e2_utils_replace_vars_add (split[0], join, &join_count);	//save the segment before the 1st '$'

	gint i;
	//scan from after 1st '$'
	for (i = 1; i < split_count; i++)
	{
		if (*split[i] == '\0'	//2 '$' chars in a row
			|| g_str_has_suffix (split[i-1], "\\")	//escaped '$'
			|| *split[i] == '(')	//shell command
			_e2_utils_replace_vars_ignore (split[i], join, &join_count);

		else if (*split[i] == '[')	//option to replace
		{
			//no protection check for internal vars
			gchar *rest = strchr (split[i], ']'); //if always ascii ], don't need g_utf8_strchr()
			if (rest != NULL)
			{
				*rest++ = '\0';
				E2_OptionSet *set = e2_option_get (split[i]+1);
				if (set != NULL)
				{
					join[join_count++] = _e2_option_str_get (set);
					_e2_utils_replace_vars_add (rest, join, &join_count);
				}
				else
				{
					*(--rest) = ']';
					_e2_utils_replace_vars_ignore (split[i], join, &join_count);
				}
			}
			else
				_e2_utils_replace_vars_ignore (split[i], join, &join_count);
		}

		else if (*split[i] == '{')	//variable to replace
		{
			//make sure separator was outside ' '
			if (p_count%2 == 1)
				_e2_utils_replace_vars_ignore (split[i], join, &join_count);
			else
			{
				rest = strchr (split[i], '}');	//if always ascii }, don't need g_utf8_strchr()
				if (rest != NULL)
				{
					*rest++ = '\0';
					const gchar *env = g_getenv (split[i]+1);
					if (env != NULL)
					{
						join[join_count++] = (gchar *) env;
						_e2_utils_replace_vars_add (rest, join, &join_count);
					}
					else
					{
						*(--rest) = '}';
						_e2_utils_replace_vars_ignore (split[i], join, &join_count);
					}
				}
				else
					_e2_utils_replace_vars_ignore (split[i], join, &join_count);
			}
		}

		else	//something else (maybe a variable)
		{
			//make sure separator was outside ' '
			if (p_count%2 == 1)
				_e2_utils_replace_vars_ignore (split[i], join, &join_count);
			else
			{
				rest = strchr (split[i], ' ');	//always ascii ' ', don't need g_utf8_strchr()
				if (rest == NULL)
					rest = strchr (split[i], '\t');	//ditto
				if (rest != NULL)
					*rest = '\0';
				const gchar *env = g_getenv (split[i]);
				if (rest == NULL)
				{
					if (env != NULL)
						join[join_count++] = (gchar *) env;
					else
						_e2_utils_replace_vars_ignore (split[i], join, &join_count);
				}
				else //rest != NULL
				{
					*rest = ' ';
					if (env != NULL)
					{
						join[join_count++] = (gchar *) env;
						_e2_utils_replace_vars_add (rest, join, &join_count);
					}
					else
						_e2_utils_replace_vars_ignore (split[i], join, &join_count);
				}
			}
		}
	}
	join[join_count] = NULL;
	gchar *retval = g_strjoinv (NULL, join);

	g_strfreev (split);
	return retval;
}
/**
 * @brief get coordinates of @a widget relative to the current screeen
 * @param widget the activated widget whose postion is to be calculated
 * @param x pointer to gint storage for the x coordinate
 * @param y pointer to gint storage for the y coordinate
 * @return
 **/
void e2_utils_get_abs_pos (GtkWidget *widget, gint *x, gint *y)
{
	if (GTK_WIDGET_TOPLEVEL (widget))
		gdk_window_get_position (widget->window, x, y);
	else
	{
		GdkWindow *window;

		if (GTK_WIDGET_NO_WINDOW (widget))
			window = widget->window;
		else
			window = gdk_window_get_parent (widget->window);

		gdk_window_get_origin (window, x, y);

		*x += widget->allocation.x;
		*y += widget->allocation.y;
	}
}
/**
 * @brief check whether more than 1 item is selected
 * @param srclist glist of selected items
 * @return TRUE if more than 1 is selected
 **/
gboolean e2_utils_multi_src (GList *srclist)
{
	gint ctr=0;
	for (; srclist != NULL ; srclist = srclist->next)
	{
		ctr++;
		if (ctr > 1) break;
	}
	return (ctr > 1);
}
